@@ -27,12 +27,13 @@ package filestore
 import (
 	"context"
 	"errors"
-	"io/ioutil"
 	"os"
 	"path"
 	"testing"
 	"time"
 
+	"go.temporal.io/server/tests/testhelper"
+
 	"github.com/golang/mock/gomock"
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
@@ -73,7 +74,7 @@ func TestVisibilityArchiverSuite(t *testing.T) {
 
 func (s *visibilityArchiverSuite) SetupSuite() {
 	var err error
-	s.testQueryDirectory, err = ioutil.TempDir("", "TestQuery")
+	s.testQueryDirectory, err = os.MkdirTemp("", "TestQuery")
 	s.Require().NoError(err)
 	s.setupVisibilityDirectory()
 	s.testArchivalURI, err = archiver.NewURI("file:///a/b/c")
@@ -162,9 +163,7 @@ func (s *visibilityArchiverSuite) TestArchive_Fail_NonRetryableErrorOption() {
 }
 
 func (s *visibilityArchiverSuite) TestArchive_Success() {
-	dir, err := ioutil.TempDir("", "TestVisibilityArchive")
-	s.NoError(err)
-	defer os.RemoveAll(dir)
+	dir := testhelper.MkdirTemp(s.T(), "", "TestVisibilityArchive")
 
 	visibilityArchiver := s.newTestVisibilityArchiver()
 	closeTimestamp := timestamp.TimeNowPtrUtc()
@@ -470,9 +469,7 @@ func (s *visibilityArchiverSuite) TestQuery_Success_SmallPageSize() {
 }
 
 func (s *visibilityArchiverSuite) TestArchiveAndQuery() {
-	dir, err := ioutil.TempDir("", "TestArchiveAndQuery")
-	s.NoError(err)
-	defer os.RemoveAll(dir)
+	dir := testhelper.MkdirTemp(s.T(), "", "TestArchiveAndQuery")
 
 	visibilityArchiver := s.newTestVisibilityArchiver()
 	mockParser := NewMockQueryParser(s.controller)
