@@ -56,6 +56,11 @@ func Error(err error) ZapTag {
 	return NewErrorTag(err)
 }
 
+// IsRetryable returns tag for IsRetryable
+func IsRetryable(isRetryable bool) ZapTag {
+	return NewBoolTag("is-retryable", isRetryable)
+}
+
 // ClusterName returns tag for ClusterName
 func ClusterName(clusterName string) ZapTag {
 	return NewStringTag("cluster-name", clusterName)
@@ -435,6 +440,11 @@ func Counter(c int) ZapTag {
 	return NewInt("counter", c)
 }
 
+// RequestCount returns tag for RequestCount
+func RequestCount(c int) ZapTag {
+	return NewInt("request-count", c)
+}
+
 // Number returns tag for Number
 func Number(n int64) ZapTag {
 	return NewInt64("number", n)
@@ -467,11 +477,6 @@ func ShardID(shardID int32) ZapTag {
 	return NewInt32("shard-id", shardID)
 }
 
-// ShardItem returns tag for ShardItem
-func ShardItem(shardItem interface{}) ZapTag {
-	return NewAnyTag("shard-item", shardItem)
-}
-
 // ShardTime returns tag for ShardTime
 func ShardTime(shardTime interface{}) ZapTag {
 	return NewAnyTag("shard-time", shardTime)
@@ -492,6 +497,16 @@ func ShardRangeID(id int64) ZapTag {
 	return NewInt64("shard-range-id", id)
 }
 
+// ShardContextState returns tag for ShardContextState
+func ShardContextState(state int) ZapTag {
+	return NewInt("shard-context-state", state)
+}
+
+// ShardContextStateRequest returns tag for ShardContextStateRequest
+func ShardContextStateRequest(r string) ZapTag {
+	return NewStringTag("shard-context-state-request", r)
+}
+
 // ReadLevel returns tag for ReadLevel
 func ReadLevel(lv int64) ZapTag {
 	return NewInt64("read-level", lv)
@@ -529,11 +544,6 @@ func TaskID(taskID int64) ZapTag {
 	return NewInt64("queue-task-id", taskID)
 }
 
-// TaskType returns tag for TaskType for queue processor
-func TaskType(taskType enumsspb.TaskType) ZapTag {
-	return NewStringTag("queue-task-type", taskType.String())
-}
-
 // TaskVersion returns tag for TaskVersion
 func TaskVersion(taskVersion int64) ZapTag {
 	return NewInt64("queue-task-version", taskVersion)
