@@ -29,7 +29,8 @@ package metrics
 import (
 	"time"
 
-	"github.com/uber-go/tally"
+	"github.com/uber-go/tally/v4"
+
 	"go.temporal.io/server/common/log"
 )
 
