@@ -54,9 +54,11 @@ type (
 		shardManager              persistence.ShardManager
 		executionManager          persistence.ExecutionManager
 
-		factory Factory
+		factory        Factory
+		faultInjection *FaultInjectionDataStoreFactory
 
 		sync.RWMutex
+		isClosed bool
 	}
 )
 
@@ -125,6 +127,7 @@ func NewBean(
 		namespaceReplicationQueue: namespaceReplicationQueue,
 		shardManager:              shardManager,
 		executionManager:          executionManager,
+		isClosed:                  false,
 	}
 }
 
@@ -156,7 +159,6 @@ func (s *BeanImpl) GetTaskManager() persistence.TaskManager {
 
 // GetNamespaceReplicationQueue get NamespaceReplicationQueue
 func (s *BeanImpl) GetNamespaceReplicationQueue() persistence.NamespaceReplicationQueue {
-
 	s.RLock()
 	defer s.RUnlock()
 
@@ -182,10 +184,13 @@ func (s *BeanImpl) GetExecutionManager() persistence.ExecutionManager {
 
 // Close cleanup connections
 func (s *BeanImpl) Close() {
-
 	s.Lock()
 	defer s.Unlock()
 
+	if s.isClosed {
+		return
+	}
+
 	s.clusterMetadataManager.Close()
 	s.metadataManager.Close()
 	s.taskManager.Close()
@@ -194,4 +199,5 @@ func (s *BeanImpl) Close() {
 	s.executionManager.Close()
 
 	s.factory.Close()
+	s.isClosed = true
 }
