@@ -33,39 +33,24 @@ import (
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	historypb "go.temporal.io/api/history/v1"
-	namespacepb "go.temporal.io/api/namespace/v1"
-	workflowpb "go.temporal.io/api/workflow/v1"
+
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	replicationspb "go.temporal.io/server/api/replication/v1"
 	"go.temporal.io/server/common/codec"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 type (
 	// Serializer is used by persistence to serialize/deserialize objects
 	// It will only be used inside persistence, so that serialize/deserialize is transparent for application
 	Serializer interface {
-		// serialize/deserialize history events
 		SerializeEvents(batch []*historypb.HistoryEvent, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
 		DeserializeEvents(data *commonpb.DataBlob) ([]*historypb.HistoryEvent, error)
 
-		// serialize/deserialize a single history event
 		SerializeEvent(event *historypb.HistoryEvent, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
 		DeserializeEvent(data *commonpb.DataBlob) (*historypb.HistoryEvent, error)
 
-		// serialize/deserialize visibility memo fields
-		SerializeVisibilityMemo(memo *commonpb.Memo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
-		DeserializeVisibilityMemo(data *commonpb.DataBlob) (*commonpb.Memo, error)
-
-		// serialize/deserialize reset points
-		SerializeResetPoints(event *workflowpb.ResetPoints, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
-		DeserializeResetPoints(data *commonpb.DataBlob) (*workflowpb.ResetPoints, error)
-
-		// serialize/deserialize bad binaries
-		SerializeBadBinaries(event *namespacepb.BadBinaries, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
-		DeserializeBadBinaries(data *commonpb.DataBlob) (*namespacepb.BadBinaries, error)
-
-		// serialize/deserialize mutable cluster metadata
 		SerializeClusterMetadata(icm *persistencespb.ClusterMetadata, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
 		DeserializeClusterMetadata(data *commonpb.DataBlob) (*persistencespb.ClusterMetadata, error)
 
@@ -108,18 +93,6 @@ type (
 		TaskQueueInfoToBlob(info *persistencespb.TaskQueueInfo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
 		TaskQueueInfoFromBlob(data *commonpb.DataBlob) (*persistencespb.TaskQueueInfo, error)
 
-		TransferTaskInfoToBlob(info *persistencespb.TransferTaskInfo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
-		TransferTaskInfoFromBlob(data *commonpb.DataBlob) (*persistencespb.TransferTaskInfo, error)
-
-		TimerTaskInfoToBlob(info *persistencespb.TimerTaskInfo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
-		TimerTaskInfoFromBlob(data *commonpb.DataBlob) (*persistencespb.TimerTaskInfo, error)
-
-		ReplicationTaskInfoToBlob(info *persistencespb.ReplicationTaskInfo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
-		ReplicationTaskInfoFromBlob(data *commonpb.DataBlob) (*persistencespb.ReplicationTaskInfo, error)
-
-		VisibilityTaskInfoToBlob(info *persistencespb.VisibilityTaskInfo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
-		VisibilityTaskInfoFromBlob(data *commonpb.DataBlob) (*persistencespb.VisibilityTaskInfo, error)
-
 		ChecksumToBlob(checksum *persistencespb.Checksum, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
 		ChecksumFromBlob(data *commonpb.DataBlob) (*persistencespb.Checksum, error)
 
@@ -128,6 +101,29 @@ type (
 
 		ReplicationTaskToBlob(replicationTask *replicationspb.ReplicationTask, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
 		ReplicationTaskFromBlob(data *commonpb.DataBlob) (*replicationspb.ReplicationTask, error)
+
+		SerializeTransferTasks(taskSlice []tasks.Task) (map[tasks.Key]commonpb.DataBlob, error)
+		DeserializeTransferTasks(blobSlice []commonpb.DataBlob) ([]tasks.Task, error)
+
+		SerializeTimerTasks(taskSlice []tasks.Task) (map[tasks.Key]commonpb.DataBlob, error)
+		DeserializeTimerTasks(blobSlice []commonpb.DataBlob) ([]tasks.Task, error)
+		SerializeVisibilityTasks(taskSlice []tasks.Task) (map[tasks.Key]commonpb.DataBlob, error)
+		DeserializeVisibilityTasks(blobSlice []commonpb.DataBlob) ([]tasks.Task, error)
+		SerializeTieredStorageTasks(taskSlice []tasks.Task) (map[tasks.Key]commonpb.DataBlob, error)
+		DeserializeTieredStorageTasks(blobSlice []commonpb.DataBlob) ([]tasks.Task, error)
+		SerializeReplicationTasks(taskSlice []tasks.Task) (map[tasks.Key]commonpb.DataBlob, error)
+		DeserializeReplicationTasks(blobSlice []commonpb.DataBlob) ([]tasks.Task, error)
+
+		// TODO deprecate method below in favor of methods above (tasks)
+
+		TransferTaskInfoToBlob(info *persistencespb.TransferTaskInfo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
+		TransferTaskInfoFromBlob(data *commonpb.DataBlob) (*persistencespb.TransferTaskInfo, error)
+		TimerTaskInfoToBlob(info *persistencespb.TimerTaskInfo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
+		TimerTaskInfoFromBlob(data *commonpb.DataBlob) (*persistencespb.TimerTaskInfo, error)
+		ReplicationTaskInfoToBlob(info *persistencespb.ReplicationTaskInfo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
+		ReplicationTaskInfoFromBlob(data *commonpb.DataBlob) (*persistencespb.ReplicationTaskInfo, error)
+		VisibilityTaskInfoToBlob(info *persistencespb.VisibilityTaskInfo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error)
+		VisibilityTaskInfoFromBlob(data *commonpb.DataBlob) (*persistencespb.VisibilityTaskInfo, error)
 	}
 
 	// SerializationError is an error type for serialization
@@ -145,7 +141,9 @@ type (
 		encodingType enumspb.EncodingType
 	}
 
-	serializerImpl struct{}
+	serializerImpl struct {
+		TaskSerializer
+	}
 )
 
 // NewSerializer returns a PayloadSerializer
@@ -212,107 +210,6 @@ func (t *serializerImpl) DeserializeEvent(data *commonpb.DataBlob) (*historypb.H
 	return event, err
 }
 
-func (t *serializerImpl) SerializeResetPoints(rp *workflowpb.ResetPoints, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error) {
-	if rp == nil {
-		rp = &workflowpb.ResetPoints{}
-	}
-	return t.serialize(rp, encodingType)
-}
-
-func (t *serializerImpl) DeserializeResetPoints(data *commonpb.DataBlob) (*workflowpb.ResetPoints, error) {
-	if data == nil {
-		return &workflowpb.ResetPoints{}, nil
-	}
-	if len(data.Data) == 0 {
-		return &workflowpb.ResetPoints{}, nil
-	}
-
-	memo := &workflowpb.ResetPoints{}
-	var err error
-	switch data.EncodingType {
-	case enumspb.ENCODING_TYPE_PROTO3:
-		// Thrift == Proto for this object so that we can maintain test behavior until thrift is gone
-		// Client API currently specifies encodingType on requests which span multiple of these objects
-		err = memo.Unmarshal(data.Data)
-	default:
-		return nil, NewDeserializationError("DeserializeResetPoints invalid encoding")
-	}
-
-	if err != nil {
-		return nil, err
-	}
-
-	return memo, err
-}
-
-func (t *serializerImpl) SerializeBadBinaries(bb *namespacepb.BadBinaries, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error) {
-	if bb == nil {
-		bb = &namespacepb.BadBinaries{}
-	}
-	return t.serialize(bb, encodingType)
-}
-
-func (t *serializerImpl) DeserializeBadBinaries(data *commonpb.DataBlob) (*namespacepb.BadBinaries, error) {
-	if data == nil {
-		return &namespacepb.BadBinaries{}, nil
-	}
-	if len(data.Data) == 0 {
-		return &namespacepb.BadBinaries{}, nil
-	}
-
-	memo := &namespacepb.BadBinaries{}
-	var err error
-	switch data.EncodingType {
-	case enumspb.ENCODING_TYPE_PROTO3:
-		// Thrift == Proto for this object so that we can maintain test behavior until thrift is gone
-		// Client API currently specifies encodingType on requests which span multiple of these objects
-		err = memo.Unmarshal(data.Data)
-	default:
-		return nil, NewDeserializationError("DeserializeBadBinaries invalid encoding")
-	}
-
-	if err != nil {
-		return nil, err
-	}
-
-	return memo, err
-}
-
-func (t *serializerImpl) SerializeVisibilityMemo(memo *commonpb.Memo, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error) {
-	if memo == nil {
-		// Return nil here to be consistent with Event
-		// This check is not duplicate as check in following serialize
-		return nil, nil
-	}
-	return t.serialize(memo, encodingType)
-}
-
-func (t *serializerImpl) DeserializeVisibilityMemo(data *commonpb.DataBlob) (*commonpb.Memo, error) {
-	if data == nil {
-		return &commonpb.Memo{}, nil
-	}
-	if len(data.Data) == 0 {
-		return &commonpb.Memo{}, nil
-	}
-
-	memo := &commonpb.Memo{}
-	var err error
-	switch data.EncodingType {
-	case enumspb.ENCODING_TYPE_PROTO3:
-		// Thrift == Proto for this object so that we can maintain test behavior until thrift is gone
-		// Client API currently specifies encodingType on requests which span multiple of these objects
-		err = memo.Unmarshal(data.Data)
-	default:
-		return nil, NewDeserializationError("DeserializeVisibilityMemo invalid encoding")
-	}
-
-	if err != nil {
-		return nil, err
-	}
-
-	return memo, err
-}
-
 func (t *serializerImpl) SerializeClusterMetadata(cm *persistencespb.ClusterMetadata, encodingType enumspb.EncodingType) (*commonpb.DataBlob, error) {
 	if cm == nil {
 		cm = &persistencespb.ClusterMetadata{}
