@@ -193,25 +193,6 @@ func (m *sqlExecutionStore) createWorkflowExecutionTx(
 			return nil, err
 		}
 
-	case p.CreateWorkflowModeContinueAsNew:
-		if currentRow == nil {
-			return nil, extractCurrentWorkflowConflictError(currentRow, "")
-		}
-
-		// currentRow
-
-		if currentRow.RunID.String() != request.PreviousRunID {
-			return nil, extractCurrentWorkflowConflictError(
-				currentRow,
-				fmt.Sprintf(
-					"Workflow execution creation condition failed. workflow ID: %v, current run ID: %v, request run ID: %v",
-					workflowID,
-					currentRow.RunID.String(),
-					request.PreviousRunID,
-				),
-			)
-		}
-
 	default:
 		return nil, serviceerror.NewInternal(fmt.Sprintf("CreteWorkflowExecution: unknown mode: %v", request.Mode))
 	}
@@ -226,8 +207,8 @@ func (m *sqlExecutionStore) createWorkflowExecutionTx(
 		newWorkflow.ExecutionState.State,
 		newWorkflow.ExecutionState.Status,
 		newWorkflow.ExecutionState.CreateRequestId,
-		newWorkflow.StartVersion,
-		lastWriteVersion); err != nil {
+		lastWriteVersion,
+	); err != nil {
 		return nil, err
 	}
 
@@ -248,21 +229,21 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	namespaceID := primitives.MustParseUUID(request.NamespaceID)
-	runID := primitives.MustParseUUID(request.Execution.RunId)
-	wfID := request.Execution.WorkflowId
+	workflowID := request.WorkflowID
+	runID := primitives.MustParseUUID(request.RunID)
 	executionsRow, err := m.Db.SelectFromExecutions(ctx, sqlplugin.ExecutionsFilter{
 		ShardID:     request.ShardID,
 		NamespaceID: namespaceID,
-		WorkflowID:  wfID,
+		WorkflowID:  workflowID,
 		RunID:       runID,
 	})
 	switch err {
 	case nil:
 		// noop
 	case sql.ErrNoRows:
-		return nil, serviceerror.NewNotFound(fmt.Sprintf("Workflow executionsRow not found.  WorkflowId: %v, RunId: %v", request.Execution.GetWorkflowId(), request.Execution.GetRunId()))
+		return nil, serviceerror.NewNotFound(fmt.Sprintf("Workflow executionsRow not found.  WorkflowId: %v, RunId: %v", workflowID, runID))
 	default:
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetWorkflowExecution: failed. Error: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetWorkflowExecution: failed. Error: %v", err))
 	}
 
 	state := &p.InternalWorkflowMutableState{
@@ -277,77 +258,77 @@ func (m *sqlExecutionStore) GetWorkflowExecution(
 		m.Db,
 		request.ShardID,
 		namespaceID,
-		wfID,
+		workflowID,
 		runID,
 	)
 	if err != nil {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetWorkflowExecution: failed to get activity info. Error: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetWorkflowExecution: failed to get activity info. Error: %v", err))
 	}
 
 	state.TimerInfos, err = getTimerInfoMap(ctx,
 		m.Db,
 		request.ShardID,
 		namespaceID,
-		wfID,
+		workflowID,
 		runID,
 	)
 	if err != nil {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetWorkflowExecution: failed to get timer info. Error: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetWorkflowExecution: failed to get timer info. Error: %v", err))
 	}
 
 	state.ChildExecutionInfos, err = getChildExecutionInfoMap(ctx,
 		m.Db,
 		request.ShardID,
 		namespaceID,
-		wfID,
+		workflowID,
 		runID,
 	)
 	if err != nil {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetWorkflowExecution: failed to get child executionsRow info. Error: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetWorkflowExecution: failed to get child executionsRow info. Error: %v", err))
 	}
 
 	state.RequestCancelInfos, err = getRequestCancelInfoMap(ctx,
 		m.Db,
 		request.ShardID,
 		namespaceID,
-		wfID,
+		workflowID,
 		runID,
 	)
 	if err != nil {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetWorkflowExecution: failed to get request cancel info. Error: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetWorkflowExecution: failed to get request cancel info. Error: %v", err))
 	}
 
 	state.SignalInfos, err = getSignalInfoMap(ctx,
 		m.Db,
 		request.ShardID,
 		namespaceID,
-		wfID,
+		workflowID,
 		runID,
 	)
 	if err != nil {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetWorkflowExecution: failed to get signal info. Error: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetWorkflowExecution: failed to get signal info. Error: %v", err))
 	}
 
 	state.BufferedEvents, err = getBufferedEvents(ctx,
 		m.Db,
 		request.ShardID,
 		namespaceID,
-		wfID,
+		workflowID,
 		runID,
 	)
 	if err != nil {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetWorkflowExecution: failed to get buffered events. Error: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetWorkflowExecution: failed to get buffered events. Error: %v", err))
 	}
 
 	state.SignalRequestedIDs, err = getSignalsRequested(ctx,
 		m.Db,
 		request.ShardID,
 		namespaceID,
-		wfID,
+		workflowID,
 		runID,
 	)
 	if err != nil {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetWorkflowExecution: failed to get signals requested. Error: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetWorkflowExecution: failed to get signals requested. Error: %v", err))
 	}
 
 	return &p.InternalGetWorkflowExecutionResponse{
@@ -416,7 +397,7 @@ func (m *sqlExecutionStore) updateWorkflowExecutionTx(
 			newRunID := primitives.MustParseUUID(newWorkflow.ExecutionState.RunId)
 
 			if !bytes.Equal(namespaceID, newNamespaceID) {
-				return serviceerror.NewInternal(fmt.Sprintf("UpdateWorkflowExecution: cannot continue as new to another namespace"))
+				return serviceerror.NewUnavailable(fmt.Sprintf("UpdateWorkflowExecution: cannot continue as new to another namespace"))
 			}
 
 			if err := assertRunIDAndUpdateCurrentExecution(ctx,
@@ -429,10 +410,9 @@ func (m *sqlExecutionStore) updateWorkflowExecutionTx(
 				newWorkflow.ExecutionState.CreateRequestId,
 				newWorkflow.ExecutionState.State,
 				newWorkflow.ExecutionState.Status,
-				newWorkflow.StartVersion,
 				lastWriteVersion,
 			); err != nil {
-				return serviceerror.NewInternal(fmt.Sprintf("UpdateWorkflowExecution: failed to continue as new current execution. Error: %v", err))
+				return err
 			}
 		} else {
 			lastWriteVersion := updateWorkflow.LastWriteVersion
@@ -447,15 +427,14 @@ func (m *sqlExecutionStore) updateWorkflowExecutionTx(
 				updateWorkflow.ExecutionState.CreateRequestId,
 				updateWorkflow.ExecutionState.State,
 				updateWorkflow.ExecutionState.Status,
-				updateWorkflow.StartVersion,
 				lastWriteVersion,
 			); err != nil {
-				return serviceerror.NewInternal(fmt.Sprintf("UpdateWorkflowExecution: failed to update current execution. Error: %v", err))
+				return err
 			}
 		}
 
 	default:
-		return serviceerror.NewInternal(fmt.Sprintf("UpdateWorkflowExecution: unknown mode: %v", request.Mode))
+		return serviceerror.NewUnavailable(fmt.Sprintf("UpdateWorkflowExecution: unknown mode: %v", request.Mode))
 	}
 
 	if err := applyWorkflowMutationTx(ctx,
@@ -481,6 +460,23 @@ func (m *sqlExecutionStore) updateWorkflowExecutionTx(
 func (m *sqlExecutionStore) ConflictResolveWorkflowExecution(
 	request *p.InternalConflictResolveWorkflowExecutionRequest,
 ) error {
+	// first append history
+	for _, req := range request.CurrentWorkflowEventsNewEvents {
+		if err := m.AppendHistoryNodes(req); err != nil {
+			return err
+		}
+	}
+	for _, req := range request.ResetWorkflowEventsNewEvents {
+		if err := m.AppendHistoryNodes(req); err != nil {
+			return err
+		}
+	}
+	for _, req := range request.NewWorkflowEventsNewEvents {
+		if err := m.AppendHistoryNodes(req); err != nil {
+			return err
+		}
+	}
+
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	return m.txExecuteShardLocked(ctx,
@@ -522,11 +518,9 @@ func (m *sqlExecutionStore) conflictResolveWorkflowExecutionTx(
 	case p.ConflictResolveWorkflowModeUpdateCurrent:
 		executionState := resetWorkflow.ExecutionState
 		lastWriteVersion := resetWorkflow.LastWriteVersion
-		startVersion := resetWorkflow.StartVersion
 		if newWorkflow != nil {
 			executionState = newWorkflow.ExecutionState
 			lastWriteVersion = newWorkflow.LastWriteVersion
-			startVersion = newWorkflow.StartVersion
 		}
 		runID := primitives.MustParseUUID(executionState.RunId)
 		createRequestID := executionState.CreateRequestId
@@ -546,10 +540,9 @@ func (m *sqlExecutionStore) conflictResolveWorkflowExecutionTx(
 				createRequestID,
 				state,
 				status,
-				startVersion,
 				lastWriteVersion,
 			); err != nil {
-				return serviceerror.NewInternal(fmt.Sprintf("ConflictResolveWorkflowExecution. Failed to comare and swap the current record. Error: %v", err))
+				return err
 			}
 		} else {
 			// reset workflow is current
@@ -565,15 +558,14 @@ func (m *sqlExecutionStore) conflictResolveWorkflowExecutionTx(
 				createRequestID,
 				state,
 				status,
-				startVersion,
 				lastWriteVersion,
 			); err != nil {
-				return serviceerror.NewInternal(fmt.Sprintf("ConflictResolveWorkflowExecution. Failed to comare and swap the current record. Error: %v", err))
+				return err
 			}
 		}
 
 	default:
-		return serviceerror.NewInternal(fmt.Sprintf("ConflictResolveWorkflowExecution: unknown mode: %v", request.Mode))
+		return serviceerror.NewUnavailable(fmt.Sprintf("ConflictResolveWorkflowExecution: unknown mode: %v", request.Mode))
 	}
 
 	if err := applyWorkflowSnapshotTxAsReset(ctx,
@@ -656,7 +648,7 @@ func (m *sqlExecutionStore) GetCurrentExecution(
 		if err == sql.ErrNoRows {
 			return nil, serviceerror.NewNotFound(err.Error())
 		}
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetCurrentExecution operation failed. Error: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetCurrentExecution operation failed. Error: %v", err))
 	}
 
 	return &p.InternalGetCurrentExecutionResponse{
@@ -666,7 +658,6 @@ func (m *sqlExecutionStore) GetCurrentExecution(
 			State:           row.State,
 			Status:          row.Status,
 		},
-		LastWriteVersion: row.LastWriteVersion,
 	}, nil
 }
 
