@@ -25,22 +25,19 @@
 package resource
 
 import (
-	"github.com/uber-go/tally"
-	sdkclient "go.temporal.io/sdk/client"
+	"github.com/uber-go/tally/v4"
+
 	"go.temporal.io/server/client"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/archiver"
 	"go.temporal.io/server/common/archiver/provider"
-	"go.temporal.io/server/common/authorization"
+	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/config"
-	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/membership"
 	"go.temporal.io/server/common/metrics"
 	persistenceClient "go.temporal.io/server/common/persistence/client"
-	esclient "go.temporal.io/server/common/persistence/visibility/elasticsearch/client"
-	"go.temporal.io/server/common/resolver"
-	"go.temporal.io/server/common/searchattribute"
+	"go.temporal.io/server/common/sdk"
 )
 
 type (
@@ -49,7 +46,6 @@ type (
 	BootstrapParams struct {
 		Name            string
 		InstanceID      string
-		Logger          log.Logger
 		ThrottledLogger log.Logger
 		NamespaceLogger log.Logger
 
@@ -57,25 +53,16 @@ type (
 		MembershipFactoryInitializer MembershipFactoryInitializerFunc
 		RPCFactory                   common.RPCFactory
 		ClientFactoryProvider        client.FactoryProvider
-		AbstractDatastoreFactory     persistenceClient.AbstractDataStoreFactory
 		PersistenceConfig            config.Persistence
-		ClusterMetadataConfig        *config.ClusterMetadata
+		ClusterMetadataConfig        *cluster.Config
 		ReplicatorConfig             config.Replicator
 		ServerMetricsReporter        metrics.Reporter
 		SDKMetricsReporter           metrics.Reporter
 		MetricsClient                metrics.Client
-		ESClient                     esclient.Client
-		ESConfig                     *config.Elasticsearch
-		DynamicConfigClient          dynamicconfig.Client
 		DCRedirectionPolicy          config.DCRedirectionPolicy
-		SdkClient                    sdkclient.Client
+		SdkClientFactory             sdk.ClientFactory
 		ArchivalMetadata             archiver.ArchivalMetadata
 		ArchiverProvider             provider.ArchiverProvider
-		Authorizer                   authorization.Authorizer
-		ClaimMapper                  authorization.ClaimMapper
-		PersistenceServiceResolver   resolver.ServiceResolver
-		AudienceGetter               authorization.JWTAudienceMapper
-		SearchAttributesMapper       searchattribute.Mapper
 	}
 
 	// MembershipMonitorFactory provides a bootstrapped membership monitor
