@@ -28,11 +28,10 @@ import (
 	"net"
 
 	"github.com/golang/mock/gomock"
-	"github.com/uber-go/tally"
+	"github.com/uber-go/tally/v4"
 	"go.temporal.io/api/workflowservice/v1"
 	"go.temporal.io/api/workflowservicemock/v1"
-	sdkclient "go.temporal.io/sdk/client"
-	sdkmocks "go.temporal.io/sdk/mocks"
+
 	"go.temporal.io/server/api/adminservice/v1"
 	"go.temporal.io/server/api/adminservicemock/v1"
 	"go.temporal.io/server/api/historyservice/v1"
@@ -43,17 +42,17 @@ import (
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/archiver"
 	"go.temporal.io/server/common/archiver/provider"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/membership"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	persistenceClient "go.temporal.io/server/common/persistence/client"
 	"go.temporal.io/server/common/persistence/serialization"
-	"go.temporal.io/server/common/persistence/visibility"
-	esclient "go.temporal.io/server/common/persistence/visibility/elasticsearch/client"
+	esclient "go.temporal.io/server/common/persistence/visibility/store/elasticsearch/client"
+	"go.temporal.io/server/common/sdk"
 	"go.temporal.io/server/common/searchattribute"
 )
 
@@ -68,7 +67,7 @@ type (
 
 		// other common resources
 
-		NamespaceCache    *cache.MockNamespaceCache
+		NamespaceCache    *namespace.MockRegistry
 		TimeSource        clock.TimeSource
 		PayloadSerializer serialization.Serializer
 		MetricsClient     metrics.Client
@@ -85,13 +84,14 @@ type (
 
 		// internal services clients
 
-		SDKClient            *sdkmocks.Client
+		SDKClientFactory     *sdk.MockClientFactory
 		FrontendClient       *workflowservicemock.MockWorkflowServiceClient
 		MatchingClient       *matchingservicemock.MockMatchingServiceClient
 		HistoryClient        *historyservicemock.MockHistoryServiceClient
 		RemoteAdminClient    *adminservicemock.MockAdminServiceClient
 		RemoteFrontendClient *workflowservicemock.MockWorkflowServiceClient
 		ClientBean           *client.MockBean
+		ClientFactory        *client.MockFactory
 		ESClient             *esclient.MockClient
 
 		// persistence clients
@@ -99,7 +99,6 @@ type (
 		MetadataMgr               *persistence.MockMetadataManager
 		ClusterMetadataMgr        *persistence.MockClusterMetadataManager
 		TaskMgr                   *persistence.MockTaskManager
-		VisibilityMgr             *visibility.MockVisibilityManager
 		NamespaceReplicationQueue persistence.NamespaceReplicationQueue
 		ShardMgr                  *persistence.MockShardManager
 		ExecutionMgr              *persistence.MockExecutionManager
@@ -109,8 +108,6 @@ type (
 	}
 )
 
-var _ Resource = (*Test)(nil)
-
 const (
 	testHostName = "test_host"
 )
@@ -139,10 +136,10 @@ func NewTest(
 	clientBean.EXPECT().GetHistoryClient().Return(historyClient).AnyTimes()
 	clientBean.EXPECT().GetRemoteAdminClient(gomock.Any()).Return(remoteAdminClient).AnyTimes()
 	clientBean.EXPECT().GetRemoteFrontendClient(gomock.Any()).Return(remoteFrontendClient).AnyTimes()
+	clientFactory := client.NewMockFactory(controller)
 
 	metadataMgr := persistence.NewMockMetadataManager(controller)
 	taskMgr := persistence.NewMockTaskManager(controller)
-	visibilityMgr := visibility.NewMockVisibilityManager(controller)
 	shardMgr := persistence.NewMockShardManager(controller)
 	executionMgr := persistence.NewMockExecutionManager(controller)
 	namespaceReplicationQueue := persistence.NewMockNamespaceReplicationQueue(controller)
@@ -177,10 +174,10 @@ func NewTest(
 
 		// other common resources
 
-		NamespaceCache:    cache.NewMockNamespaceCache(controller),
+		NamespaceCache:    namespace.NewMockRegistry(controller),
 		TimeSource:        clock.NewRealTimeSource(),
 		PayloadSerializer: serialization.NewSerializer(),
-		MetricsClient:     metrics.NewClient(scope, serviceMetricsIndex),
+		MetricsClient:     metrics.NewClient(&metrics.ClientConfig{}, scope, serviceMetricsIndex),
 		ArchivalMetadata:  archiver.NewMockArchivalMetadata(controller),
 		ArchiverProvider:  provider.NewMockArchiverProvider(controller),
 
@@ -194,13 +191,14 @@ func NewTest(
 
 		// internal services clients
 
-		SDKClient:            &sdkmocks.Client{},
+		SDKClientFactory:     sdk.NewMockClientFactory(controller),
 		FrontendClient:       frontendClient,
 		MatchingClient:       matchingClient,
 		HistoryClient:        historyClient,
 		RemoteAdminClient:    remoteAdminClient,
 		RemoteFrontendClient: remoteFrontendClient,
 		ClientBean:           clientBean,
+		ClientFactory:        clientFactory,
 		ESClient:             esclient.NewMockClient(controller),
 
 		// persistence clients
@@ -208,7 +206,6 @@ func NewTest(
 		MetadataMgr:               metadataMgr,
 		ClusterMetadataMgr:        clusterMetadataManager,
 		TaskMgr:                   taskMgr,
-		VisibilityMgr:             visibilityMgr,
 		NamespaceReplicationQueue: namespaceReplicationQueue,
 		ShardMgr:                  shardMgr,
 		ExecutionMgr:              executionMgr,
@@ -259,8 +256,8 @@ func (s *Test) GetClusterMetadataManager() persistence.ClusterMetadataManager {
 
 // other common resources
 
-// GetNamespaceCache for testing
-func (s *Test) GetNamespaceCache() cache.NamespaceCache {
+// GetNamespaceRegistry for testing
+func (s *Test) GetNamespaceRegistry() namespace.Registry {
 	return s.NamespaceCache
 }
 
@@ -318,14 +315,9 @@ func (s *Test) GetWorkerServiceResolver() membership.ServiceResolver {
 
 // internal services clients
 
-// GetSDKClient for testing
-func (s *Test) GetSDKClient() sdkclient.Client {
-	return s.SDKClient
-}
-
-// GetFrontendRawClient for testing
-func (s *Test) GetFrontendRawClient() workflowservice.WorkflowServiceClient {
-	return s.FrontendClient
+// GetSDKClientFactory for testing
+func (s *Test) GetSDKClientFactory() sdk.ClientFactory {
+	return s.SDKClientFactory
 }
 
 // GetFrontendClient for testing
@@ -374,6 +366,11 @@ func (s *Test) GetClientBean() client.Bean {
 	return s.ClientBean
 }
 
+// GetClientFactory for testing
+func (s *Test) GetClientFactory() client.Factory {
+	return s.ClientFactory
+}
+
 // persistence clients
 
 // GetMetadataManager for testing
@@ -386,11 +383,6 @@ func (s *Test) GetTaskManager() persistence.TaskManager {
 	return s.TaskMgr
 }
 
-// GetVisibilityManager for testing
-func (s *Test) GetVisibilityManager() visibility.VisibilityManager {
-	return s.VisibilityMgr
-}
-
 // GetNamespaceReplicationQueue for testing
 func (s *Test) GetNamespaceReplicationQueue() persistence.NamespaceReplicationQueue {
 	// user should implement this method for test
@@ -440,3 +432,11 @@ func (h *Test) GetSearchAttributesManager() searchattribute.Manager {
 func (h *Test) GetSearchAttributesMapper() searchattribute.Mapper {
 	return h.SearchAttributesMapper
 }
+
+func (h *Test) RefreshNamespaceCache() {
+	h.NamespaceCache.Refresh()
+}
+
+func (h *Test) GetFaultInjection() *persistenceClient.FaultInjectionDataStoreFactory {
+	return nil
+}
