@@ -178,16 +178,7 @@ func (s *integrationSuite) TestCronWorkflow_Failed() {
 					}}, nil
 			}
 
-			startedAttributes := startedEvent.GetWorkflowExecutionStartedEventAttributes()
-			if err := payloads.Decode(startedAttributes.GetLastCompletionResult(), &counter); err != nil {
-				return []*commandpb.Command{
-					{
-						CommandType: enumspb.COMMAND_TYPE_FAIL_WORKFLOW_EXECUTION,
-						Attributes: &commandpb.Command_FailWorkflowExecutionCommandAttributes{FailWorkflowExecutionCommandAttributes: &commandpb.FailWorkflowExecutionCommandAttributes{
-							Failure: failure.NewServerFailure("cannot decode last completion result", true),
-						}},
-					}}, nil
-			}
+			counter = s.decodePayloadsInt(startedEvent.GetWorkflowExecutionStartedEventAttributes().GetLastCompletionResult())
 		}
 
 		executions = append(executions, execution)
@@ -256,7 +247,7 @@ func (s *integrationSuite) TestCronWorkflow_Failed() {
 	s.Logger.Info("Process first cron run which fails")
 	_, err = poller.PollAndProcessWorkflowTask(true, false)
 	s.Logger.Info("First cron run processed")
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	// Make sure the cron workflow start running at a proper time, in this case 3 seconds after the
 	// startWorkflowExecution request
@@ -267,27 +258,27 @@ func (s *integrationSuite) TestCronWorkflow_Failed() {
 	s.Logger.Info("Process second cron run which succeeds")
 	_, err = poller.PollAndProcessWorkflowTask(false, false)
 	s.Logger.Info("Second cron run processed")
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	s.Logger.Info("Process third cron run which fails")
 	_, err = poller.PollAndProcessWorkflowTask(false, false)
 	s.Logger.Info("Third cron run processed")
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	s.Logger.Info("Process fourth cron run which succeeds")
 	_, err = poller.PollAndProcessWorkflowTask(false, false)
 	s.Logger.Info("Fourth cron run processed")
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	s.Logger.Info("Process fifth cron run which fails")
 	_, err = poller.PollAndProcessWorkflowTask(false, false)
 	s.Logger.Info("Fifth cron run processed")
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	s.Logger.Info("Process sixth cron run which succeeds")
 	_, err = poller.PollAndProcessWorkflowTask(false, false)
 	s.Logger.Info("Sixth cron run processed")
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	s.Logger.Info("Terminate cron workflow")
 	_, terminateErr := s.engine.TerminateWorkflowExecution(NewContext(), &workflowservice.TerminateWorkflowExecutionRequest{
@@ -332,43 +323,24 @@ func (s *integrationSuite) TestCronWorkflow_Failed() {
 	s.Equal(enumspb.WORKFLOW_EXECUTION_STATUS_TERMINATED, lastExecution.GetStatus())
 
 	finalCronExecution := closedExecutions[1]
-	s.Equal(enumspb.WORKFLOW_EXECUTION_STATUS_CONTINUED_AS_NEW, finalCronExecution.GetStatus())
+	s.Equal(enumspb.WORKFLOW_EXECUTION_STATUS_COMPLETED, finalCronExecution.GetStatus())
 	events := s.getHistory(s.namespace, finalCronExecution.GetExecution())
 	lastEvent := events[len(events)-1]
-	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW, lastEvent.GetEventType())
-	attributes := lastEvent.GetWorkflowExecutionContinuedAsNewEventAttributes()
-	s.Equal(enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE, attributes.GetInitiator())
-	var r int
-	err = payloads.Decode(attributes.GetLastCompletionResult(), &r)
-	s.NoError(err)
-	s.Equal(3, r)
+	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_COMPLETED, lastEvent.GetEventType())
+	attrs1 := lastEvent.GetWorkflowExecutionCompletedEventAttributes()
+	s.Equal(3, s.decodePayloadsInt(attrs1.GetResult()))
 
 	lastFailedExecution := closedExecutions[2]
-	s.Equal(enumspb.WORKFLOW_EXECUTION_STATUS_CONTINUED_AS_NEW, lastFailedExecution.GetStatus())
+	s.Equal(enumspb.WORKFLOW_EXECUTION_STATUS_FAILED, lastFailedExecution.GetStatus())
 	// check when workflow timeout, continueAsNew event contains expected fields
 	events = s.getHistory(s.namespace, lastFailedExecution.GetExecution())
 	lastEvent = events[len(events)-1]
-	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW, lastEvent.GetEventType())
-	attributes = lastEvent.GetWorkflowExecutionContinuedAsNewEventAttributes()
-	s.Equal(enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE, attributes.GetInitiator())
-	s.Equal("cron-test-error", attributes.GetFailure().GetMessage())
-	s.NotNil(attributes.GetLastCompletionResult())
-	err = payloads.Decode(attributes.GetLastCompletionResult(), &r)
-	s.NoError(err)
-	s.Equal(2, r)
+	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_FAILED, lastEvent.GetEventType())
+	attrs2 := lastEvent.GetWorkflowExecutionFailedEventAttributes()
+	s.Equal("cron-test-error", attrs2.GetFailure().GetMessage())
 }
 
 func (s *integrationSuite) TestCronWorkflow() {
-	s.T().Skip(`
-	    integration_test.go:1034:
-	        	Error Trace:	integration_test.go:1034
-	        	Error:      	Not equal:
-	        	            	expected: 0
-	        	            	actual  : 2
-	        	Test:       	TestIntegrationSuite/TestCronWorkflow
-	        	Messages:   	exected backof 2-0 should be multiplier of target backoff 3
-	`)
-
 	id := "integration-wf-cron-test"
 	wt := "integration-wf-cron-type"
 	tl := "integration-wf-cron-taskqueue"
@@ -387,6 +359,13 @@ func (s *integrationSuite) TestCronWorkflow() {
 		},
 	}
 
+	// can't do simply s.Equal because "type" is added
+	checkSearchAttrs := func(sa *commonpb.SearchAttributes) {
+		field := sa.IndexedFields["CustomKeywordField"]
+		s.Equal(searchAttr.IndexedFields["CustomKeywordField"].Data, field.Data)
+		s.Equal([]byte("Keyword"), field.Metadata["type"])
+	}
+
 	request := &workflowservice.StartWorkflowExecutionRequest{
 		RequestId:           uuid.New(),
 		Namespace:           s.namespace,
@@ -402,6 +381,14 @@ func (s *integrationSuite) TestCronWorkflow() {
 		SearchAttributes:    searchAttr,
 	}
 
+	// Because of rounding in GetBackoffForNextSchedule, we'll tend to stay aligned to whatever
+	// phase we start in relative to second boundaries, but drift slightly later within the second
+	// over time. If we cross a second boundary, one of our intervals will end up being 2s instead
+	// of 3s. To avoid this, wait until we can start early in the second.
+	for time.Now().Nanosecond()/int(time.Millisecond) > 150 {
+		time.Sleep(50 * time.Millisecond)
+	}
+
 	startWorkflowTS := time.Now().UTC()
 	we, err0 := s.engine.StartWorkflowExecution(NewContext(), request)
 	s.NoError(err0)
@@ -410,11 +397,8 @@ func (s *integrationSuite) TestCronWorkflow() {
 
 	var executions []*commonpb.WorkflowExecution
 
-	attemptCount := 1
-
 	wtHandler := func(execution *commonpb.WorkflowExecution, wt *commonpb.WorkflowType,
 		previousStartedEventID, startedEventID int64, history *historypb.History) ([]*commandpb.Command, error) {
-		counter := 0
 		if previousStartedEventID == common.EmptyEventID {
 			startedEvent := history.Events[0]
 			if startedEvent.GetEventType() != enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_STARTED {
@@ -427,21 +411,12 @@ func (s *integrationSuite) TestCronWorkflow() {
 					}}, nil
 			}
 
-			startedAttributes := startedEvent.GetWorkflowExecutionStartedEventAttributes()
-			if err := payloads.Decode(startedAttributes.GetLastCompletionResult(), &counter); err != nil {
-				return []*commandpb.Command{
-					{
-						CommandType: enumspb.COMMAND_TYPE_FAIL_WORKFLOW_EXECUTION,
-						Attributes: &commandpb.Command_FailWorkflowExecutionCommandAttributes{FailWorkflowExecutionCommandAttributes: &commandpb.FailWorkflowExecutionCommandAttributes{
-							Failure: failure.NewServerFailure("cannot decode last completion result", true),
-						}},
-					}}, nil
-			}
+			// Just check that it can be decoded
+			s.decodePayloadsInt(startedEvent.GetWorkflowExecutionStartedEventAttributes().GetLastCompletionResult())
 		}
 
 		executions = append(executions, execution)
-		attemptCount++
-		if attemptCount == 3 {
+		if len(executions) >= 3 {
 			return []*commandpb.Command{
 				{
 					CommandType: enumspb.COMMAND_TYPE_COMPLETE_WORKFLOW_EXECUTION,
@@ -490,7 +465,7 @@ func (s *integrationSuite) TestCronWorkflow() {
 	s.Equal(targetBackoffDuration, executionInfo.GetExecutionTime().Sub(timestamp.TimeValue(executionInfo.GetStartTime())))
 
 	_, err = poller.PollAndProcessWorkflowTask(false, false)
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	// Make sure the cron workflow start running at a proper time, in this case 3 seconds after the
 	// startWorkflowExecution request
@@ -499,12 +474,12 @@ func (s *integrationSuite) TestCronWorkflow() {
 	s.True(backoffDuration < targetBackoffDuration+backoffDurationTolerance)
 
 	_, err = poller.PollAndProcessWorkflowTask(false, false)
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	_, err = poller.PollAndProcessWorkflowTask(false, false)
-	s.True(err == nil, err)
+	s.NoError(err)
 
-	s.Equal(4, attemptCount)
+	s.Equal(3, len(executions))
 
 	_, terminateErr := s.engine.TerminateWorkflowExecution(NewContext(), &workflowservice.TerminateWorkflowExecutionRequest{
 		Namespace: s.namespace,
@@ -513,42 +488,34 @@ func (s *integrationSuite) TestCronWorkflow() {
 		},
 	})
 	s.NoError(terminateErr)
-	events := s.getHistory(s.namespace, executions[0])
+
+	// first two should be failures
+	for i := 0; i < 2; i++ {
+		events := s.getHistory(s.namespace, executions[i])
+
+		startAttrs := events[0].GetWorkflowExecutionStartedEventAttributes()
+		s.Equal(memo, startAttrs.Memo)
+		checkSearchAttrs(startAttrs.SearchAttributes)
+
+		lastEvent := events[len(events)-1]
+		s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_FAILED, lastEvent.GetEventType())
+		failAttrs := lastEvent.GetWorkflowExecutionFailedEventAttributes()
+		s.Equal("cron-test-error", failAttrs.GetFailure().GetMessage())
+		s.Equal(executions[i+1].RunId, failAttrs.GetNewExecutionRunId())
+	}
+
+	// third should be completed
+	events := s.getHistory(s.namespace, executions[2])
+
+	startAttrs := events[0].GetWorkflowExecutionStartedEventAttributes()
+	s.Equal(memo, startAttrs.Memo)
+	checkSearchAttrs(startAttrs.SearchAttributes)
+
 	lastEvent := events[len(events)-1]
-	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW, lastEvent.GetEventType())
-	attributes := lastEvent.GetWorkflowExecutionContinuedAsNewEventAttributes()
-	s.Equal(enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE, attributes.GetInitiator())
-	s.Equal("cron-test-error", attributes.GetFailure().GetMessage())
-	s.Nil(attributes.GetLastCompletionResult())
-	s.Equal(memo, attributes.Memo)
-	s.Equal(searchAttr, attributes.SearchAttributes)
+	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_COMPLETED, lastEvent.GetEventType())
 
-	events = s.getHistory(s.namespace, executions[1])
-	lastEvent = events[len(events)-1]
-	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW, lastEvent.GetEventType())
-	attributes = lastEvent.GetWorkflowExecutionContinuedAsNewEventAttributes()
-	s.Equal(enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE, attributes.GetInitiator())
-	s.Equal("", attributes.GetFailure().GetMessage())
-
-	var r string
-	err = payloads.Decode(attributes.GetLastCompletionResult(), &r)
-	s.NoError(err)
-	s.Equal("cron-test-result", r)
-	s.Equal(memo, attributes.Memo)
-	s.Equal(searchAttr, attributes.SearchAttributes)
-
-	events = s.getHistory(s.namespace, executions[2])
-	lastEvent = events[len(events)-1]
-	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW, lastEvent.GetEventType())
-	attributes = lastEvent.GetWorkflowExecutionContinuedAsNewEventAttributes()
-	s.Equal(enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE, attributes.GetInitiator())
-	s.Equal("cron-test-error", attributes.GetFailure().GetMessage())
-
-	err = payloads.Decode(attributes.GetLastCompletionResult(), &r)
-	s.NoError(err)
-	s.Equal("cron-test-result", r)
-	s.Equal(memo, attributes.Memo)
-	s.Equal(searchAttr, attributes.SearchAttributes)
+	completedAttrs := lastEvent.GetWorkflowExecutionCompletedEventAttributes()
+	s.Equal("cron-test-result", s.decodePayloadsString(completedAttrs.Result))
 
 	startFilter.LatestTime = timestamp.TimePtr(time.Now().UTC())
 	var closedExecutions []*workflowpb.WorkflowExecutionInfo
@@ -584,19 +551,19 @@ func (s *integrationSuite) TestCronWorkflow() {
 		return closedExecutions[i].GetStartTime().Before(timestamp.TimeValue(closedExecutions[j].GetStartTime()))
 	})
 	lastExecution := closedExecutions[0]
-	for i := 1; i != 4; i++ {
+	for i := 1; i < 4; i++ {
 		executionInfo := closedExecutions[i]
 		expectedBackoff := executionInfo.GetExecutionTime().Sub(timestamp.TimeValue(lastExecution.GetExecutionTime()))
-		// The execution time calculate based on last execution close time
-		// However, the current execution time is based on the current start time
-		// This code is to remove the diff between current start time and last execution close time
+		// The execution time calculated based on last execution close time.
+		// However, the current execution time is based on the current start time.
+		// This code is to remove the diff between current start time and last execution close time.
 		// TODO: Remove this line once we unify the time source
 		executionTimeDiff := executionInfo.GetStartTime().Sub(timestamp.TimeValue(lastExecution.GetCloseTime()))
-		// The backoff between any two executions should be multiplier of the target backoff duration which is 3 in this test
+		// The backoff between any two executions should be a multiplier of the target backoff duration which is 3 in this test
 		s.Equal(
 			0,
 			int((expectedBackoff-executionTimeDiff).Round(time.Second).Seconds())%int(targetBackoffDuration.Seconds()),
-			"exected backoff %v-%v=%v should be multiplier of target backoff %v",
+			"expected backoff %v-%v=%v should be multiplier of target backoff %v",
 			expectedBackoff.Seconds(),
 			executionTimeDiff.Seconds(),
 			(expectedBackoff - executionTimeDiff).Round(time.Second).Seconds(),
@@ -655,16 +622,7 @@ func (s *integrationSuite) TestCronWorkflow_Success() {
 					}}, nil
 			}
 
-			startedAttributes := startedEvent.GetWorkflowExecutionStartedEventAttributes()
-			if err := payloads.Decode(startedAttributes.GetLastCompletionResult(), &counter); err != nil {
-				return []*commandpb.Command{
-					{
-						CommandType: enumspb.COMMAND_TYPE_FAIL_WORKFLOW_EXECUTION,
-						Attributes: &commandpb.Command_FailWorkflowExecutionCommandAttributes{FailWorkflowExecutionCommandAttributes: &commandpb.FailWorkflowExecutionCommandAttributes{
-							Failure: failure.NewServerFailure("cannot decode last completion result", true),
-						}},
-					}}, nil
-			}
+			counter = s.decodePayloadsInt(startedEvent.GetWorkflowExecutionStartedEventAttributes().GetLastCompletionResult())
 		}
 
 		executions = append(executions, execution)
@@ -719,7 +677,7 @@ func (s *integrationSuite) TestCronWorkflow_Success() {
 	s.Logger.Info("Process first cron run")
 	_, err = poller.PollAndProcessWorkflowTask(true, false)
 	s.Logger.Info("First cron run processed")
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	// Make sure the cron workflow start running at a proper time, in this case 3 seconds after the
 	// startWorkflowExecution request
@@ -730,7 +688,7 @@ func (s *integrationSuite) TestCronWorkflow_Success() {
 	s.Logger.Info("Process second cron run")
 	_, err = poller.PollAndProcessWorkflowTask(false, false)
 	s.Logger.Info("Second cron run processed")
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	describeWorkflowExecutionFunc := func(execution *commonpb.WorkflowExecution) (*workflowservice.DescribeWorkflowExecutionResponse, error) {
 		return s.engine.DescribeWorkflowExecution(NewContext(), &workflowservice.DescribeWorkflowExecutionRequest{
@@ -756,7 +714,7 @@ func (s *integrationSuite) TestCronWorkflow_Success() {
 	s.Logger.Info("Process fourth cron run")
 	_, err = poller.PollAndProcessWorkflowTask(false, false)
 	s.Logger.Info("Third cron run processed")
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	s.Logger.Info("Terminate cron workflow")
 	_, terminateErr := s.engine.TerminateWorkflowExecution(NewContext(), &workflowservice.TerminateWorkflowExecutionRequest{
@@ -801,29 +759,21 @@ func (s *integrationSuite) TestCronWorkflow_Success() {
 	s.Equal(enumspb.WORKFLOW_EXECUTION_STATUS_TERMINATED, lastExecution.GetStatus())
 
 	finalCronExecution := closedExecutions[1]
-	s.Equal(enumspb.WORKFLOW_EXECUTION_STATUS_CONTINUED_AS_NEW, finalCronExecution.GetStatus())
+	s.Equal(enumspb.WORKFLOW_EXECUTION_STATUS_COMPLETED, finalCronExecution.GetStatus())
 	events := s.getHistory(s.namespace, finalCronExecution.GetExecution())
 	lastEvent := events[len(events)-1]
-	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW, lastEvent.GetEventType())
-	attributes := lastEvent.GetWorkflowExecutionContinuedAsNewEventAttributes()
-	s.Equal(enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE, attributes.GetInitiator())
-	var r int
-	err = payloads.Decode(attributes.GetLastCompletionResult(), &r)
-	s.NoError(err)
-	s.Equal(3, r)
+	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_COMPLETED, lastEvent.GetEventType())
+	attrs1 := lastEvent.GetWorkflowExecutionCompletedEventAttributes()
+	s.Equal(3, s.decodePayloadsInt(attrs1.GetResult()))
 
 	timedoutExecution := closedExecutions[2]
-	// When cron workflow timesout we continue as new to start the new run
-	s.Equal(enumspb.WORKFLOW_EXECUTION_STATUS_CONTINUED_AS_NEW, timedoutExecution.GetStatus())
+	// When cron workflow timesout we continue as new to start the new run,
+	// but with status set to TIMED_OUT
+	s.Equal(enumspb.WORKFLOW_EXECUTION_STATUS_TIMED_OUT, timedoutExecution.GetStatus())
 	// check when workflow timeout, continueAsNew event contains expected fields
 	events = s.getHistory(s.namespace, timedoutExecution.GetExecution())
 	lastEvent = events[len(events)-1]
-	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW, lastEvent.GetEventType())
-	attributes = lastEvent.GetWorkflowExecutionContinuedAsNewEventAttributes()
-	s.Equal(enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE, attributes.GetInitiator())
-	s.Equal(enumspb.TIMEOUT_TYPE_START_TO_CLOSE, attributes.GetFailure().GetTimeoutFailureInfo().GetTimeoutType())
-	s.NotNil(attributes.GetLastCompletionResult())
-	err = payloads.Decode(attributes.GetLastCompletionResult(), &r)
-	s.NoError(err)
-	s.Equal(2, r)
+	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_TIMED_OUT, lastEvent.GetEventType())
+	attrs2 := lastEvent.GetWorkflowExecutionTimedOutEventAttributes()
+	s.Equal(attrs2.GetRetryState(), enumspb.RETRY_STATE_RETRY_POLICY_NOT_SET)
 }
