@@ -26,7 +26,6 @@ package host
 
 import (
 	"fmt"
-	"io/ioutil"
 	"os"
 	"time"
 
@@ -37,15 +36,15 @@ import (
 	historypb "go.temporal.io/api/history/v1"
 	namespacepb "go.temporal.io/api/namespace/v1"
 	"go.temporal.io/api/workflowservice/v1"
-	"gopkg.in/yaml.v2"
+	"gopkg.in/yaml.v3"
 
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
-	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
+	"go.temporal.io/server/common/persistence/visibility/store/elasticsearch/client"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/common/rpc"
 	"go.temporal.io/server/environment"
@@ -94,22 +93,27 @@ func (s *IntegrationBase) setupSuite(defaultClusterConfigFile string) {
 		s.adminClient = s.testCluster.GetAdminClient()
 	}
 
-	s.testRawHistoryNamespaceName = "TestRawHistoryNamespace"
 	s.namespace = s.randomizeStr("integration-test-namespace")
-	s.Require().NoError(
-		s.registerNamespace(s.namespace, 24*time.Hour, enumspb.ARCHIVAL_STATE_DISABLED, "", enumspb.ARCHIVAL_STATE_DISABLED, ""))
-	s.Require().NoError(
-		s.registerNamespace(s.testRawHistoryNamespaceName, 24*time.Hour, enumspb.ARCHIVAL_STATE_DISABLED, "", enumspb.ARCHIVAL_STATE_DISABLED, ""))
+	s.Require().NoError(s.registerNamespace(s.namespace, 24*time.Hour, enumspb.ARCHIVAL_STATE_DISABLED, "", enumspb.ARCHIVAL_STATE_DISABLED, ""))
+
+	s.testRawHistoryNamespaceName = "TestRawHistoryNamespace"
+	s.Require().NoError(s.registerNamespace(s.testRawHistoryNamespaceName, 24*time.Hour, enumspb.ARCHIVAL_STATE_DISABLED, "", enumspb.ARCHIVAL_STATE_DISABLED, ""))
 
 	s.foreignNamespace = s.randomizeStr("integration-foreign-test-namespace")
-	s.Require().NoError(
-		s.registerNamespace(s.foreignNamespace, 24*time.Hour, enumspb.ARCHIVAL_STATE_DISABLED, "", enumspb.ARCHIVAL_STATE_DISABLED, ""))
+	s.Require().NoError(s.registerNamespace(s.foreignNamespace, 24*time.Hour, enumspb.ARCHIVAL_STATE_DISABLED, "", enumspb.ARCHIVAL_STATE_DISABLED, ""))
 
-	s.Require().NoError(s.registerArchivalNamespace())
+	if clusterConfig.EnableArchival {
+		s.archivalNamespace = s.randomizeStr("integration-archival-enabled-namespace")
+		s.Require().NoError(s.registerArchivalNamespace(s.archivalNamespace))
+	}
 
-	// this sleep is necessary because namespacev2 cache gets refreshed in the
-	// background only every namespaceCacheRefreshInterval period
-	time.Sleep(cache.NamespaceCacheRefreshInterval + time.Second)
+	if clusterConfig.FrontendAddress == "" {
+		// Poke all the in-process namespace caches to refresh without waiting for the usual refresh interval.
+		s.testCluster.RefreshNamespaceCache()
+	} else {
+		// Wait for one whole cycle of the namespace cache v2 refresh interval to be sure that our namespaces are loaded.
+		time.Sleep(namespace.CacheRefreshInterval + time.Second)
+	}
 }
 
 func (s *IntegrationBase) setupLogger() {
@@ -126,7 +130,7 @@ func GetTestClusterConfig(configFile string) (*TestClusterConfig, error) {
 	}
 	// This is just reading a config so it's less of a security concern
 	// #nosec
-	confContent, err := ioutil.ReadFile(configLocation)
+	confContent, err := os.ReadFile(configLocation)
 	if err != nil {
 		return nil, fmt.Errorf("failed to read test cluster config file %v: %v", configLocation, err)
 	}
@@ -138,18 +142,26 @@ func GetTestClusterConfig(configFile string) (*TestClusterConfig, error) {
 
 	options.FrontendAddress = TestFlags.FrontendAddr
 	if options.ESConfig != nil {
-		options.ESConfig.Indices[config.VisibilityAppName] += uuid.New()
+		options.ESConfig.Indices[client.VisibilityAppName] += uuid.New()
 	}
 	return &options, nil
 }
 
 func (s *IntegrationBase) tearDownSuite() {
+	s.Require().NoError(s.deleteNamespace(s.namespace))
+	s.Require().NoError(s.deleteNamespace(s.testRawHistoryNamespaceName))
+	s.Require().NoError(s.deleteNamespace(s.foreignNamespace))
+	if s.archivalNamespace != "" {
+		s.Require().NoError(s.deleteNamespace(s.archivalNamespace))
+	}
+
 	if s.testCluster != nil {
 		s.testCluster.TearDownCluster()
 		s.testCluster = nil
-		s.engine = nil
-		s.adminClient = nil
 	}
+
+	s.engine = nil
+	s.adminClient = nil
 }
 
 func (s *IntegrationBase) registerNamespace(
@@ -175,6 +187,21 @@ func (s *IntegrationBase) registerNamespace(
 	return err
 }
 
+func (s *IntegrationBase) deleteNamespace(
+	namespace string,
+) error {
+	ctx, cancel := rpc.NewContextWithTimeoutAndHeaders(10000 * time.Second)
+	defer cancel()
+	_, err := s.engine.UpdateNamespace(ctx, &workflowservice.UpdateNamespaceRequest{
+		Namespace: namespace,
+		UpdateInfo: &namespacepb.UpdateNamespaceInfo{
+			State: enumspb.NAMESPACE_STATE_DELETED,
+		},
+	})
+
+	return err
+}
+
 func (s *IntegrationBase) randomizeStr(id string) string {
 	return fmt.Sprintf("%v-%v", id, uuid.New())
 }
@@ -212,14 +239,13 @@ func (s *IntegrationBase) getHistory(namespace string, execution *commonpb.Workf
 // To register archival namespace we can't use frontend API as the retention period is set to 0 for testing,
 // and request will be rejected by frontend. Here we make a call directly to persistence to register
 // the namespace.
-func (s *IntegrationBase) registerArchivalNamespace() error {
-	s.archivalNamespace = s.randomizeStr("integration-archival-enabled-namespace")
+func (s *IntegrationBase) registerArchivalNamespace(archivalNamespace string) error {
 	currentClusterName := s.testCluster.testBase.ClusterMetadata.GetCurrentClusterName()
 	namespaceRequest := &persistence.CreateNamespaceRequest{
 		Namespace: &persistencespb.NamespaceDetail{
 			Info: &persistencespb.NamespaceInfo{
 				Id:    uuid.New(),
-				Name:  s.archivalNamespace,
+				Name:  archivalNamespace,
 				State: enumspb.NAMESPACE_STATE_REGISTERED,
 			},
 			Config: &persistencespb.NamespaceConfig{
@@ -244,7 +270,7 @@ func (s *IntegrationBase) registerArchivalNamespace() error {
 	response, err := s.testCluster.testBase.MetadataManager.CreateNamespace(namespaceRequest)
 
 	s.Logger.Info("Register namespace succeeded",
-		tag.WorkflowNamespace(s.archivalNamespace),
+		tag.WorkflowNamespace(archivalNamespace),
 		tag.WorkflowNamespaceID(response.ID),
 	)
 	return err
