@@ -28,7 +28,6 @@ import (
 	"context"
 	"flag"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"sync/atomic"
 	"testing"
@@ -57,15 +56,15 @@ import (
 	taskqueuepb "go.temporal.io/api/taskqueue/v1"
 	"go.temporal.io/api/workflowservice/v1"
 	"google.golang.org/grpc"
-	"gopkg.in/yaml.v2"
+	"gopkg.in/yaml.v3"
 
 	"go.temporal.io/server/api/adminservice/v1"
 	"go.temporal.io/server/api/adminservicemock/v1"
 	"go.temporal.io/server/api/historyservice/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
+	"go.temporal.io/server/common/namespace"
 	test "go.temporal.io/server/common/testing"
 	"go.temporal.io/server/environment"
 	"go.temporal.io/server/host"
@@ -117,7 +116,7 @@ func (s *nDCIntegrationTestSuite) SetupSuite() {
 	}
 	environment.SetupEnv()
 
-	confContent, err := ioutil.ReadFile(fileName)
+	confContent, err := os.ReadFile(fileName)
 	s.Require().NoError(err)
 	confContent = []byte(os.ExpandEnv(string(confContent)))
 
@@ -1627,6 +1626,8 @@ func (s *nDCIntegrationTestSuite) registerNamespace() {
 		WorkflowExecutionRetentionPeriod: timestamp.DurationPtr(1 * time.Hour * 24),
 	})
 	s.Require().NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(2 * namespace.CacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: s.namespace,
@@ -1635,8 +1636,6 @@ func (s *nDCIntegrationTestSuite) registerNamespace() {
 	s.Require().NoError(err)
 	s.Require().NotNil(resp)
 	s.namespaceID = resp.GetNamespaceInfo().GetId()
-	// Wait for namespace cache to pick the change
-	time.Sleep(2 * cache.NamespaceCacheRefreshInterval)
 
 	s.logger.Info("Registered namespace", tag.WorkflowNamespace(s.namespace), tag.WorkflowNamespaceID(s.namespaceID))
 }
@@ -1763,7 +1762,6 @@ func (s *nDCIntegrationTestSuite) applyEventsThroughFetcher(
 			TaskType:     taskType,
 			SourceTaskId: 1,
 			Attributes: &replicationspb.ReplicationTask_HistoryTaskV2Attributes{HistoryTaskV2Attributes: &replicationspb.HistoryTaskV2Attributes{
-				TaskId:              1,
 				NamespaceId:         s.namespaceID,
 				WorkflowId:          workflowID,
 				RunId:               runID,
