@@ -22,17 +22,19 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
+//go:build !race
 // +build !race
+
 // need to run xdc tests with race detector off because of ringpop bug causing data race issue
 
 package xdc
 
 import (
 	"bytes"
+	"context"
 	"encoding/binary"
 	"errors"
 	"flag"
-	"io/ioutil"
 	"os"
 	"strconv"
 	"testing"
@@ -50,14 +52,19 @@ import (
 	"go.temporal.io/api/serviceerror"
 	taskqueuepb "go.temporal.io/api/taskqueue/v1"
 	"go.temporal.io/api/workflowservice/v1"
-	"gopkg.in/yaml.v2"
+	"go.temporal.io/sdk/activity"
+	sdkclient "go.temporal.io/sdk/client"
+	sdkworker "go.temporal.io/sdk/worker"
+	"go.temporal.io/sdk/workflow"
+	"gopkg.in/yaml.v3"
 
+	"go.temporal.io/server/api/historyservice/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/convert"
 	"go.temporal.io/server/common/failure"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/payloads"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/environment"
@@ -77,7 +84,7 @@ type (
 )
 
 const (
-	cacheRefreshInterval = cache.NamespaceCacheRefreshInterval + 5*time.Second
+	cacheRefreshInterval = namespace.CacheRefreshInterval + 5*time.Second
 )
 
 var (
@@ -106,7 +113,7 @@ func (s *integrationClustersTestSuite) SetupSuite() {
 	}
 	environment.SetupEnv()
 
-	confContent, err := ioutil.ReadFile(fileName)
+	confContent, err := os.ReadFile(fileName)
 	s.Require().NoError(err)
 	confContent = []byte(os.ExpandEnv(string(confContent)))
 
@@ -132,6 +139,11 @@ func (s *integrationClustersTestSuite) TearDownSuite() {
 	s.cluster2.TearDownCluster()
 }
 
+func (s *integrationClustersTestSuite) decodePayloadsString(ps *commonpb.Payloads) (r string) {
+	s.NoError(payloads.Decode(ps, &r))
+	return
+}
+
 func (s *integrationClustersTestSuite) TestNamespaceFailover() {
 	namespace := "test-namespace-for-fail-over-" + common.GenerateRandomString(5)
 	client1 := s.cluster1.GetFrontendClient() // active
@@ -144,6 +156,8 @@ func (s *integrationClustersTestSuite) TestNamespaceFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: namespace,
@@ -151,8 +165,6 @@ func (s *integrationClustersTestSuite) TestNamespaceFailover() {
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 	resp2, err := client2.DescribeNamespace(host.NewContext(), descReq)
@@ -208,10 +220,10 @@ func (s *integrationClustersTestSuite) TestNamespaceFailover() {
 }
 
 func (s *integrationClustersTestSuite) TestSimpleWorkflowFailover() {
-	namespace := "test-simple-workflow-failover-" + common.GenerateRandomString(5)
+	namespaceName := "test-simple-workflow-failover-" + common.GenerateRandomString(5)
 	client1 := s.cluster1.GetFrontendClient() // active
 	regReq := &workflowservice.RegisterNamespaceRequest{
-		Namespace:                        namespace,
+		Namespace:                        namespaceName,
 		IsGlobalNamespace:                true,
 		Clusters:                         clusterReplicationConfig,
 		ActiveClusterName:                clusterName[0],
@@ -219,15 +231,15 @@ func (s *integrationClustersTestSuite) TestSimpleWorkflowFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
-		Namespace: namespace,
+		Namespace: namespaceName,
 	}
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cache.NamespaceCacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 	resp2, err := client2.DescribeNamespace(host.NewContext(), descReq)
@@ -244,7 +256,7 @@ func (s *integrationClustersTestSuite) TestSimpleWorkflowFailover() {
 	taskQueue := &taskqueuepb.TaskQueue{Name: tq}
 	startReq := &workflowservice.StartWorkflowExecutionRequest{
 		RequestId:           uuid.New(),
-		Namespace:           namespace,
+		Namespace:           namespaceName,
 		WorkflowId:          id,
 		WorkflowType:        workflowType,
 		TaskQueue:           taskQueue,
@@ -314,7 +326,7 @@ func (s *integrationClustersTestSuite) TestSimpleWorkflowFailover() {
 
 	poller := host.TaskPoller{
 		Engine:              client1,
-		Namespace:           namespace,
+		Namespace:           namespaceName,
 		TaskQueue:           taskQueue,
 		Identity:            identity,
 		WorkflowTaskHandler: wtHandler,
@@ -326,7 +338,7 @@ func (s *integrationClustersTestSuite) TestSimpleWorkflowFailover() {
 
 	poller2 := host.TaskPoller{
 		Engine:              client2,
-		Namespace:           namespace,
+		Namespace:           namespaceName,
 		TaskQueue:           taskQueue,
 		Identity:            identity,
 		WorkflowTaskHandler: wtHandler,
@@ -348,7 +360,7 @@ func (s *integrationClustersTestSuite) TestSimpleWorkflowFailover() {
 	queryResultCh := make(chan QueryResult)
 	queryWorkflowFn := func(client workflowservice.WorkflowServiceClient, queryType string) {
 		queryResp, err := client.QueryWorkflow(host.NewContext(), &workflowservice.QueryWorkflowRequest{
-			Namespace: namespace,
+			Namespace: namespaceName,
 			Execution: &commonpb.WorkflowExecution{
 				WorkflowId: id,
 				RunId:      we.RunId,
@@ -377,10 +389,7 @@ func (s *integrationClustersTestSuite) TestSimpleWorkflowFailover() {
 	s.NoError(queryResult.Err)
 	s.NotNil(queryResult.Resp)
 	s.NotNil(queryResult.Resp.QueryResult)
-	var queryResultString string
-	err = payloads.Decode(queryResult.Resp.QueryResult, &queryResultString)
-	s.NoError(err)
-	s.Equal("query-result", queryResultString)
+	s.Equal("query-result", s.decodePayloadsString(queryResult.Resp.GetQueryResult()))
 
 	// Wait a while so the events are replicated.
 	time.Sleep(5 * time.Second)
@@ -402,15 +411,13 @@ func (s *integrationClustersTestSuite) TestSimpleWorkflowFailover() {
 	s.NoError(queryResult.Err)
 	s.NotNil(queryResult.Resp)
 	s.NotNil(queryResult.Resp.QueryResult)
-	err = payloads.Decode(queryResult.Resp.QueryResult, &queryResultString)
-	s.NoError(err)
-	s.Equal("query-result", queryResultString)
+	s.Equal("query-result", s.decodePayloadsString(queryResult.Resp.GetQueryResult()))
 
-	s.failover(namespace, clusterName[1], int64(2), client1)
+	s.failover(namespaceName, clusterName[1], int64(2), client1)
 
 	// check history matched
 	getHistoryReq := &workflowservice.GetWorkflowExecutionHistoryRequest{
-		Namespace: namespace,
+		Namespace: namespaceName,
 		Execution: &commonpb.WorkflowExecution{
 			WorkflowId: id,
 			RunId:      rid,
@@ -447,9 +454,7 @@ func (s *integrationClustersTestSuite) TestSimpleWorkflowFailover() {
 	s.NoError(queryResult.Err)
 	s.NotNil(queryResult.Resp)
 	s.NotNil(queryResult.Resp.QueryResult)
-	err = payloads.Decode(queryResult.Resp.QueryResult, &queryResultString)
-	s.NoError(err)
-	s.Equal("query-result", queryResultString)
+	s.Equal("query-result", s.decodePayloadsString(queryResult.Resp.GetQueryResult()))
 
 	// call QueryWorkflow in separate goroutinue (because it is blocking). That will generate a query task
 	go queryWorkflowFn(client2, queryType)
@@ -468,9 +473,7 @@ func (s *integrationClustersTestSuite) TestSimpleWorkflowFailover() {
 	s.NoError(queryResult.Err)
 	s.NotNil(queryResult.Resp)
 	s.NotNil(queryResult.Resp.QueryResult)
-	err = payloads.Decode(queryResult.Resp.QueryResult, &queryResultString)
-	s.NoError(err)
-	s.Equal("query-result", queryResultString)
+	s.Equal("query-result", s.decodePayloadsString(queryResult.Resp.GetQueryResult()))
 
 	// make process in cluster 2
 	err = poller2.PollAndProcessActivityTask(false)
@@ -509,6 +512,8 @@ func (s *integrationClustersTestSuite) TestStickyWorkflowTaskFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: namespace,
@@ -516,8 +521,6 @@ func (s *integrationClustersTestSuite) TestStickyWorkflowTaskFailover() {
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 
@@ -648,10 +651,10 @@ func (s *integrationClustersTestSuite) TestStickyWorkflowTaskFailover() {
 }
 
 func (s *integrationClustersTestSuite) TestStartWorkflowExecution_Failover_WorkflowIDReusePolicy() {
-	namespace := "test-start-workflow-failover-ID-reuse-policy" + common.GenerateRandomString(5)
+	namespaceName := "test-start-workflow-failover-ID-reuse-policy" + common.GenerateRandomString(5)
 	client1 := s.cluster1.GetFrontendClient() // active
 	regReq := &workflowservice.RegisterNamespaceRequest{
-		Namespace:                        namespace,
+		Namespace:                        namespaceName,
 		IsGlobalNamespace:                true,
 		Clusters:                         clusterReplicationConfig,
 		ActiveClusterName:                clusterName[0],
@@ -659,15 +662,15 @@ func (s *integrationClustersTestSuite) TestStartWorkflowExecution_Failover_Workf
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(namespace.CacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
-		Namespace: namespace,
+		Namespace: namespaceName,
 	}
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cache.NamespaceCacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 	resp2, err := client2.DescribeNamespace(host.NewContext(), descReq)
@@ -684,7 +687,7 @@ func (s *integrationClustersTestSuite) TestStartWorkflowExecution_Failover_Workf
 	taskQueue := &taskqueuepb.TaskQueue{Name: tl}
 	startReq := &workflowservice.StartWorkflowExecutionRequest{
 		RequestId:             uuid.New(),
-		Namespace:             namespace,
+		Namespace:             namespaceName,
 		WorkflowId:            id,
 		WorkflowType:          workflowType,
 		TaskQueue:             taskQueue,
@@ -714,7 +717,7 @@ func (s *integrationClustersTestSuite) TestStartWorkflowExecution_Failover_Workf
 
 	poller := host.TaskPoller{
 		Engine:              client1,
-		Namespace:           namespace,
+		Namespace:           namespaceName,
 		TaskQueue:           taskQueue,
 		Identity:            identity,
 		WorkflowTaskHandler: wtHandler,
@@ -725,7 +728,7 @@ func (s *integrationClustersTestSuite) TestStartWorkflowExecution_Failover_Workf
 
 	poller2 := host.TaskPoller{
 		Engine:              client2,
-		Namespace:           namespace,
+		Namespace:           namespaceName,
 		TaskQueue:           taskQueue,
 		Identity:            identity,
 		WorkflowTaskHandler: wtHandler,
@@ -740,7 +743,7 @@ func (s *integrationClustersTestSuite) TestStartWorkflowExecution_Failover_Workf
 	s.NoError(err)
 	s.Equal(1, workflowCompleteTimes)
 
-	s.failover(namespace, clusterName[1], int64(2), client1)
+	s.failover(namespaceName, clusterName[1], int64(2), client1)
 
 	// start the same workflow in cluster 2 is not allowed if policy is AllowDuplicateFailedOnly
 	startReq.RequestId = uuid.New()
@@ -782,6 +785,8 @@ func (s *integrationClustersTestSuite) TestTerminateFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: namespace,
@@ -789,8 +794,6 @@ func (s *integrationClustersTestSuite) TestTerminateFailover() {
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 
@@ -953,6 +956,8 @@ func (s *integrationClustersTestSuite) TestResetWorkflowFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: namespace,
@@ -960,8 +965,6 @@ func (s *integrationClustersTestSuite) TestResetWorkflowFailover() {
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 
@@ -1110,6 +1113,8 @@ func (s *integrationClustersTestSuite) TestContinueAsNewFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: namespace,
@@ -1117,8 +1122,6 @@ func (s *integrationClustersTestSuite) TestContinueAsNewFailover() {
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 
@@ -1234,6 +1237,8 @@ func (s *integrationClustersTestSuite) TestSignalFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: namespace,
@@ -1241,8 +1246,6 @@ func (s *integrationClustersTestSuite) TestSignalFailover() {
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 
@@ -1401,6 +1404,8 @@ func (s *integrationClustersTestSuite) TestUserTimerFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: namespace,
@@ -1408,8 +1413,6 @@ func (s *integrationClustersTestSuite) TestUserTimerFailover() {
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 
@@ -1547,201 +1550,6 @@ func (s *integrationClustersTestSuite) TestUserTimerFailover() {
 	}
 }
 
-func (s *integrationClustersTestSuite) TestActivityHeartbeatFailover() {
-	namespace := "test-activity-heartbeat-workflow-failover-" + common.GenerateRandomString(5)
-	client1 := s.cluster1.GetFrontendClient() // active
-	regReq := &workflowservice.RegisterNamespaceRequest{
-		Namespace:                        namespace,
-		IsGlobalNamespace:                true,
-		Clusters:                         clusterReplicationConfig,
-		ActiveClusterName:                clusterName[0],
-		WorkflowExecutionRetentionPeriod: timestamp.DurationPtr(1 * time.Hour * 24),
-	}
-	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
-	s.NoError(err)
-
-	descReq := &workflowservice.DescribeNamespaceRequest{
-		Namespace: namespace,
-	}
-	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
-	s.NoError(err)
-	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
-
-	client2 := s.cluster2.GetFrontendClient() // standby
-
-	// Start a workflow
-	id := "integration-activity-heartbeat-workflow-failover-test"
-	wt := "integration-activity-heartbeat-workflow-failover-test-type"
-	tl := "integration-activity-heartbeat-workflow-failover-test-taskqueue"
-	identity1 := "worker1"
-	identity2 := "worker2"
-	workflowType := &commonpb.WorkflowType{Name: wt}
-	taskQueue := &taskqueuepb.TaskQueue{Name: tl}
-	startReq := &workflowservice.StartWorkflowExecutionRequest{
-		RequestId:           uuid.New(),
-		Namespace:           namespace,
-		WorkflowId:          id,
-		WorkflowType:        workflowType,
-		TaskQueue:           taskQueue,
-		Input:               nil,
-		WorkflowRunTimeout:  timestamp.DurationPtr(300 * time.Second),
-		WorkflowTaskTimeout: timestamp.DurationPtr(10 * time.Second),
-		Identity:            identity1,
-	}
-	var we *workflowservice.StartWorkflowExecutionResponse
-	for i := 0; i < 10; i++ {
-		we, err = client1.StartWorkflowExecution(host.NewContext(), startReq)
-		if err == nil {
-			break
-		}
-		time.Sleep(1 * time.Second)
-	}
-	s.NoError(err)
-	s.NotNil(we.GetRunId())
-
-	s.logger.Info("StartWorkflowExecution", tag.WorkflowRunID(we.GetRunId()))
-
-	activitySent := false
-	wtHandler := func(execution *commonpb.WorkflowExecution, wt *commonpb.WorkflowType,
-		previousStartedEventID, startedEventID int64, history *historypb.History) ([]*commandpb.Command, error) {
-		if !activitySent {
-			activitySent = true
-			return []*commandpb.Command{{
-				CommandType: enumspb.COMMAND_TYPE_SCHEDULE_ACTIVITY_TASK,
-				Attributes: &commandpb.Command_ScheduleActivityTaskCommandAttributes{ScheduleActivityTaskCommandAttributes: &commandpb.ScheduleActivityTaskCommandAttributes{
-					ActivityId:             strconv.Itoa(1),
-					ActivityType:           &commonpb.ActivityType{Name: "some random activity type"},
-					TaskQueue:              &taskqueuepb.TaskQueue{Name: tl},
-					Input:                  payloads.EncodeString("some random input"),
-					ScheduleToCloseTimeout: timestamp.DurationPtr(1000 * time.Second),
-					ScheduleToStartTimeout: timestamp.DurationPtr(1000 * time.Second),
-					StartToCloseTimeout:    timestamp.DurationPtr(1000 * time.Second),
-					HeartbeatTimeout:       timestamp.DurationPtr(3 * time.Second),
-					RetryPolicy: &commonpb.RetryPolicy{
-						InitialInterval:        timestamp.DurationPtr(1 * time.Second),
-						MaximumAttempts:        3,
-						MaximumInterval:        timestamp.DurationPtr(1 * time.Second),
-						NonRetryableErrorTypes: []string{"bad-bug"},
-						BackoffCoefficient:     1,
-					},
-				}},
-			}}, nil
-		}
-
-		return []*commandpb.Command{{
-			CommandType: enumspb.COMMAND_TYPE_COMPLETE_WORKFLOW_EXECUTION,
-			Attributes: &commandpb.Command_CompleteWorkflowExecutionCommandAttributes{CompleteWorkflowExecutionCommandAttributes: &commandpb.CompleteWorkflowExecutionCommandAttributes{
-				Result: payloads.EncodeString("Done"),
-			}},
-		}}, nil
-	}
-
-	// activity handler
-	activity1Called := false
-	heartbeatDetails := payloads.EncodeString("details")
-	atHandler1 := func(execution *commonpb.WorkflowExecution, activityType *commonpb.ActivityType,
-		activityID string, input *commonpb.Payloads, taskToken []byte) (*commonpb.Payloads, bool, error) {
-		activity1Called = true
-		_, err = client1.RecordActivityTaskHeartbeat(host.NewContext(), &workflowservice.RecordActivityTaskHeartbeatRequest{
-			TaskToken: taskToken, Details: heartbeatDetails})
-		s.NoError(err)
-		time.Sleep(5 * time.Second)
-		return payloads.EncodeString("Activity Result"), false, nil
-	}
-
-	// activity handler
-	activity2Called := false
-	atHandler2 := func(execution *commonpb.WorkflowExecution, activityType *commonpb.ActivityType,
-		activityID string, input *commonpb.Payloads, taskToken []byte) (*commonpb.Payloads, bool, error) {
-		activity2Called = true
-		return payloads.EncodeString("Activity Result"), false, nil
-	}
-
-	poller1 := &host.TaskPoller{
-		Engine:              client1,
-		Namespace:           namespace,
-		TaskQueue:           taskQueue,
-		Identity:            identity1,
-		WorkflowTaskHandler: wtHandler,
-		ActivityTaskHandler: atHandler1,
-		Logger:              s.logger,
-		T:                   s.T(),
-	}
-
-	poller2 := &host.TaskPoller{
-		Engine:              client2,
-		Namespace:           namespace,
-		TaskQueue:           taskQueue,
-		Identity:            identity2,
-		WorkflowTaskHandler: wtHandler,
-		ActivityTaskHandler: atHandler2,
-		Logger:              s.logger,
-		T:                   s.T(),
-	}
-
-	describeWorkflowExecution := func(client workflowservice.WorkflowServiceClient) (*workflowservice.DescribeWorkflowExecutionResponse, error) {
-		return client.DescribeWorkflowExecution(host.NewContext(), &workflowservice.DescribeWorkflowExecutionRequest{
-			Namespace: namespace,
-			Execution: &commonpb.WorkflowExecution{
-				WorkflowId: id,
-				RunId:      we.RunId,
-			},
-		})
-	}
-
-	_, err = poller1.PollAndProcessWorkflowTask(false, false)
-	s.NoError(err)
-	err = poller1.PollAndProcessActivityTask(false)
-	s.IsType(&serviceerror.NotFound{}, err)
-
-	s.failover(namespace, clusterName[1], int64(2), client1)
-
-	// Make sure the heartbeat details are sent to cluster2 even when the activity at cluster1
-	// has heartbeat timeout. Also make sure the information is recorded when the activity state
-	// is "Scheduled"
-	dweResponse, err := describeWorkflowExecution(client2)
-	s.NoError(err)
-	pendingActivities := dweResponse.GetPendingActivities()
-	s.Equal(1, len(pendingActivities))
-	s.Equal(enumspb.PENDING_ACTIVITY_STATE_SCHEDULED, pendingActivities[0].GetState())
-	s.Equal(heartbeatDetails, pendingActivities[0].GetHeartbeatDetails())
-	s.Equal(enumspb.TIMEOUT_TYPE_HEARTBEAT, pendingActivities[0].GetLastFailure().GetTimeoutFailureInfo().GetTimeoutType())
-	s.Equal(identity1, pendingActivities[0].GetLastWorkerIdentity())
-
-	for i := 0; i < 10; i++ {
-		poller2.PollAndProcessActivityTask(false)
-		if activity2Called {
-			break
-		} else {
-			time.Sleep(1 * time.Second)
-		}
-	}
-
-	s.True(activity1Called)
-	s.True(activity2Called)
-
-	historyResponse, err := client2.GetWorkflowExecutionHistory(host.NewContext(), &workflowservice.GetWorkflowExecutionHistoryRequest{
-		Namespace: namespace,
-		Execution: &commonpb.WorkflowExecution{
-			WorkflowId: id,
-		},
-	})
-	s.NoError(err)
-	history := historyResponse.History
-
-	activityRetryFound := false
-	for _, event := range history.Events {
-		if event.GetEventType() == enumspb.EVENT_TYPE_ACTIVITY_TASK_STARTED {
-			attribute := event.GetActivityTaskStartedEventAttributes()
-			s.True(attribute.GetAttempt() > 1)
-			activityRetryFound = true
-		}
-	}
-	s.True(activityRetryFound)
-}
-
 func (s *integrationClustersTestSuite) TestTransientWorkflowTaskFailover() {
 	namespace := "test-transient-workflow-task-workflow-failover-" + common.GenerateRandomString(5)
 	client1 := s.cluster1.GetFrontendClient() // active
@@ -1754,6 +1562,8 @@ func (s *integrationClustersTestSuite) TestTransientWorkflowTaskFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: namespace,
@@ -1761,8 +1571,6 @@ func (s *integrationClustersTestSuite) TestTransientWorkflowTaskFailover() {
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 
@@ -1861,6 +1669,8 @@ func (s *integrationClustersTestSuite) TestCronWorkflowFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: namespace,
@@ -1868,8 +1678,6 @@ func (s *integrationClustersTestSuite) TestCronWorkflowFailover() {
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 
@@ -1946,6 +1754,8 @@ func (s *integrationClustersTestSuite) TestWorkflowRetryFailover() {
 	}
 	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
 	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
 
 	descReq := &workflowservice.DescribeNamespaceRequest{
 		Namespace: namespace,
@@ -1953,8 +1763,6 @@ func (s *integrationClustersTestSuite) TestWorkflowRetryFailover() {
 	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
 	s.NoError(err)
 	s.NotNil(resp)
-	// Wait for namespace cache to pick the change
-	time.Sleep(cacheRefreshInterval)
 
 	client2 := s.cluster2.GetFrontendClient() // standby
 
@@ -2016,14 +1824,14 @@ func (s *integrationClustersTestSuite) TestWorkflowRetryFailover() {
 	_, err = poller2.PollAndProcessWorkflowTask(false, false)
 	s.NoError(err)
 	events := s.getHistory(client2, namespace, executions[0])
-	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW, events[len(events)-1].GetEventType())
+	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_FAILED, events[len(events)-1].GetEventType())
 	s.Equal(int32(1), events[0].GetWorkflowExecutionStartedEventAttributes().GetAttempt())
 
 	// second attempt
 	_, err = poller2.PollAndProcessWorkflowTask(false, false)
 	s.NoError(err)
 	events = s.getHistory(client2, namespace, executions[1])
-	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW, events[len(events)-1].GetEventType())
+	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_FAILED, events[len(events)-1].GetEventType())
 	s.Equal(int32(2), events[0].GetWorkflowExecutionStartedEventAttributes().GetAttempt())
 
 	// third attempt. Still failing, should stop retry.
@@ -2034,6 +1842,113 @@ func (s *integrationClustersTestSuite) TestWorkflowRetryFailover() {
 	s.Equal(int32(3), events[0].GetWorkflowExecutionStartedEventAttributes().GetAttempt())
 }
 
+func (s *integrationClustersTestSuite) TestActivityHeartbeatFailover() {
+	namespace := "test-activity-heartbeat-workflow-failover-" + common.GenerateRandomString(5)
+	s.registerNamespace(namespace)
+
+	taskqueue := "integration-activity-heartbeat-workflow-failover-test-taskqueue"
+	client1, worker1 := s.newClientAndWorker(s.cluster1.GetHost().FrontendGRPCAddress(), namespace, taskqueue, "worker1")
+	client2, worker2 := s.newClientAndWorker(s.cluster2.GetHost().FrontendGRPCAddress(), namespace, taskqueue, "worker2")
+
+	lastAttemptCount := 0
+	expectedHeartbeatValue := 100
+	activityWithHB := func(ctx context.Context) error {
+		lastAttemptCount = int(activity.GetInfo(ctx).Attempt)
+		if activity.HasHeartbeatDetails(ctx) {
+			var retrievedHeartbeatValue int
+			if err := activity.GetHeartbeatDetails(ctx, &retrievedHeartbeatValue); err == nil {
+				s.Equal(expectedHeartbeatValue, retrievedHeartbeatValue)
+				return nil
+			}
+		}
+		activity.RecordHeartbeat(ctx, expectedHeartbeatValue)
+		time.Sleep(time.Second * 10)
+		return errors.New("no heartbeat progress found")
+	}
+	testWorkflowFn := func(ctx workflow.Context) error {
+		ao := workflow.ActivityOptions{
+			StartToCloseTimeout: time.Second * 1000,
+			HeartbeatTimeout:    time.Second * 3,
+		}
+		ctx = workflow.WithActivityOptions(ctx, ao)
+		err := workflow.ExecuteActivity(ctx, activityWithHB).Get(ctx, nil)
+		return err
+	}
+	worker1.RegisterWorkflow(testWorkflowFn)
+	worker1.RegisterActivity(activityWithHB)
+	worker1.Start()
+
+	// Start a workflow
+	startTime := time.Now()
+	workflowID := "integration-activity-heartbeat-workflow-failover-test"
+	run1, err := client1.ExecuteWorkflow(host.NewContext(), sdkclient.StartWorkflowOptions{
+		ID:                 workflowID,
+		TaskQueue:          taskqueue,
+		WorkflowRunTimeout: time.Second * 300,
+	}, testWorkflowFn)
+
+	s.NoError(err)
+	s.NotEmpty(run1.GetRunID())
+
+	s.logger.Info("StartWorkflowExecution", tag.WorkflowRunID(run1.GetRunID()))
+	time.Sleep(time.Second * 4) // wait for heartbeat from activity to be reported and activity timed out on heartbeat
+
+	worker1.Stop() // stop worker1 so cluster 1 won't make any progress
+	s.failover(namespace, clusterName[1], int64(2), s.cluster1.GetFrontendClient())
+
+	// verify things are replicated over
+	resp, err := s.cluster1.GetHistoryClient().GetReplicationStatus(context.Background(), &historyservice.GetReplicationStatusRequest{})
+	s.NoError(err)
+	s.Equal(1, len(resp.Shards)) // test cluster has only one history shard
+	shard := resp.Shards[0]
+	s.True(shard.MaxReplicationTaskId > 0)
+	s.NotNil(shard.ShardLocalTime)
+	s.True(shard.ShardLocalTime.Before(time.Now()))
+	s.True(shard.ShardLocalTime.After(startTime))
+	s.NotNil(shard.RemoteClusters)
+	standbyAckInfo, ok := shard.RemoteClusters[clusterName[1]]
+	s.True(ok)
+	s.Equal(shard.MaxReplicationTaskId, standbyAckInfo.AckedTaskId)
+	s.NotNil(standbyAckInfo.AckedTaskVisibilityTime)
+	s.True(standbyAckInfo.AckedTaskVisibilityTime.Before(time.Now()))
+	s.True(standbyAckInfo.AckedTaskVisibilityTime.After(startTime))
+
+	// Make sure the heartbeat details are sent to cluster2 even when the activity at cluster1
+	// has heartbeat timeout. Also make sure the information is recorded when the activity state
+	// is "Scheduled"
+	dweResponse, err := client2.DescribeWorkflowExecution(host.NewContext(), workflowID, "")
+	s.NoError(err)
+	pendingActivities := dweResponse.GetPendingActivities()
+	s.Equal(1, len(pendingActivities))
+	s.Equal(enumspb.PENDING_ACTIVITY_STATE_SCHEDULED, pendingActivities[0].GetState())
+	heartbeatPayload := pendingActivities[0].GetHeartbeatDetails()
+	var heartbeatValue int
+	s.NoError(payloads.Decode(heartbeatPayload, &heartbeatValue))
+	s.Equal(expectedHeartbeatValue, heartbeatValue)
+	s.Equal(enumspb.TIMEOUT_TYPE_HEARTBEAT, pendingActivities[0].GetLastFailure().GetTimeoutFailureInfo().GetTimeoutType())
+	s.Equal("worker1", pendingActivities[0].GetLastWorkerIdentity())
+
+	// start worker2
+	worker2.RegisterWorkflow(testWorkflowFn)
+	worker2.RegisterActivity(activityWithHB)
+	worker2.Start()
+	defer worker2.Stop()
+
+	// ExecuteWorkflow return existing running workflow if it already started
+	run2, err := client2.ExecuteWorkflow(host.NewContext(), sdkclient.StartWorkflowOptions{
+		ID:                 workflowID,
+		TaskQueue:          taskqueue,
+		WorkflowRunTimeout: time.Second * 300,
+	}, testWorkflowFn)
+	s.NoError(err)
+	// verify we get the same execution as in cluster1
+	s.Equal(run1.GetRunID(), run2.GetRunID())
+
+	err = run2.Get(host.NewContext(), nil)
+	s.NoError(err) // workflow succeed
+	s.Equal(2, lastAttemptCount)
+}
+
 func (s *integrationClustersTestSuite) getHistory(client host.FrontendClient, namespace string, execution *commonpb.WorkflowExecution) []*historypb.HistoryEvent {
 	historyResponse, err := client.GetWorkflowExecutionHistory(host.NewContext(), &workflowservice.GetWorkflowExecutionHistoryRequest{
 		Namespace:       namespace,
@@ -2080,3 +1995,41 @@ func (s *integrationClustersTestSuite) failover(
 	// wait till failover completed
 	time.Sleep(cacheRefreshInterval)
 }
+
+func (s *integrationClustersTestSuite) registerNamespace(namespace string) {
+	client1 := s.cluster1.GetFrontendClient() // active
+	regReq := &workflowservice.RegisterNamespaceRequest{
+		Namespace:                        namespace,
+		IsGlobalNamespace:                true,
+		Clusters:                         clusterReplicationConfig,
+		ActiveClusterName:                clusterName[0],
+		WorkflowExecutionRetentionPeriod: timestamp.DurationPtr(1 * time.Hour * 24),
+	}
+	_, err := client1.RegisterNamespace(host.NewContext(), regReq)
+	s.NoError(err)
+	// Wait for namespace cache to pick the change
+	time.Sleep(cacheRefreshInterval)
+
+	descReq := &workflowservice.DescribeNamespaceRequest{
+		Namespace: namespace,
+	}
+	resp, err := client1.DescribeNamespace(host.NewContext(), descReq)
+	s.NoError(err)
+	s.NotNil(resp)
+	s.Equal(namespace, resp.NamespaceInfo.Name)
+	s.Equal(true, resp.IsGlobalNamespace)
+}
+
+func (s *integrationClustersTestSuite) newClientAndWorker(hostport, namespace, taskqueue, identity string) (sdkclient.Client, sdkworker.Worker) {
+	sdkClient1, err := sdkclient.NewClient(sdkclient.Options{
+		HostPort:  hostport,
+		Namespace: namespace,
+	})
+	s.NoError(err)
+
+	worker1 := sdkworker.New(sdkClient1, taskqueue, sdkworker.Options{
+		Identity: identity,
+	})
+
+	return sdkClient1, worker1
+}
