@@ -31,11 +31,14 @@ import (
 	"go.temporal.io/api/workflowservice/v1"
 	healthpb "google.golang.org/grpc/health/grpc_health_v1"
 
+	"go.temporal.io/server/client"
 	"go.temporal.io/server/common"
+	"go.temporal.io/server/common/clock"
+	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/metrics"
-	"go.temporal.io/server/common/resource"
+	"go.temporal.io/server/common/namespace"
 )
 
 var _ Handler = (*DCRedirectionHandlerImpl)(nil)
@@ -43,13 +46,15 @@ var _ Handler = (*DCRedirectionHandlerImpl)(nil)
 type (
 	// DCRedirectionHandlerImpl is simple wrapper over frontend service, doing redirection based on policy
 	DCRedirectionHandlerImpl struct {
-		resource.Resource
-
 		currentClusterName string
 		config             *Config
 		redirectionPolicy  DCRedirectionPolicy
 		tokenSerializer    common.TaskTokenSerializer
 		frontendHandler    Handler
+		logger             log.Logger
+		clientBean         client.Bean
+		metricsClient      metrics.Client
+		timeSource         clock.TimeSource
 	}
 )
 
@@ -57,22 +62,30 @@ type (
 func NewDCRedirectionHandler(
 	wfHandler Handler,
 	policy config.DCRedirectionPolicy,
+	logger log.Logger,
+	clientBean client.Bean,
+	metricsClient metrics.Client,
+	timeSource clock.TimeSource,
+	namespaceRegistry namespace.Registry,
+	clusterMetadata cluster.Metadata,
 ) *DCRedirectionHandlerImpl {
-	resource := wfHandler.GetResource()
 	dcRedirectionPolicy := RedirectionPolicyGenerator(
-		resource.GetClusterMetadata(),
+		clusterMetadata,
 		wfHandler.GetConfig(),
-		resource.GetNamespaceCache(),
+		namespaceRegistry,
 		policy,
 	)
 
 	return &DCRedirectionHandlerImpl{
-		Resource:           resource,
-		currentClusterName: resource.GetClusterMetadata().GetCurrentClusterName(),
+		currentClusterName: clusterMetadata.GetCurrentClusterName(),
 		config:             wfHandler.GetConfig(),
 		redirectionPolicy:  dcRedirectionPolicy,
 		tokenSerializer:    common.NewProtoTaskTokenSerializer(),
 		frontendHandler:    wfHandler,
+		logger:             logger,
+		clientBean:         clientBean,
+		metricsClient:      metricsClient,
+		timeSource:         timeSource,
 	}
 }
 
@@ -86,11 +99,6 @@ func (handler *DCRedirectionHandlerImpl) Stop() {
 	handler.frontendHandler.Stop()
 }
 
-// GetResource return resource
-func (handler *DCRedirectionHandlerImpl) GetResource() resource.Resource {
-	return handler.Resource
-}
-
 // GetConfig return config
 func (handler *DCRedirectionHandlerImpl) GetConfig() *Config {
 	return handler.frontendHandler.GetConfig()
@@ -210,13 +218,13 @@ func (handler *DCRedirectionHandlerImpl) DescribeTaskQueue(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.DescribeTaskQueue(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.DescribeTaskQueue(ctx, request)
 		}
 		return err
@@ -240,13 +248,13 @@ func (handler *DCRedirectionHandlerImpl) DescribeWorkflowExecution(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.DescribeWorkflowExecution(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.DescribeWorkflowExecution(ctx, request)
 		}
 		return err
@@ -270,13 +278,13 @@ func (handler *DCRedirectionHandlerImpl) GetWorkflowExecutionHistory(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.GetWorkflowExecutionHistory(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.GetWorkflowExecutionHistory(ctx, request)
 		}
 		return err
@@ -300,13 +308,13 @@ func (handler *DCRedirectionHandlerImpl) ListArchivedWorkflowExecutions(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.ListArchivedWorkflowExecutions(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.ListArchivedWorkflowExecutions(ctx, request)
 		}
 		return err
@@ -330,13 +338,13 @@ func (handler *DCRedirectionHandlerImpl) ListClosedWorkflowExecutions(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.ListClosedWorkflowExecutions(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.ListClosedWorkflowExecutions(ctx, request)
 		}
 		return err
@@ -360,13 +368,13 @@ func (handler *DCRedirectionHandlerImpl) ListOpenWorkflowExecutions(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.ListOpenWorkflowExecutions(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.ListOpenWorkflowExecutions(ctx, request)
 		}
 		return err
@@ -390,13 +398,13 @@ func (handler *DCRedirectionHandlerImpl) ListWorkflowExecutions(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.ListWorkflowExecutions(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.ListWorkflowExecutions(ctx, request)
 		}
 		return err
@@ -419,13 +427,13 @@ func (handler *DCRedirectionHandlerImpl) ScanWorkflowExecutions(
 	defer func() {
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.ScanWorkflowExecutions(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.ScanWorkflowExecutions(ctx, request)
 		}
 		return err
@@ -449,13 +457,13 @@ func (handler *DCRedirectionHandlerImpl) CountWorkflowExecutions(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.CountWorkflowExecutions(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.CountWorkflowExecutions(ctx, request)
 		}
 		return err
@@ -495,13 +503,13 @@ func (handler *DCRedirectionHandlerImpl) PollActivityTaskQueue(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.PollActivityTaskQueue(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.PollActivityTaskQueue(ctx, request)
 		}
 		return err
@@ -525,13 +533,13 @@ func (handler *DCRedirectionHandlerImpl) PollWorkflowTaskQueue(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.PollWorkflowTaskQueue(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.PollWorkflowTaskQueue(ctx, request)
 		}
 		return err
@@ -554,13 +562,13 @@ func (handler *DCRedirectionHandlerImpl) QueryWorkflow(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.QueryWorkflow(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.QueryWorkflow(ctx, request)
 		}
 		return err
@@ -589,13 +597,13 @@ func (handler *DCRedirectionHandlerImpl) RecordActivityTaskHeartbeat(
 		return nil, err
 	}
 
-	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, token.GetNamespaceId(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, namespace.ID(token.GetNamespaceId()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RecordActivityTaskHeartbeat(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RecordActivityTaskHeartbeat(ctx, request)
 		}
 		return err
@@ -619,13 +627,13 @@ func (handler *DCRedirectionHandlerImpl) RecordActivityTaskHeartbeatById(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RecordActivityTaskHeartbeatById(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RecordActivityTaskHeartbeatById(ctx, request)
 		}
 		return err
@@ -649,13 +657,13 @@ func (handler *DCRedirectionHandlerImpl) RequestCancelWorkflowExecution(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RequestCancelWorkflowExecution(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RequestCancelWorkflowExecution(ctx, request)
 		}
 		return err
@@ -679,13 +687,13 @@ func (handler *DCRedirectionHandlerImpl) ResetStickyTaskQueue(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.ResetStickyTaskQueue(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.ResetStickyTaskQueue(ctx, request)
 		}
 		return err
@@ -709,13 +717,13 @@ func (handler *DCRedirectionHandlerImpl) ResetWorkflowExecution(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.ResetWorkflowExecution(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.ResetWorkflowExecution(ctx, request)
 		}
 		return err
@@ -744,13 +752,13 @@ func (handler *DCRedirectionHandlerImpl) RespondActivityTaskCanceled(
 		return resp, err
 	}
 
-	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, token.GetNamespaceId(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, namespace.ID(token.GetNamespaceId()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RespondActivityTaskCanceled(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RespondActivityTaskCanceled(ctx, request)
 		}
 		return err
@@ -774,13 +782,13 @@ func (handler *DCRedirectionHandlerImpl) RespondActivityTaskCanceledById(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RespondActivityTaskCanceledById(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RespondActivityTaskCanceledById(ctx, request)
 		}
 		return err
@@ -809,13 +817,13 @@ func (handler *DCRedirectionHandlerImpl) RespondActivityTaskCompleted(
 		return resp, err
 	}
 
-	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, token.GetNamespaceId(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, namespace.ID(token.GetNamespaceId()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RespondActivityTaskCompleted(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RespondActivityTaskCompleted(ctx, request)
 		}
 		return err
@@ -839,13 +847,13 @@ func (handler *DCRedirectionHandlerImpl) RespondActivityTaskCompletedById(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RespondActivityTaskCompletedById(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RespondActivityTaskCompletedById(ctx, request)
 		}
 		return err
@@ -874,13 +882,13 @@ func (handler *DCRedirectionHandlerImpl) RespondActivityTaskFailed(
 		return resp, err
 	}
 
-	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, token.GetNamespaceId(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, namespace.ID(token.GetNamespaceId()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RespondActivityTaskFailed(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RespondActivityTaskFailed(ctx, request)
 		}
 		return err
@@ -904,13 +912,13 @@ func (handler *DCRedirectionHandlerImpl) RespondActivityTaskFailedById(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RespondActivityTaskFailedById(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RespondActivityTaskFailedById(ctx, request)
 		}
 		return err
@@ -939,13 +947,13 @@ func (handler *DCRedirectionHandlerImpl) RespondWorkflowTaskCompleted(
 		return nil, err
 	}
 
-	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, token.GetNamespaceId(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, namespace.ID(token.GetNamespaceId()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RespondWorkflowTaskCompleted(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RespondWorkflowTaskCompleted(ctx, request)
 		}
 		return err
@@ -974,13 +982,13 @@ func (handler *DCRedirectionHandlerImpl) RespondWorkflowTaskFailed(
 		return resp, err
 	}
 
-	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, token.GetNamespaceId(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, namespace.ID(token.GetNamespaceId()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RespondWorkflowTaskFailed(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RespondWorkflowTaskFailed(ctx, request)
 		}
 		return err
@@ -1009,13 +1017,13 @@ func (handler *DCRedirectionHandlerImpl) RespondQueryTaskCompleted(
 		return resp, err
 	}
 
-	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, token.GetNamespaceId(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceIDRedirect(ctx, namespace.ID(token.GetNamespaceId()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.RespondQueryTaskCompleted(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.RespondQueryTaskCompleted(ctx, request)
 		}
 		return err
@@ -1039,13 +1047,13 @@ func (handler *DCRedirectionHandlerImpl) SignalWithStartWorkflowExecution(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.SignalWithStartWorkflowExecution(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.SignalWithStartWorkflowExecution(ctx, request)
 		}
 		return err
@@ -1069,13 +1077,13 @@ func (handler *DCRedirectionHandlerImpl) SignalWorkflowExecution(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.SignalWorkflowExecution(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.SignalWorkflowExecution(ctx, request)
 		}
 		return err
@@ -1098,13 +1106,13 @@ func (handler *DCRedirectionHandlerImpl) StartWorkflowExecution(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.StartWorkflowExecution(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.StartWorkflowExecution(ctx, request)
 		}
 		return err
@@ -1128,13 +1136,13 @@ func (handler *DCRedirectionHandlerImpl) TerminateWorkflowExecution(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.TerminateWorkflowExecution(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.TerminateWorkflowExecution(ctx, request)
 		}
 		return err
@@ -1158,13 +1166,13 @@ func (handler *DCRedirectionHandlerImpl) ListTaskQueuePartitions(
 		handler.afterCall(scope, startTime, cluster, &retError)
 	}()
 
-	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, request.GetNamespace(), apiName, func(targetDC string) error {
+	err = handler.redirectionPolicy.WithNamespaceRedirect(ctx, namespace.Name(request.GetNamespace()), apiName, func(targetDC string) error {
 		cluster = targetDC
 		switch {
 		case targetDC == handler.currentClusterName:
 			resp, err = handler.frontendHandler.ListTaskQueuePartitions(ctx, request)
 		default:
-			remoteClient := handler.GetRemoteFrontendClient(targetDC)
+			remoteClient := handler.clientBean.GetRemoteFrontendClient(targetDC)
 			resp, err = remoteClient.ListTaskQueuePartitions(ctx, request)
 		}
 		return err
@@ -1185,7 +1193,7 @@ func (handler *DCRedirectionHandlerImpl) beforeCall(
 	scope int,
 ) (metrics.Scope, time.Time) {
 
-	return handler.GetMetricsClient().Scope(scope), handler.GetTimeSource().Now()
+	return handler.metricsClient.Scope(scope), handler.timeSource.Now()
 }
 
 func (handler *DCRedirectionHandlerImpl) afterCall(
@@ -1195,11 +1203,11 @@ func (handler *DCRedirectionHandlerImpl) afterCall(
 	retError *error,
 ) {
 
-	log.CapturePanic(handler.GetLogger(), retError)
+	log.CapturePanic(handler.logger, retError)
 
 	scope = scope.Tagged(metrics.TargetClusterTag(cluster))
 	scope.IncCounter(metrics.ClientRedirectionRequests)
-	scope.RecordTimer(metrics.ClientRedirectionLatency, handler.GetTimeSource().Now().Sub(startTime))
+	scope.RecordTimer(metrics.ClientRedirectionLatency, handler.timeSource.Now().Sub(startTime))
 	if *retError != nil {
 		scope.IncCounter(metrics.ClientRedirectionFailures)
 	}
