@@ -38,11 +38,11 @@ import (
 	"go.temporal.io/api/serviceerror"
 
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/persistence/versionhistory"
 	"go.temporal.io/server/service/history/shard"
@@ -136,34 +136,34 @@ type (
 
 		checkWorkflowExists(
 			ctx context.Context,
-			namespaceID string,
+			namespaceID namespace.ID,
 			workflowID string,
 			runID string,
 		) (bool, error)
 		getCurrentWorkflowRunID(
 			ctx context.Context,
-			namespaceID string,
+			namespaceID namespace.ID,
 			workflowID string,
 		) (string, error)
 		loadNDCWorkflow(
 			ctx context.Context,
-			namespaceID string,
+			namespaceID namespace.ID,
 			workflowID string,
 			runID string,
 		) (nDCWorkflow, error)
 	}
 
 	nDCTransactionMgrImpl struct {
-		shard            shard.Context
-		namespaceCache   cache.NamespaceCache
-		historyCache     *workflow.Cache
-		clusterMetadata  cluster.Metadata
-		executionManager persistence.ExecutionManager
-		serializer       serialization.Serializer
-		metricsClient    metrics.Client
-		workflowResetter workflowResetter
-		eventsReapplier  nDCEventsReapplier
-		logger           log.Logger
+		shard             shard.Context
+		namespaceRegistry namespace.Registry
+		historyCache      workflow.Cache
+		clusterMetadata   cluster.Metadata
+		executionManager  persistence.ExecutionManager
+		serializer        serialization.Serializer
+		metricsClient     metrics.Client
+		workflowResetter  workflowResetter
+		eventsReapplier   nDCEventsReapplier
+		logger            log.Logger
 
 		createMgr nDCTransactionMgrForNewWorkflow
 		updateMgr nDCTransactionMgrForExistingWorkflow
@@ -174,19 +174,19 @@ var _ nDCTransactionMgr = (*nDCTransactionMgrImpl)(nil)
 
 func newNDCTransactionMgr(
 	shard shard.Context,
-	historyCache *workflow.Cache,
+	historyCache workflow.Cache,
 	eventsReapplier nDCEventsReapplier,
 	logger log.Logger,
 ) *nDCTransactionMgrImpl {
 
 	transactionMgr := &nDCTransactionMgrImpl{
-		shard:            shard,
-		namespaceCache:   shard.GetNamespaceCache(),
-		historyCache:     historyCache,
-		clusterMetadata:  shard.GetClusterMetadata(),
-		executionManager: shard.GetExecutionManager(),
-		serializer:       shard.GetService().GetPayloadSerializer(),
-		metricsClient:    shard.GetMetricsClient(),
+		shard:             shard,
+		namespaceRegistry: shard.GetNamespaceRegistry(),
+		historyCache:      historyCache,
+		clusterMetadata:   shard.GetClusterMetadata(),
+		executionManager:  shard.GetExecutionManager(),
+		serializer:        shard.GetPayloadSerializer(),
+		metricsClient:     shard.GetMetricsClient(),
 		workflowResetter: newWorkflowResetter(
 			shard,
 			historyCache,
@@ -286,7 +286,7 @@ func (r *nDCTransactionMgrImpl) backfillWorkflowEventsReapply(
 	}
 	isWorkflowRunning := targetWorkflow.getMutableState().IsWorkflowExecutionRunning()
 	targetWorkflowActiveCluster := r.clusterMetadata.ClusterNameForFailoverVersion(
-		targetWorkflow.getMutableState().GetNamespaceEntry().GetFailoverVersion(),
+		targetWorkflow.getMutableState().GetNamespaceEntry().FailoverVersion(),
 	)
 	currentCluster := r.clusterMetadata.GetCurrentClusterName()
 	isActiveCluster := targetWorkflowActiveCluster == currentCluster
@@ -317,7 +317,7 @@ func (r *nDCTransactionMgrImpl) backfillWorkflowEventsReapply(
 		// need to reset target workflow (which is also the current workflow)
 		// to accept events to be reapplied
 		baseMutableState := targetWorkflow.getMutableState()
-		namespaceID := baseMutableState.GetExecutionInfo().NamespaceId
+		namespaceID := namespace.ID(baseMutableState.GetExecutionInfo().NamespaceId)
 		workflowID := baseMutableState.GetExecutionInfo().WorkflowId
 		baseRunID := baseMutableState.GetExecutionState().GetRunId()
 		resetRunID := uuid.New()
@@ -327,7 +327,7 @@ func (r *nDCTransactionMgrImpl) backfillWorkflowEventsReapply(
 		//  since cannot reapply event to a finished workflow which had no workflow task started
 		if baseRebuildLastEventID == common.EmptyEventID {
 			r.logger.Warn("cannot reapply event to a finished workflow",
-				tag.WorkflowNamespaceID(namespaceID),
+				tag.WorkflowNamespaceID(namespaceID.String()),
 				tag.WorkflowID(workflowID),
 			)
 			r.metricsClient.IncCounter(metrics.HistoryReapplyEventsScope, metrics.EventReapplySkippedCount)
@@ -385,7 +385,7 @@ func (r *nDCTransactionMgrImpl) backfillWorkflowEventsReapply(
 
 func (r *nDCTransactionMgrImpl) checkWorkflowExists(
 	_ context.Context,
-	namespaceID string,
+	namespaceID namespace.ID,
 	workflowID string,
 	runID string,
 ) (bool, error) {
@@ -393,11 +393,9 @@ func (r *nDCTransactionMgrImpl) checkWorkflowExists(
 	_, err := r.shard.GetExecutionManager().GetWorkflowExecution(
 		&persistence.GetWorkflowExecutionRequest{
 			ShardID:     r.shard.GetShardID(),
-			NamespaceID: namespaceID,
-			Execution: commonpb.WorkflowExecution{
-				WorkflowId: workflowID,
-				RunId:      runID,
-			},
+			NamespaceID: namespaceID.String(),
+			WorkflowID:  workflowID,
+			RunID:       runID,
 		},
 	)
 
@@ -413,14 +411,14 @@ func (r *nDCTransactionMgrImpl) checkWorkflowExists(
 
 func (r *nDCTransactionMgrImpl) getCurrentWorkflowRunID(
 	_ context.Context,
-	namespaceID string,
+	namespaceID namespace.ID,
 	workflowID string,
 ) (string, error) {
 
 	resp, err := r.shard.GetExecutionManager().GetCurrentExecution(
 		&persistence.GetCurrentExecutionRequest{
 			ShardID:     r.shard.GetShardID(),
-			NamespaceID: namespaceID,
+			NamespaceID: namespaceID.String(),
 			WorkflowID:  workflowID,
 		},
 	)
@@ -437,7 +435,7 @@ func (r *nDCTransactionMgrImpl) getCurrentWorkflowRunID(
 
 func (r *nDCTransactionMgrImpl) loadNDCWorkflow(
 	ctx context.Context,
-	namespaceID string,
+	namespaceID namespace.ID,
 	workflowID string,
 	runID string,
 ) (nDCWorkflow, error) {
@@ -462,7 +460,7 @@ func (r *nDCTransactionMgrImpl) loadNDCWorkflow(
 		release(err)
 		return nil, err
 	}
-	return newNDCWorkflow(ctx, r.namespaceCache, r.clusterMetadata, weContext, msBuilder, release), nil
+	return newNDCWorkflow(ctx, r.namespaceRegistry, r.clusterMetadata, weContext, msBuilder, release), nil
 }
 
 func (r *nDCTransactionMgrImpl) isWorkflowCurrent(
@@ -479,7 +477,7 @@ func (r *nDCTransactionMgrImpl) isWorkflowCurrent(
 	// target workflow is not guaranteed to be current workflow, do additional check
 	executionInfo := targetWorkflow.getMutableState().GetExecutionInfo()
 	executionState := targetWorkflow.getMutableState().GetExecutionState()
-	namespaceID := executionInfo.NamespaceId
+	namespaceID := namespace.ID(executionInfo.NamespaceId)
 	workflowID := executionInfo.WorkflowId
 	runID := executionState.RunId
 
