@@ -33,7 +33,6 @@ import (
 	"github.com/pborman/uuid"
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
-	"github.com/uber-go/tally"
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	historypb "go.temporal.io/api/history/v1"
@@ -52,9 +51,9 @@ import (
 	"go.temporal.io/server/api/historyservicemock/v1"
 	replicationspb "go.temporal.io/server/api/replication/v1"
 	"go.temporal.io/server/client"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/resource"
 )
@@ -68,7 +67,7 @@ type (
 		mockResource                *resource.Test
 		mockShard                   *shard.ContextTest
 		mockEngine                  *shard.MockEngine
-		mockNamespaceCache          *cache.MockNamespaceCache
+		mockNamespaceCache          *namespace.MockRegistry
 		mockClientBean              *client.MockBean
 		mockAdminClient             *adminservicemock.MockAdminServiceClient
 		mockClusterMetadata         *cluster.MockMetadata
@@ -78,6 +77,7 @@ type (
 
 		mockExecutionManager *persistence.MockExecutionManager
 
+		shardID     int32
 		config      *configs.Config
 		requestChan chan *replicationTaskRequest
 
@@ -91,7 +91,7 @@ func TestReplicationTaskProcessorSuite(t *testing.T) {
 }
 
 func (s *replicationTaskProcessorSuite) SetupSuite() {
-
+	rand.Seed(time.Now().UnixNano())
 }
 
 func (s *replicationTaskProcessorSuite) TearDownSuite() {
@@ -105,11 +105,12 @@ func (s *replicationTaskProcessorSuite) SetupTest() {
 	s.config = tests.NewDynamicConfig()
 	s.requestChan = make(chan *replicationTaskRequest, 10)
 
+	s.shardID = rand.Int31()
 	s.mockShard = shard.NewTestContext(
 		s.controller,
 		&persistence.ShardInfoWithFailover{
 			ShardInfo: &persistencespb.ShardInfo{
-				ShardId:          0,
+				ShardId:          s.shardID,
 				RangeId:          1,
 				TransferAckLevel: 0,
 				ClusterReplicationLevel: map[string]int64{
@@ -129,7 +130,7 @@ func (s *replicationTaskProcessorSuite) SetupTest() {
 	s.mockReplicationTaskExecutor = NewMockreplicationTaskExecutor(s.controller)
 	s.mockHistoryClient = historyservicemock.NewMockHistoryServiceClient(s.controller)
 	s.mockReplicationTaskFetcher = NewMockReplicationTaskFetcher(s.controller)
-	rateLimiter := quotas.NewDefaultOutgoingDynamicRateLimiter(
+	rateLimiter := quotas.NewDefaultOutgoingRateLimiter(
 		func() float64 { return 100 },
 	)
 	s.mockReplicationTaskFetcher.EXPECT().GetSourceCluster().Return(cluster.TestAlternativeClusterName).AnyTimes()
@@ -138,7 +139,7 @@ func (s *replicationTaskProcessorSuite) SetupTest() {
 	s.mockClusterMetadata.EXPECT().GetCurrentClusterName().Return(cluster.TestCurrentClusterName).AnyTimes()
 	s.mockClusterMetadata.EXPECT().GetAllClusterInfo().Return(cluster.TestAllClusterInfo).AnyTimes()
 
-	metricsClient := metrics.NewClient(tally.NoopScope, metrics.History)
+	metricsClient := metrics.NewNoopMetricsClient()
 
 	s.replicationTaskProcessor = NewReplicationTaskProcessor(
 		s.mockShard,
@@ -152,6 +153,7 @@ func (s *replicationTaskProcessorSuite) SetupTest() {
 
 func (s *replicationTaskProcessorSuite) TearDownTest() {
 	s.controller.Finish()
+	s.mockShard.StopForTest()
 }
 
 func (s *replicationTaskProcessorSuite) TestHandleSyncShardStatus_Stale() {
@@ -166,7 +168,7 @@ func (s *replicationTaskProcessorSuite) TestHandleSyncShardStatus_Success() {
 	now := timestamp.TimeNowPtrUtc()
 	s.mockEngine.EXPECT().SyncShardStatus(gomock.Any(), &historyservice.SyncShardStatusRequest{
 		SourceCluster: cluster.TestAlternativeClusterName,
-		ShardId:       0,
+		ShardId:       s.shardID,
 		StatusTime:    now,
 	}).Return(nil)
 
@@ -240,7 +242,7 @@ func (s *replicationTaskProcessorSuite) TestHandleReplicationDLQTask_SyncActivit
 	workflowID := uuid.New()
 	runID := uuid.NewRandom().String()
 	request := &persistence.PutReplicationTaskToDLQRequest{
-		ShardID:           s.mockShard.GetShardID(),
+		ShardID:           s.shardID,
 		SourceClusterName: cluster.TestAlternativeClusterName,
 		TaskInfo: &persistencespb.ReplicationTaskInfo{
 			NamespaceId: namespaceID,
@@ -261,7 +263,7 @@ func (s *replicationTaskProcessorSuite) TestHandleReplicationDLQTask_History() {
 	runID := uuid.NewRandom().String()
 
 	request := &persistence.PutReplicationTaskToDLQRequest{
-		ShardID:           s.mockShard.GetShardID(),
+		ShardID:           s.shardID,
 		SourceClusterName: cluster.TestAlternativeClusterName,
 		TaskInfo: &persistencespb.ReplicationTaskInfo{
 			NamespaceId:  namespaceID,
@@ -293,7 +295,7 @@ func (s *replicationTaskProcessorSuite) TestConvertTaskToDLQTask_SyncActivity()
 		}},
 	}
 	request := &persistence.PutReplicationTaskToDLQRequest{
-		ShardID:           s.mockShard.GetShardID(),
+		ShardID:           s.shardID,
 		SourceClusterName: cluster.TestAlternativeClusterName,
 		TaskInfo: &persistencespb.ReplicationTaskInfo{
 			NamespaceId: namespaceID,
@@ -340,7 +342,7 @@ func (s *replicationTaskProcessorSuite) TestConvertTaskToDLQTask_History() {
 		},
 	}
 	request := &persistence.PutReplicationTaskToDLQRequest{
-		ShardID:           s.mockShard.GetShardID(),
+		ShardID:           s.shardID,
 		SourceClusterName: cluster.TestAlternativeClusterName,
 		TaskInfo: &persistencespb.ReplicationTaskInfo{
 			NamespaceId:  namespaceID,
@@ -361,7 +363,7 @@ func (s *replicationTaskProcessorSuite) TestConvertTaskToDLQTask_History() {
 func (s *replicationTaskProcessorSuite) TestCleanupReplicationTask_Noop() {
 	ackedTaskID := int64(12345)
 	s.mockResource.ShardMgr.EXPECT().UpdateShard(gomock.Any()).Return(nil)
-	err := s.mockShard.UpdateClusterReplicationLevel(cluster.TestAlternativeClusterName, ackedTaskID)
+	err := s.mockShard.UpdateClusterReplicationLevel(cluster.TestAlternativeClusterName, ackedTaskID, time.Time{})
 	s.NoError(err)
 
 	s.replicationTaskProcessor.minTxAckedTaskID = ackedTaskID
@@ -372,12 +374,12 @@ func (s *replicationTaskProcessorSuite) TestCleanupReplicationTask_Noop() {
 func (s *replicationTaskProcessorSuite) TestCleanupReplicationTask_Cleanup() {
 	ackedTaskID := int64(12345)
 	s.mockResource.ShardMgr.EXPECT().UpdateShard(gomock.Any()).Return(nil)
-	err := s.mockShard.UpdateClusterReplicationLevel(cluster.TestAlternativeClusterName, ackedTaskID)
+	err := s.mockShard.UpdateClusterReplicationLevel(cluster.TestAlternativeClusterName, ackedTaskID, time.Time{})
 	s.NoError(err)
 
 	s.replicationTaskProcessor.minTxAckedTaskID = ackedTaskID - 1
 	s.mockExecutionManager.EXPECT().RangeCompleteReplicationTask(&persistence.RangeCompleteReplicationTaskRequest{
-		ShardID:            s.mockShard.GetShardID(),
+		ShardID:            s.shardID,
 		InclusiveEndTaskID: ackedTaskID,
 	}).Return(nil)
 	err = s.replicationTaskProcessor.cleanupReplicationTasks()
@@ -430,13 +432,15 @@ func (s *replicationTaskProcessorSuite) TestPaginationFn_Success_More() {
 	s.replicationTaskProcessor.rxTaskBackoff = rxTaskBackoff
 
 	requestToken := &replicationspb.ReplicationToken{
-		ShardId:                s.mockShard.GetShardID(),
-		LastProcessedMessageId: maxRxProcessedTaskID,
-		LastRetrievedMessageId: maxRxReceivedTaskID,
+		ShardId:                     s.shardID,
+		LastProcessedMessageId:      maxRxProcessedTaskID,
+		LastProcessedVisibilityTime: &time.Time{},
+		LastRetrievedMessageId:      maxRxReceivedTaskID,
 	}
 
 	go func() {
 		request := <-s.requestChan
+		defer close(request.respChan)
 		s.Equal(requestToken, request.token)
 		request.respChan <- &replicationspb.ReplicationMessages{
 			SyncShardStatus:        syncShardTask,
@@ -444,7 +448,6 @@ func (s *replicationTaskProcessorSuite) TestPaginationFn_Success_More() {
 			LastRetrievedMessageId: lastRetrievedMessageID,
 			HasMore:                true,
 		}
-		close(request.respChan)
 	}()
 
 	tasks, _, err := s.replicationTaskProcessor.paginationFn(nil)
@@ -502,13 +505,15 @@ func (s *replicationTaskProcessorSuite) TestPaginationFn_Success_NoMore() {
 	s.replicationTaskProcessor.rxTaskBackoff = rxTaskBackoff
 
 	requestToken := &replicationspb.ReplicationToken{
-		ShardId:                s.mockShard.GetShardID(),
-		LastProcessedMessageId: maxRxProcessedTaskID,
-		LastRetrievedMessageId: maxRxReceivedTaskID,
+		ShardId:                     s.shardID,
+		LastProcessedMessageId:      maxRxProcessedTaskID,
+		LastProcessedVisibilityTime: &time.Time{},
+		LastRetrievedMessageId:      maxRxReceivedTaskID,
 	}
 
 	go func() {
 		request := <-s.requestChan
+		defer close(request.respChan)
 		s.Equal(requestToken, request.token)
 		request.respChan <- &replicationspb.ReplicationMessages{
 			SyncShardStatus:        syncShardTask,
@@ -516,7 +521,6 @@ func (s *replicationTaskProcessorSuite) TestPaginationFn_Success_NoMore() {
 			LastRetrievedMessageId: lastRetrievedMessageID,
 			HasMore:                false,
 		}
-		close(request.respChan)
 	}()
 
 	tasks, _, err := s.replicationTaskProcessor.paginationFn(nil)
@@ -538,15 +542,16 @@ func (s *replicationTaskProcessorSuite) TestPaginationFn_Error() {
 	s.replicationTaskProcessor.rxTaskBackoff = rxTaskBackoff
 
 	requestToken := &replicationspb.ReplicationToken{
-		ShardId:                s.mockShard.GetShardID(),
-		LastProcessedMessageId: maxRxProcessedTaskID,
-		LastRetrievedMessageId: maxRxReceivedTaskID,
+		ShardId:                     s.shardID,
+		LastProcessedMessageId:      maxRxProcessedTaskID,
+		LastProcessedVisibilityTime: &time.Time{},
+		LastRetrievedMessageId:      maxRxReceivedTaskID,
 	}
 
 	go func() {
 		request := <-s.requestChan
+		defer close(request.respChan)
 		s.Equal(requestToken, request.token)
-		close(request.respChan)
 	}()
 
 	tasks, _, err := s.replicationTaskProcessor.paginationFn(nil)
