@@ -41,15 +41,17 @@ import (
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	replicationspb "go.temporal.io/server/api/replication/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/convert"
+	"go.temporal.io/server/common/definition"
 	"go.temporal.io/server/common/failure"
 	"go.temporal.io/server/common/log"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/payloads"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 	"go.temporal.io/server/service/history/tests"
 	"go.temporal.io/server/service/history/workflow"
 )
@@ -61,7 +63,7 @@ type (
 
 		controller          *gomock.Controller
 		mockShard           *shard.ContextTest
-		mockNamespaceCache  *cache.MockNamespaceCache
+		mockNamespaceCache  *namespace.MockRegistry
 		mockMutableState    *workflow.MockMutableState
 		mockClusterMetadata *cluster.MockMetadata
 
@@ -119,14 +121,15 @@ func (s *replicatorQueueProcessorSuite) SetupTest() {
 
 func (s *replicatorQueueProcessorSuite) TearDownTest() {
 	s.controller.Finish()
+	s.mockShard.StopForTest()
 }
 
 func (s *replicatorQueueProcessorSuite) TestNotifyNewTasks_NotInitialized() {
 	s.replicatorQueueProcessor.maxTaskID = nil
 
-	s.replicatorQueueProcessor.NotifyNewTasks([]persistence.Task{
-		&persistence.HistoryReplicationTask{TaskID: 456},
-		&persistence.HistoryReplicationTask{TaskID: 123},
+	s.replicatorQueueProcessor.NotifyNewTasks([]tasks.Task{
+		&tasks.HistoryReplicationTask{TaskID: 456},
+		&tasks.HistoryReplicationTask{TaskID: 123},
 	})
 
 	s.Equal(*s.replicatorQueueProcessor.maxTaskID, int64(456))
@@ -135,13 +138,13 @@ func (s *replicatorQueueProcessorSuite) TestNotifyNewTasks_NotInitialized() {
 func (s *replicatorQueueProcessorSuite) TestNotifyNewTasks_Initialized() {
 	s.replicatorQueueProcessor.maxTaskID = convert.Int64Ptr(123)
 
-	s.replicatorQueueProcessor.NotifyNewTasks([]persistence.Task{
-		&persistence.HistoryReplicationTask{TaskID: 100},
+	s.replicatorQueueProcessor.NotifyNewTasks([]tasks.Task{
+		&tasks.HistoryReplicationTask{TaskID: 100},
 	})
 	s.Equal(*s.replicatorQueueProcessor.maxTaskID, int64(123))
 
-	s.replicatorQueueProcessor.NotifyNewTasks([]persistence.Task{
-		&persistence.HistoryReplicationTask{TaskID: 234},
+	s.replicatorQueueProcessor.NotifyNewTasks([]tasks.Task{
+		&tasks.HistoryReplicationTask{TaskID: 234},
 	})
 	s.Equal(*s.replicatorQueueProcessor.maxTaskID, int64(234))
 }
@@ -206,30 +209,32 @@ func (s *replicatorQueueProcessorSuite) TestTaskIDRange_Initialized_UseHighestTr
 
 func (s *replicatorQueueProcessorSuite) TestSyncActivity_WorkflowMissing() {
 	ctx := context.Background()
-	namespace := "some random namespace name"
+	namespaceName := namespace.Name("some random namespace name")
 	namespaceID := tests.NamespaceID
 	workflowID := "some random workflow ID"
 	runID := uuid.New()
 	scheduleID := int64(144)
+	version := int64(288)
 	taskID := int64(1444)
-	task := &persistencespb.ReplicationTaskInfo{
-		TaskType:    enumsspb.TASK_TYPE_REPLICATION_SYNC_ACTIVITY,
-		TaskId:      taskID,
-		NamespaceId: namespaceID,
-		WorkflowId:  workflowID,
-		RunId:       runID,
-		ScheduledId: scheduleID,
+	task := &tasks.SyncActivityTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			namespaceID.String(),
+			workflowID,
+			runID,
+		),
+		VisibilityTimestamp: time.Now().UTC(),
+		TaskID:              taskID,
+		Version:             version,
+		ScheduledID:         scheduleID,
 	}
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(&persistence.GetWorkflowExecutionRequest{
 		ShardID:     s.mockShard.GetShardID(),
-		NamespaceID: namespaceID,
-		Execution: commonpb.WorkflowExecution{
-			WorkflowId: workflowID,
-			RunId:      runID,
-		},
+		NamespaceID: namespaceID.String(),
+		WorkflowID:  workflowID,
+		RunID:       runID,
 	}).Return(nil, serviceerror.NewNotFound(""))
-	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Id: namespaceID, Name: namespace},
+	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Id: namespaceID.String(), Name: namespaceName.String()},
 		&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -239,7 +244,6 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_WorkflowMissing() {
 			},
 		},
 		1234,
-		nil,
 	), nil).AnyTimes()
 
 	result, err := s.replicatorQueueProcessor.generateSyncActivityTask(ctx, task)
@@ -249,20 +253,23 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_WorkflowMissing() {
 
 func (s *replicatorQueueProcessorSuite) TestSyncActivity_WorkflowCompleted() {
 	ctx := context.Background()
-	namespace := "some random namespace name"
+	namespaceName := namespace.Name("some random namespace name")
 	namespaceID := tests.NamespaceID
 	workflowID := "some random workflow ID"
 	runID := uuid.New()
 	scheduleID := int64(144)
 	taskID := int64(1444)
 	version := int64(2333)
-	task := &persistencespb.ReplicationTaskInfo{
-		TaskType:    enumsspb.TASK_TYPE_REPLICATION_SYNC_ACTIVITY,
-		TaskId:      taskID,
-		NamespaceId: namespaceID,
-		WorkflowId:  workflowID,
-		RunId:       runID,
-		ScheduledId: scheduleID,
+	task := &tasks.SyncActivityTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			namespaceID.String(),
+			workflowID,
+			runID,
+		),
+		VisibilityTimestamp: time.Now().UTC(),
+		TaskID:              taskID,
+		Version:             version,
+		ScheduledID:         scheduleID,
 	}
 
 	context, release, _ := s.replicatorQueueProcessor.historyCache.GetOrCreateWorkflowExecution(
@@ -278,8 +285,8 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_WorkflowCompleted() {
 	release(nil)
 	s.mockMutableState.EXPECT().StartTransaction(gomock.Any()).Return(false, nil)
 	s.mockMutableState.EXPECT().IsWorkflowExecutionRunning().Return(false).AnyTimes()
-	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Id: namespaceID, Name: namespace},
+	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Id: namespaceID.String(), Name: namespaceName.String()},
 		&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -289,7 +296,6 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_WorkflowCompleted() {
 			},
 		},
 		version,
-		nil,
 	), nil).AnyTimes()
 
 	result, err := s.replicatorQueueProcessor.generateSyncActivityTask(ctx, task)
@@ -299,20 +305,23 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_WorkflowCompleted() {
 
 func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityCompleted() {
 	ctx := context.Background()
-	namespace := "some random namespace name"
+	namespaceName := namespace.Name("some random namespace name")
 	namespaceID := tests.NamespaceID
 	workflowID := "some random workflow ID"
 	runID := uuid.New()
 	scheduleID := int64(144)
 	taskID := int64(1444)
 	version := int64(2333)
-	task := &persistencespb.ReplicationTaskInfo{
-		TaskType:    enumsspb.TASK_TYPE_REPLICATION_SYNC_ACTIVITY,
-		TaskId:      taskID,
-		NamespaceId: namespaceID,
-		WorkflowId:  workflowID,
-		RunId:       runID,
-		ScheduledId: scheduleID,
+	task := &tasks.SyncActivityTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			namespaceID.String(),
+			workflowID,
+			runID,
+		),
+		VisibilityTimestamp: time.Now().UTC(),
+		TaskID:              taskID,
+		Version:             version,
+		ScheduledID:         scheduleID,
 	}
 
 	context, release, _ := s.replicatorQueueProcessor.historyCache.GetOrCreateWorkflowExecution(
@@ -330,8 +339,8 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityCompleted() {
 	s.mockMutableState.EXPECT().StartTransaction(gomock.Any()).Return(false, nil)
 	s.mockMutableState.EXPECT().IsWorkflowExecutionRunning().Return(true).AnyTimes()
 	s.mockMutableState.EXPECT().GetActivityInfo(scheduleID).Return(nil, false).AnyTimes()
-	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Id: namespaceID, Name: namespace},
+	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Id: namespaceID.String(), Name: namespaceName.String()},
 		&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -341,7 +350,6 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityCompleted() {
 			},
 		},
 		version,
-		nil,
 	), nil).AnyTimes()
 
 	result, err := s.replicatorQueueProcessor.generateSyncActivityTask(ctx, task)
@@ -351,20 +359,24 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityCompleted() {
 
 func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityRetry() {
 	ctx := context.Background()
-	namespace := "some random namespace name"
+	namespaceName := namespace.Name("some random namespace name")
 	namespaceID := tests.NamespaceID
 	workflowID := "some random workflow ID"
 	runID := uuid.New()
 	scheduleID := int64(144)
 	taskID := int64(1444)
 	version := int64(2333)
-	task := &persistencespb.ReplicationTaskInfo{
-		TaskType:    enumsspb.TASK_TYPE_REPLICATION_SYNC_ACTIVITY,
-		TaskId:      taskID,
-		NamespaceId: namespaceID,
-		WorkflowId:  workflowID,
-		RunId:       runID,
-		ScheduledId: scheduleID,
+	taskTimestamp := time.Now().UTC()
+	task := &tasks.SyncActivityTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			namespaceID.String(),
+			workflowID,
+			runID,
+		),
+		VisibilityTimestamp: taskTimestamp,
+		TaskID:              taskID,
+		Version:             version,
+		ScheduledID:         scheduleID,
 	}
 
 	context, release, _ := s.replicatorQueueProcessor.historyCache.GetOrCreateWorkflowExecution(
@@ -418,8 +430,8 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityRetry() {
 		},
 	}
 	s.mockMutableState.EXPECT().GetExecutionInfo().Return(&persistencespb.WorkflowExecutionInfo{VersionHistories: versionHistories}).AnyTimes()
-	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Id: namespaceID, Name: namespace},
+	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Id: namespaceID.String(), Name: namespaceName.String()},
 		&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -429,7 +441,6 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityRetry() {
 			},
 		},
 		version,
-		nil,
 	), nil).AnyTimes()
 
 	result, err := s.replicatorQueueProcessor.generateSyncActivityTask(ctx, task)
@@ -439,7 +450,7 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityRetry() {
 		TaskType:     enumsspb.REPLICATION_TASK_TYPE_SYNC_ACTIVITY_TASK,
 		Attributes: &replicationspb.ReplicationTask_SyncActivityTaskAttributes{
 			SyncActivityTaskAttributes: &replicationspb.SyncActivityTaskAttributes{
-				NamespaceId:        namespaceID,
+				NamespaceId:        namespaceID.String(),
 				WorkflowId:         workflowID,
 				RunId:              runID,
 				Version:            activityVersion,
@@ -455,25 +466,30 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityRetry() {
 				VersionHistory:     versionHistory,
 			},
 		},
+		VisibilityTime: timestamp.TimePtr(taskTimestamp),
 	}, result)
 }
 
 func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityRunning() {
 	ctx := context.Background()
-	namespace := "some random namespace name"
+	namespaceName := namespace.Name("some random namespace name")
 	namespaceID := tests.NamespaceID
 	workflowID := "some random workflow ID"
 	runID := uuid.New()
 	scheduleID := int64(144)
 	taskID := int64(1444)
 	version := int64(2333)
-	task := &persistencespb.ReplicationTaskInfo{
-		TaskType:    enumsspb.TASK_TYPE_REPLICATION_SYNC_ACTIVITY,
-		TaskId:      taskID,
-		NamespaceId: namespaceID,
-		WorkflowId:  workflowID,
-		RunId:       runID,
-		ScheduledId: scheduleID,
+	taskTimestamp := time.Now().UTC()
+	task := &tasks.SyncActivityTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			namespaceID.String(),
+			workflowID,
+			runID,
+		),
+		VisibilityTimestamp: taskTimestamp,
+		TaskID:              taskID,
+		Version:             version,
+		ScheduledID:         scheduleID,
 	}
 
 	context, release, _ := s.replicatorQueueProcessor.historyCache.GetOrCreateWorkflowExecution(
@@ -529,8 +545,8 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityRunning() {
 		},
 	}
 	s.mockMutableState.EXPECT().GetExecutionInfo().Return(&persistencespb.WorkflowExecutionInfo{VersionHistories: versionHistories}).AnyTimes()
-	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Id: namespaceID, Name: namespace},
+	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Id: namespaceID.String(), Name: namespaceName.String()},
 		&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -540,7 +556,6 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityRunning() {
 			},
 		},
 		version,
-		nil,
 	), nil).AnyTimes()
 
 	result, err := s.replicatorQueueProcessor.generateSyncActivityTask(ctx, task)
@@ -550,7 +565,7 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityRunning() {
 		TaskType:     enumsspb.REPLICATION_TASK_TYPE_SYNC_ACTIVITY_TASK,
 		Attributes: &replicationspb.ReplicationTask_SyncActivityTaskAttributes{
 			SyncActivityTaskAttributes: &replicationspb.SyncActivityTaskAttributes{
-				NamespaceId:        namespaceID,
+				NamespaceId:        namespaceID.String(),
 				WorkflowId:         workflowID,
 				RunId:              runID,
 				Version:            activityVersion,
@@ -566,5 +581,6 @@ func (s *replicatorQueueProcessorSuite) TestSyncActivity_ActivityRunning() {
 				VersionHistory:     versionHistory,
 			},
 		},
+		VisibilityTime: timestamp.TimePtr(taskTimestamp),
 	}, result)
 }
