@@ -25,8 +25,10 @@
 package shard
 
 import (
+	"context"
 	"errors"
 	"fmt"
+	"strconv"
 	"sync"
 	"testing"
 	"time"
@@ -71,9 +73,10 @@ type (
 		mockShardManager  *persistence.MockShardManager
 		mockEngineFactory *MockEngineFactory
 
-		config          *configs.Config
-		logger          log.Logger
-		shardController *ControllerImpl
+		config               *configs.Config
+		logger               log.Logger
+		shardController      *ControllerImpl
+		mockHostInfoProvider *resource.MockHostInfoProvider
 	}
 )
 
@@ -94,17 +97,49 @@ func (s *controllerSuite) SetupTest() {
 	s.mockServiceResolver = s.mockResource.HistoryServiceResolver
 	s.mockClusterMetadata = s.mockResource.ClusterMetadata
 	s.hostInfo = s.mockResource.GetHostInfo()
+	s.mockHostInfoProvider = resource.NewMockHostInfoProvider(s.controller)
+	s.mockHostInfoProvider.EXPECT().HostInfo().Return(s.hostInfo).AnyTimes()
 
 	s.logger = s.mockResource.Logger
 	s.config = tests.NewDynamicConfig()
 
-	s.shardController = NewController(s.mockResource, s.mockEngineFactory, s.config)
+	s.shardController = NewController(
+		s.mockEngineFactory,
+		s.config,
+		s.mockResource.Logger,
+		s.mockResource.GetThrottledLogger(),
+		s.mockResource.GetExecutionManager(),
+		s.mockResource.GetShardManager(),
+		s.mockResource.GetClientBean(),
+		s.mockResource.GetHistoryClient(),
+		s.mockResource.GetHistoryServiceResolver(),
+		s.mockResource.GetMetricsClient(),
+		s.mockResource.GetPayloadSerializer(),
+		s.mockResource.GetTimeSource(),
+		s.mockResource.GetNamespaceRegistry(),
+		s.mockResource.GetSearchAttributesProvider(),
+		s.mockResource.GetSearchAttributesMapper(),
+		s.mockResource.GetClusterMetadata(),
+		s.mockResource.GetArchivalMetadata(),
+		s.mockHostInfoProvider,
+	)
 }
 
 func (s *controllerSuite) TearDownTest() {
 	s.controller.Finish()
 }
 
+type getOrCreateShardRequestMatcher int32
+
+func (s getOrCreateShardRequestMatcher) Matches(x interface{}) bool {
+	req, ok := x.(*persistence.GetOrCreateShardRequest)
+	return ok && req.ShardID == int32(s)
+}
+
+func (s getOrCreateShardRequestMatcher) String() string {
+	return strconv.Itoa(int(s))
+}
+
 func (s *controllerSuite) TestAcquireShardSuccess() {
 	numShards := int32(10)
 	s.config.NumberOfShards = numShards
@@ -123,8 +158,8 @@ func (s *controllerSuite) TestAcquireShardSuccess() {
 			s.mockHistoryEngine.EXPECT().Start().Return()
 			s.mockServiceResolver.EXPECT().Lookup(convert.Int32ToString(shardID)).Return(s.hostInfo, nil).Times(2)
 			s.mockEngineFactory.EXPECT().CreateEngine(gomock.Any()).Return(s.mockHistoryEngine)
-			s.mockShardManager.EXPECT().GetShard(&persistence.GetShardRequest{ShardID: shardID}).Return(
-				&persistence.GetShardResponse{
+			s.mockShardManager.EXPECT().GetOrCreateShard(getOrCreateShardRequestMatcher(shardID)).Return(
+				&persistence.GetOrCreateShardResponse{
 					ShardInfo: &persistencespb.ShardInfo{
 						ShardId:             shardID,
 						Owner:               s.hostInfo.Identity(),
@@ -177,8 +212,10 @@ func (s *controllerSuite) TestAcquireShardSuccess() {
 	s.mockClusterMetadata.EXPECT().GetAllClusterInfo().Return(cluster.TestSingleDCClusterInfo).AnyTimes()
 	s.shardController.acquireShards()
 	count := 0
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
+	defer cancel()
 	for _, shardID := range myShards {
-		s.NotNil(s.shardController.GetEngineForShard(shardID))
+		s.NotNil(s.shardController.GetEngineForShard(ctx, shardID))
 		count++
 	}
 	s.Equal(2, count)
@@ -205,8 +242,8 @@ func (s *controllerSuite) TestAcquireShardsConcurrently() {
 			s.mockHistoryEngine.EXPECT().Start().Return()
 			s.mockServiceResolver.EXPECT().Lookup(convert.Int32ToString(shardID)).Return(s.hostInfo, nil).Times(2)
 			s.mockEngineFactory.EXPECT().CreateEngine(gomock.Any()).Return(s.mockHistoryEngine)
-			s.mockShardManager.EXPECT().GetShard(&persistence.GetShardRequest{ShardID: shardID}).Return(
-				&persistence.GetShardResponse{
+			s.mockShardManager.EXPECT().GetOrCreateShard(getOrCreateShardRequestMatcher(shardID)).Return(
+				&persistence.GetOrCreateShardResponse{
 					ShardInfo: &persistencespb.ShardInfo{
 						ShardId:             shardID,
 						Owner:               s.hostInfo.Identity(),
@@ -259,8 +296,10 @@ func (s *controllerSuite) TestAcquireShardsConcurrently() {
 	s.mockClusterMetadata.EXPECT().GetAllClusterInfo().Return(cluster.TestSingleDCClusterInfo).AnyTimes()
 	s.shardController.acquireShards()
 	count := 0
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
+	defer cancel()
 	for _, shardID := range myShards {
-		s.NotNil(s.shardController.GetEngineForShard(shardID))
+		s.NotNil(s.shardController.GetEngineForShard(ctx, shardID))
 		count++
 	}
 	s.Equal(2, count)
@@ -274,9 +313,11 @@ func (s *controllerSuite) TestAcquireShardLookupFailure() {
 	}
 
 	s.shardController.acquireShards()
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
+	defer cancel()
 	for shardID := int32(1); shardID <= numShards; shardID++ {
 		s.mockServiceResolver.EXPECT().Lookup(convert.Int32ToString(shardID)).Return(nil, errors.New("ring failure"))
-		s.Nil(s.shardController.GetEngineForShard(shardID))
+		s.Nil(s.shardController.GetEngineForShard(ctx, shardID))
 	}
 }
 
@@ -294,8 +335,8 @@ func (s *controllerSuite) TestAcquireShardRenewSuccess() {
 		s.mockHistoryEngine.EXPECT().Start().Return()
 		s.mockServiceResolver.EXPECT().Lookup(convert.Int32ToString(shardID)).Return(s.hostInfo, nil).Times(2)
 		s.mockEngineFactory.EXPECT().CreateEngine(gomock.Any()).Return(s.mockHistoryEngine)
-		s.mockShardManager.EXPECT().GetShard(&persistence.GetShardRequest{ShardID: shardID}).Return(
-			&persistence.GetShardResponse{
+		s.mockShardManager.EXPECT().GetOrCreateShard(getOrCreateShardRequestMatcher(shardID)).Return(
+			&persistence.GetOrCreateShardResponse{
 				ShardInfo: &persistencespb.ShardInfo{
 					ShardId:             shardID,
 					Owner:               s.hostInfo.Identity(),
@@ -349,8 +390,10 @@ func (s *controllerSuite) TestAcquireShardRenewSuccess() {
 	}
 	s.shardController.acquireShards()
 
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
+	defer cancel()
 	for shardID := int32(1); shardID <= numShards; shardID++ {
-		s.NotNil(s.shardController.GetEngineForShard(shardID))
+		s.NotNil(s.shardController.GetEngineForShard(ctx, shardID))
 	}
 }
 
@@ -368,8 +411,8 @@ func (s *controllerSuite) TestAcquireShardRenewLookupFailed() {
 		s.mockHistoryEngine.EXPECT().Start().Return()
 		s.mockServiceResolver.EXPECT().Lookup(convert.Int32ToString(shardID)).Return(s.hostInfo, nil).Times(2)
 		s.mockEngineFactory.EXPECT().CreateEngine(gomock.Any()).Return(s.mockHistoryEngine)
-		s.mockShardManager.EXPECT().GetShard(&persistence.GetShardRequest{ShardID: shardID}).Return(
-			&persistence.GetShardResponse{
+		s.mockShardManager.EXPECT().GetOrCreateShard(getOrCreateShardRequestMatcher(shardID)).Return(
+			&persistence.GetOrCreateShardResponse{
 				ShardInfo: &persistencespb.ShardInfo{
 					ShardId:             shardID,
 					Owner:               s.hostInfo.Identity(),
@@ -423,15 +466,36 @@ func (s *controllerSuite) TestAcquireShardRenewLookupFailed() {
 	}
 	s.shardController.acquireShards()
 
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
+	defer cancel()
 	for shardID := int32(1); shardID <= numShards; shardID++ {
-		s.NotNil(s.shardController.GetEngineForShard(shardID))
+		s.NotNil(s.shardController.GetEngineForShard(ctx, shardID))
 	}
 }
 
 func (s *controllerSuite) TestHistoryEngineClosed() {
 	numShards := int32(4)
 	s.config.NumberOfShards = numShards
-	s.shardController = NewController(s.mockResource, s.mockEngineFactory, s.config)
+	s.shardController = NewController(
+		s.mockEngineFactory,
+		s.config,
+		s.mockResource.Logger,
+		s.mockResource.GetThrottledLogger(),
+		s.mockResource.GetExecutionManager(),
+		s.mockResource.GetShardManager(),
+		s.mockResource.GetClientBean(),
+		s.mockResource.GetHistoryClient(),
+		s.mockResource.GetHistoryServiceResolver(),
+		s.mockResource.GetMetricsClient(),
+		s.mockResource.GetPayloadSerializer(),
+		s.mockResource.GetTimeSource(),
+		s.mockResource.GetNamespaceRegistry(),
+		s.mockResource.GetSearchAttributesProvider(),
+		s.mockResource.GetSearchAttributesMapper(),
+		s.mockResource.GetClusterMetadata(),
+		s.mockResource.GetArchivalMetadata(),
+		s.mockHostInfoProvider,
+	)
 	historyEngines := make(map[int32]*MockEngine)
 	for shardID := int32(1); shardID <= numShards; shardID++ {
 		mockEngine := NewMockEngine(s.controller)
@@ -439,6 +503,9 @@ func (s *controllerSuite) TestHistoryEngineClosed() {
 		s.setupMocksForAcquireShard(shardID, mockEngine, 5, 6)
 	}
 
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
+	defer cancel()
+
 	s.mockServiceResolver.EXPECT().AddListener(shardControllerMembershipUpdateListenerName,
 		gomock.Any()).Return(nil).AnyTimes()
 	// when shard is initialized, it will use the 2 mock function below to initialize the "current" time of each cluster
@@ -451,7 +518,7 @@ func (s *controllerSuite) TestHistoryEngineClosed() {
 		go func() {
 			for attempt := 0; attempt < 10; attempt++ {
 				for shardID := int32(1); shardID <= numShards; shardID++ {
-					engine, err := s.shardController.GetEngineForShard(shardID)
+					engine, err := s.shardController.GetEngineForShard(ctx, shardID)
 					s.Nil(err)
 					s.NotNil(engine)
 				}
@@ -467,7 +534,7 @@ func (s *controllerSuite) TestHistoryEngineClosed() {
 		mockEngine := historyEngines[shardID]
 		mockEngine.EXPECT().Stop().Return()
 		s.mockServiceResolver.EXPECT().Lookup(convert.Int32ToString(shardID)).Return(differentHostInfo, nil).AnyTimes()
-		s.shardController.shardClosedCallback(shardID, nil)
+		s.shardController.CloseShardByID(shardID)
 	}
 
 	for w := 0; w < 10; w++ {
@@ -475,7 +542,7 @@ func (s *controllerSuite) TestHistoryEngineClosed() {
 		go func() {
 			for attempt := 0; attempt < 10; attempt++ {
 				for shardID := int32(3); shardID <= numShards; shardID++ {
-					engine, err := s.shardController.GetEngineForShard(shardID)
+					engine, err := s.shardController.GetEngineForShard(ctx, shardID)
 					s.Nil(err)
 					s.NotNil(engine)
 					time.Sleep(20 * time.Millisecond)
@@ -491,7 +558,7 @@ func (s *controllerSuite) TestHistoryEngineClosed() {
 			shardLost := false
 			for attempt := 0; !shardLost && attempt < 10; attempt++ {
 				for shardID := int32(1); shardID <= 2; shardID++ {
-					_, err := s.shardController.GetEngineForShard(shardID)
+					_, err := s.shardController.GetEngineForShard(ctx, shardID)
 					if err != nil {
 						s.logger.Error("ShardLost", tag.Error(err))
 						shardLost = true
@@ -519,7 +586,27 @@ func (s *controllerSuite) TestHistoryEngineClosed() {
 func (s *controllerSuite) TestShardControllerClosed() {
 	numShards := int32(4)
 	s.config.NumberOfShards = numShards
-	s.shardController = NewController(s.mockResource, s.mockEngineFactory, s.config)
+	s.shardController = NewController(
+		s.mockEngineFactory,
+		s.config,
+		s.mockResource.Logger,
+		s.mockResource.GetThrottledLogger(),
+		s.mockResource.GetExecutionManager(),
+		s.mockResource.GetShardManager(),
+		s.mockResource.GetClientBean(),
+		s.mockResource.GetHistoryClient(),
+		s.mockResource.GetHistoryServiceResolver(),
+		s.mockResource.GetMetricsClient(),
+		s.mockResource.GetPayloadSerializer(),
+		s.mockResource.GetTimeSource(),
+		s.mockResource.GetNamespaceRegistry(),
+		s.mockResource.GetSearchAttributesProvider(),
+		s.mockResource.GetSearchAttributesMapper(),
+		s.mockResource.GetClusterMetadata(),
+		s.mockResource.GetArchivalMetadata(),
+		s.mockHostInfoProvider,
+	)
+
 	historyEngines := make(map[int32]*MockEngine)
 	for shardID := int32(1); shardID <= numShards; shardID++ {
 		mockEngine := NewMockEngine(s.controller)
@@ -527,6 +614,9 @@ func (s *controllerSuite) TestShardControllerClosed() {
 		s.setupMocksForAcquireShard(shardID, mockEngine, 5, 6)
 	}
 
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
+	defer cancel()
+
 	s.mockServiceResolver.EXPECT().AddListener(shardControllerMembershipUpdateListenerName, gomock.Any()).Return(nil).AnyTimes()
 	// when shard is initialized, it will use the 2 mock function below to initialize the "current" time of each cluster
 	s.mockClusterMetadata.EXPECT().GetCurrentClusterName().Return(cluster.TestCurrentClusterName).AnyTimes()
@@ -540,7 +630,7 @@ func (s *controllerSuite) TestShardControllerClosed() {
 			shardLost := false
 			for attempt := 0; !shardLost && attempt < 10; attempt++ {
 				for shardID := int32(1); shardID <= numShards; shardID++ {
-					_, err := s.shardController.GetEngineForShard(shardID)
+					_, err := s.shardController.GetEngineForShard(ctx, shardID)
 					if err != nil {
 						s.logger.Error("ShardLost", tag.Error(err))
 						shardLost = true
@@ -577,8 +667,8 @@ func (s *controllerSuite) setupMocksForAcquireShard(shardID int32, mockEngine *M
 	mockEngine.EXPECT().Start()
 	s.mockServiceResolver.EXPECT().Lookup(convert.Int32ToString(shardID)).Return(s.hostInfo, nil).Times(2)
 	s.mockEngineFactory.EXPECT().CreateEngine(newContextMatcher(shardID)).Return(mockEngine)
-	s.mockShardManager.EXPECT().GetShard(&persistence.GetShardRequest{ShardID: shardID}).Return(
-		&persistence.GetShardResponse{
+	s.mockShardManager.EXPECT().GetOrCreateShard(getOrCreateShardRequestMatcher(shardID)).Return(
+		&persistence.GetOrCreateShardResponse{
 			ShardInfo: &persistencespb.ShardInfo{
 				ShardId:             shardID,
 				Owner:               s.hostInfo.Identity(),
