@@ -28,7 +28,8 @@ import (
 	"context"
 	"fmt"
 
-	commandpb "go.temporal.io/api/command/v1"
+	"github.com/pborman/uuid"
+
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	"go.temporal.io/api/serviceerror"
@@ -36,16 +37,17 @@ import (
 
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	"go.temporal.io/server/api/matchingservice/v1"
-	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/backoff"
 	"go.temporal.io/server/common/failure"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/service/history/configs"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 	"go.temporal.io/server/service/history/workflow"
 )
 
@@ -54,6 +56,7 @@ type (
 		*timerQueueTaskExecutorBase
 
 		queueProcessor *timerQueueActiveProcessorImpl
+		matchingClient matchingservice.MatchingServiceClient
 	}
 )
 
@@ -64,6 +67,7 @@ func newTimerQueueActiveTaskExecutor(
 	logger log.Logger,
 	metricsClient metrics.Client,
 	config *configs.Config,
+	matchingClient matchingservice.MatchingServiceClient,
 ) queueTaskExecutor {
 	return &timerQueueActiveTaskExecutor{
 		timerQueueTaskExecutorBase: newTimerQueueTaskExecutorBase(
@@ -74,38 +78,35 @@ func newTimerQueueActiveTaskExecutor(
 			config,
 		),
 		queueProcessor: queueProcessor,
+		matchingClient: matchingClient,
 	}
 }
 
 func (t *timerQueueActiveTaskExecutor) execute(
 	ctx context.Context,
-	taskInfo queueTaskInfo,
+	taskInfo tasks.Task,
 	shouldProcessTask bool,
 ) error {
-	timerTask, ok := taskInfo.(*persistencespb.TimerTaskInfo)
-	if !ok {
-		return errUnexpectedQueueTask
-	}
 
 	if !shouldProcessTask {
 		return nil
 	}
 
-	switch timerTask.TaskType {
-	case enumsspb.TASK_TYPE_USER_TIMER:
-		return t.executeUserTimerTimeoutTask(ctx, timerTask)
-	case enumsspb.TASK_TYPE_ACTIVITY_TIMEOUT:
-		return t.executeActivityTimeoutTask(ctx, timerTask)
-	case enumsspb.TASK_TYPE_WORKFLOW_TASK_TIMEOUT:
-		return t.executeWorkflowTaskTimeoutTask(ctx, timerTask)
-	case enumsspb.TASK_TYPE_WORKFLOW_RUN_TIMEOUT:
-		return t.executeWorkflowTimeoutTask(ctx, timerTask)
-	case enumsspb.TASK_TYPE_ACTIVITY_RETRY_TIMER:
-		return t.executeActivityRetryTimerTask(ctx, timerTask)
-	case enumsspb.TASK_TYPE_WORKFLOW_BACKOFF_TIMER:
-		return t.executeWorkflowBackoffTimerTask(ctx, timerTask)
-	case enumsspb.TASK_TYPE_DELETE_HISTORY_EVENT:
-		return t.executeDeleteHistoryEventTask(ctx, timerTask)
+	switch task := taskInfo.(type) {
+	case *tasks.UserTimerTask:
+		return t.executeUserTimerTimeoutTask(ctx, task)
+	case *tasks.ActivityTimeoutTask:
+		return t.executeActivityTimeoutTask(ctx, task)
+	case *tasks.WorkflowTaskTimeoutTask:
+		return t.executeWorkflowTaskTimeoutTask(ctx, task)
+	case *tasks.WorkflowTimeoutTask:
+		return t.executeWorkflowTimeoutTask(ctx, task)
+	case *tasks.ActivityRetryTimerTask:
+		return t.executeActivityRetryTimerTask(ctx, task)
+	case *tasks.WorkflowBackoffTimerTask:
+		return t.executeWorkflowBackoffTimerTask(ctx, task)
+	case *tasks.DeleteHistoryEventTask:
+		return t.executeDeleteHistoryEventTask(ctx, task)
 	default:
 		return errUnknownTimerTask
 	}
@@ -113,7 +114,7 @@ func (t *timerQueueActiveTaskExecutor) execute(
 
 func (t *timerQueueActiveTaskExecutor) executeUserTimerTimeoutTask(
 	ctx context.Context,
-	task *persistencespb.TimerTaskInfo,
+	task *tasks.UserTimerTask,
 ) (retError error) {
 	var cancel context.CancelFunc
 	ctx, cancel = context.WithTimeout(ctx, taskTimeout)
@@ -173,7 +174,7 @@ Loop:
 
 func (t *timerQueueActiveTaskExecutor) executeActivityTimeoutTask(
 	ctx context.Context,
-	task *persistencespb.TimerTaskInfo,
+	task *tasks.ActivityTimeoutTask,
 ) (retError error) {
 	var cancel context.CancelFunc
 	ctx, cancel = context.WithTimeout(ctx, taskTimeout)
@@ -210,9 +211,8 @@ func (t *timerQueueActiveTaskExecutor) executeActivityTimeoutTask(
 	// for updating workflow execution. In that case, only one new heartbeat timeout task should be
 	// created.
 	isHeartBeatTask := task.TimeoutType == enumspb.TIMEOUT_TYPE_HEARTBEAT
-	activityInfo, heartbeatTimeoutVis, ok := mutableState.GetActivityInfoWithTimerHeartbeat(task.GetEventId())
-	goVisibilityTS := timestamp.TimeValue(task.VisibilityTime)
-	if isHeartBeatTask && ok && (heartbeatTimeoutVis.Before(goVisibilityTS) || heartbeatTimeoutVis.Equal(goVisibilityTS)) {
+	activityInfo, heartbeatTimeoutVis, ok := mutableState.GetActivityInfoWithTimerHeartbeat(task.EventID)
+	if isHeartBeatTask && ok && (heartbeatTimeoutVis.Before(task.GetVisibilityTime()) || heartbeatTimeoutVis.Equal(task.GetVisibilityTime())) {
 		activityInfo.TimerTaskStatus = activityInfo.TimerTaskStatus &^ workflow.TimerTaskStatusCreatedHeartbeat
 		if err := mutableState.UpdateActivity(activityInfo); err != nil {
 			return err
@@ -259,7 +259,7 @@ Loop:
 		}
 
 		t.emitTimeoutMetricScopeWithNamespaceTag(
-			mutableState.GetExecutionInfo().NamespaceId,
+			namespace.ID(mutableState.GetExecutionInfo().NamespaceId),
 			metrics.TimerActiveTaskActivityTimeoutScope,
 			timerSequenceID.TimerType,
 		)
@@ -283,7 +283,7 @@ Loop:
 
 func (t *timerQueueActiveTaskExecutor) executeWorkflowTaskTimeoutTask(
 	ctx context.Context,
-	task *persistencespb.TimerTaskInfo,
+	task *tasks.WorkflowTaskTimeoutTask,
 ) (retError error) {
 	var cancel context.CancelFunc
 	ctx, cancel = context.WithTimeout(ctx, taskTimeout)
@@ -309,13 +309,11 @@ func (t *timerQueueActiveTaskExecutor) executeWorkflowTaskTimeoutTask(
 		return nil
 	}
 
-	scheduleID := task.GetEventId()
-	workflowTask, ok := mutableState.GetWorkflowTaskInfo(scheduleID)
+	workflowTask, ok := mutableState.GetWorkflowTaskInfo(task.EventID)
 	if !ok {
-		t.logger.Debug("Potentially duplicate task.", tag.TaskID(task.GetTaskId()), tag.WorkflowScheduleID(scheduleID), tag.TaskType(enumsspb.TASK_TYPE_WORKFLOW_TASK_TIMEOUT))
 		return nil
 	}
-	ok, err = verifyTaskVersion(t.shard, t.logger, task.GetNamespaceId(), workflowTask.Version, task.Version, task)
+	ok, err = verifyTaskVersion(t.shard, t.logger, namespace.ID(task.NamespaceID), workflowTask.Version, task.Version, task)
 	if err != nil || !ok {
 		return err
 	}
@@ -328,7 +326,7 @@ func (t *timerQueueActiveTaskExecutor) executeWorkflowTaskTimeoutTask(
 	switch task.TimeoutType {
 	case enumspb.TIMEOUT_TYPE_START_TO_CLOSE:
 		t.emitTimeoutMetricScopeWithNamespaceTag(
-			mutableState.GetExecutionInfo().NamespaceId,
+			namespace.ID(mutableState.GetExecutionInfo().NamespaceId),
 			metrics.TimerActiveTaskWorkflowTaskTimeoutScope,
 			enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
 		)
@@ -347,11 +345,11 @@ func (t *timerQueueActiveTaskExecutor) executeWorkflowTaskTimeoutTask(
 		}
 
 		t.emitTimeoutMetricScopeWithNamespaceTag(
-			mutableState.GetExecutionInfo().NamespaceId,
+			namespace.ID(mutableState.GetExecutionInfo().NamespaceId),
 			metrics.TimerActiveTaskWorkflowTaskTimeoutScope,
 			enumspb.TIMEOUT_TYPE_SCHEDULE_TO_START,
 		)
-		_, err := mutableState.AddWorkflowTaskScheduleToStartTimeoutEvent(scheduleID)
+		_, err := mutableState.AddWorkflowTaskScheduleToStartTimeoutEvent(task.EventID)
 		if err != nil {
 			return err
 		}
@@ -363,7 +361,7 @@ func (t *timerQueueActiveTaskExecutor) executeWorkflowTaskTimeoutTask(
 
 func (t *timerQueueActiveTaskExecutor) executeWorkflowBackoffTimerTask(
 	ctx context.Context,
-	task *persistencespb.TimerTaskInfo,
+	task *tasks.WorkflowBackoffTimerTask,
 ) (retError error) {
 	var cancel context.CancelFunc
 	ctx, cancel = context.WithTimeout(ctx, taskTimeout)
@@ -406,7 +404,7 @@ func (t *timerQueueActiveTaskExecutor) executeWorkflowBackoffTimerTask(
 
 func (t *timerQueueActiveTaskExecutor) executeActivityRetryTimerTask(
 	ctx context.Context,
-	task *persistencespb.TimerTaskInfo,
+	task *tasks.ActivityRetryTimerTask,
 ) (retError error) {
 	var cancel context.CancelFunc
 	ctx, cancel = context.WithTimeout(ctx, taskTimeout)
@@ -433,9 +431,8 @@ func (t *timerQueueActiveTaskExecutor) executeActivityRetryTimerTask(
 	}
 
 	// generate activity task
-	scheduledID := task.GetEventId()
-	activityInfo, ok := mutableState.GetActivityInfo(scheduledID)
-	if !ok || task.ScheduleAttempt < activityInfo.Attempt || activityInfo.StartedId != common.EmptyEventID {
+	activityInfo, ok := mutableState.GetActivityInfo(task.EventID)
+	if !ok || task.Attempt < activityInfo.Attempt || activityInfo.StartedId != common.EmptyEventID {
 		if ok {
 			t.logger.Info("Duplicate activity retry timer task",
 				tag.WorkflowID(mutableState.GetExecutionInfo().WorkflowId),
@@ -445,36 +442,16 @@ func (t *timerQueueActiveTaskExecutor) executeActivityRetryTimerTask(
 				tag.Attempt(activityInfo.Attempt),
 				tag.FailoverVersion(activityInfo.Version),
 				tag.TimerTaskStatus(activityInfo.TimerTaskStatus),
-				tag.ScheduleAttempt(task.ScheduleAttempt))
+				tag.ScheduleAttempt(task.Attempt))
 		}
 		return nil
 	}
-	ok, err = verifyTaskVersion(t.shard, t.logger, task.GetNamespaceId(), activityInfo.Version, task.Version, task)
+	ok, err = verifyTaskVersion(t.shard, t.logger, namespace.ID(task.NamespaceID), activityInfo.Version, task.Version, task)
 	if err != nil || !ok {
 		return err
 	}
 
-	targetNamespaceID := namespaceID
-	if activityInfo.NamespaceId != "" {
-		targetNamespaceID = activityInfo.NamespaceId
-	} else {
-		// TODO remove this block after Mar, 1th, 2020
-		//  previously, NamespaceID in activity info is not used, so need to get
-		//  schedule event from DB checking whether activity to be scheduled
-		//  belongs to this namespace
-		scheduledEvent, err := mutableState.GetActivityScheduledEvent(scheduledID)
-		if err != nil {
-			return err
-		}
-		if scheduledEvent.GetActivityTaskScheduledEventAttributes().GetNamespace() != "" {
-			namespaceEntry, err := t.shard.GetNamespaceCache().GetNamespace(scheduledEvent.GetActivityTaskScheduledEventAttributes().GetNamespace())
-			if err != nil {
-				return serviceerror.NewInternal("unable to re-schedule activity across namespace.")
-			}
-			targetNamespaceID = namespaceEntry.GetInfo().Id
-		}
-	}
-
+	targetNamespaceID := activityInfo.NamespaceId
 	taskQueue := &taskqueuepb.TaskQueue{
 		Name: activityInfo.TaskQueue,
 		Kind: enumspb.TASK_QUEUE_KIND_NORMAL,
@@ -486,12 +463,12 @@ func (t *timerQueueActiveTaskExecutor) executeActivityRetryTimerTask(
 
 	ctx, cancel = context.WithTimeout(context.Background(), transferActiveTaskDefaultTimeout)
 	defer cancel()
-	_, retError = t.shard.GetService().GetMatchingClient().AddActivityTask(ctx, &matchingservice.AddActivityTaskRequest{
+	_, retError = t.matchingClient.AddActivityTask(ctx, &matchingservice.AddActivityTaskRequest{
 		NamespaceId:            targetNamespaceID,
-		SourceNamespaceId:      namespaceID,
+		SourceNamespaceId:      namespaceID.String(),
 		Execution:              &execution,
 		TaskQueue:              taskQueue,
-		ScheduleId:             scheduledID,
+		ScheduleId:             task.EventID,
 		ScheduleToStartTimeout: timestamp.DurationPtr(scheduleToStartTimeout),
 	})
 
@@ -500,7 +477,7 @@ func (t *timerQueueActiveTaskExecutor) executeActivityRetryTimerTask(
 
 func (t *timerQueueActiveTaskExecutor) executeWorkflowTimeoutTask(
 	ctx context.Context,
-	task *persistencespb.TimerTaskInfo,
+	task *tasks.WorkflowTimeoutTask,
 ) (retError error) {
 	var cancel context.CancelFunc
 	ctx, cancel = context.WithTimeout(ctx, taskTimeout)
@@ -531,72 +508,74 @@ func (t *timerQueueActiveTaskExecutor) executeWorkflowTimeoutTask(
 	if err != nil {
 		return err
 	}
-	ok, err := verifyTaskVersion(t.shard, t.logger, task.GetNamespaceId(), startVersion, task.Version, task)
+	ok, err := verifyTaskVersion(t.shard, t.logger, namespace.ID(task.NamespaceID), startVersion, task.Version, task)
 	if err != nil || !ok {
 		return err
 	}
 
 	eventBatchFirstEventID := mutableState.GetNextEventID()
-
 	timeoutFailure := failure.NewTimeoutFailure("workflow timeout", enumspb.TIMEOUT_TYPE_START_TO_CLOSE)
 	backoffInterval := backoff.NoBackoff
 	retryState := enumspb.RETRY_STATE_TIMEOUT
-	continueAsNewInitiator := enumspb.CONTINUE_AS_NEW_INITIATOR_RETRY
+	initiator := enumspb.CONTINUE_AS_NEW_INITIATOR_UNSPECIFIED
 
 	wfExpTime := timestamp.TimeValue(mutableState.GetExecutionInfo().WorkflowExecutionExpirationTime)
-	// Retry if WorkflowExpirationTime is not set or workflow is not expired.
 	if wfExpTime.IsZero() || wfExpTime.After(t.shard.GetTimeSource().Now()) {
 		backoffInterval, retryState = mutableState.GetRetryBackoffDuration(timeoutFailure)
-
-		if backoffInterval == backoff.NoBackoff {
-			// Check if a cron backoff is needed.
-			backoffInterval, err = mutableState.GetCronBackoffDuration()
-			if err != nil {
-				return err
-			}
-			continueAsNewInitiator = enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE
+		if backoffInterval != backoff.NoBackoff {
+			// We have a retry policy and we should retry.
+			initiator = enumspb.CONTINUE_AS_NEW_INITIATOR_RETRY
+		} else if backoffInterval = mutableState.GetCronBackoffDuration(); backoffInterval != backoff.NoBackoff {
+			// We have a cron schedule.
+			initiator = enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE
 		}
 	}
 
-	// No more retries, or workflow is expired.
-	if backoffInterval == backoff.NoBackoff {
-		if err := workflow.TimeoutWorkflow(mutableState, eventBatchFirstEventID, retryState); err != nil {
-			return err
-		}
+	var newRunID string
+	if initiator != enumspb.CONTINUE_AS_NEW_INITIATOR_UNSPECIFIED {
+		newRunID = uuid.New()
+	}
 
+	// First add timeout workflow event, no matter what we're doing next.
+	if err := workflow.TimeoutWorkflow(
+		mutableState,
+		eventBatchFirstEventID,
+		retryState,
+		newRunID,
+	); err != nil {
+		return err
+	}
+
+	// No more retries, or workflow is expired.
+	if initiator == enumspb.CONTINUE_AS_NEW_INITIATOR_UNSPECIFIED {
 		// We apply the update to execution using optimistic concurrency.  If it fails due to a conflict than reload
 		// the history and try the operation again.
 		return t.updateWorkflowExecution(weContext, mutableState, false)
 	}
 
-	// workflow timeout, but a retry or cron is needed, so we do continue as new to retry or cron
 	startEvent, err := mutableState.GetStartEvent()
 	if err != nil {
 		return err
 	}
+	startAttr := startEvent.GetWorkflowExecutionStartedEventAttributes()
 
-	startAttributes := startEvent.GetWorkflowExecutionStartedEventAttributes()
-	continueAsNewAttributes := &commandpb.ContinueAsNewWorkflowExecutionCommandAttributes{
-		WorkflowType:         startAttributes.WorkflowType,
-		TaskQueue:            startAttributes.TaskQueue,
-		Input:                startAttributes.Input,
-		WorkflowRunTimeout:   startAttributes.WorkflowRunTimeout,
-		WorkflowTaskTimeout:  startAttributes.WorkflowTaskTimeout,
-		BackoffStartInterval: &backoffInterval,
-		RetryPolicy:          startAttributes.RetryPolicy,
-		Initiator:            continueAsNewInitiator,
-		Failure:              timeoutFailure,
-		LastCompletionResult: startAttributes.LastCompletionResult,
-		CronSchedule:         mutableState.GetExecutionInfo().CronSchedule,
-		Header:               startAttributes.Header,
-		Memo:                 startAttributes.Memo,
-		SearchAttributes:     startAttributes.SearchAttributes,
+	newMutableState, err := createMutableState(
+		t.shard,
+		mutableState.GetNamespaceEntry(),
+		newRunID,
+	)
+	if err != nil {
+		return err
 	}
-	newMutableState, err := workflow.RetryWorkflow(
+	err = workflow.SetupNewWorkflowForRetryOrCron(
 		mutableState,
-		eventBatchFirstEventID,
-		startAttributes.GetParentWorkflowNamespace(),
-		continueAsNewAttributes,
+		newMutableState,
+		newRunID,
+		startAttr,
+		startAttr.LastCompletionResult,
+		timeoutFailure,
+		backoffInterval,
+		initiator,
 	)
 	if err != nil {
 		return err
@@ -607,7 +586,7 @@ func (t *timerQueueActiveTaskExecutor) executeWorkflowTimeoutTask(
 	return weContext.UpdateWorkflowExecutionWithNewAsActive(
 		t.shard.GetTimeSource().Now(),
 		workflow.NewContext(
-			newExecutionInfo.NamespaceId,
+			namespace.ID(newExecutionInfo.NamespaceId),
 			commonpb.WorkflowExecution{
 				WorkflowId: newExecutionInfo.WorkflowId,
 				RunId:      newExecutionState.RunId,
@@ -657,15 +636,15 @@ func (t *timerQueueActiveTaskExecutor) updateWorkflowExecution(
 }
 
 func (t *timerQueueActiveTaskExecutor) emitTimeoutMetricScopeWithNamespaceTag(
-	namespaceID string,
+	namespaceID namespace.ID,
 	scope int,
 	timerType enumspb.TimeoutType,
 ) {
-	namespaceEntry, err := t.shard.GetNamespaceCache().GetNamespaceByID(namespaceID)
+	namespaceEntry, err := t.shard.GetNamespaceRegistry().GetNamespaceByID(namespaceID)
 	if err != nil {
 		return
 	}
-	metricsScope := t.metricsClient.Scope(scope).Tagged(metrics.NamespaceTag(namespaceEntry.GetInfo().Name))
+	metricsScope := t.metricsClient.Scope(scope).Tagged(metrics.NamespaceTag(namespaceEntry.Name().String()))
 	switch timerType {
 	case enumspb.TIMEOUT_TYPE_SCHEDULE_TO_START:
 		metricsScope.IncCounter(metrics.ScheduleToStartTimeoutCounter)
