@@ -28,15 +28,15 @@ import (
 	"context"
 
 	"go.temporal.io/server/api/matchingservice/v1"
-	persistencespb "go.temporal.io/server/api/persistence/v1"
-	"go.temporal.io/server/common/collection"
+	"go.temporal.io/server/client"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
-	"go.temporal.io/server/common/persistence/visibility"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/xdc"
 	"go.temporal.io/server/service/history/configs"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 type (
@@ -58,13 +58,12 @@ type (
 func newTransferQueueStandbyProcessor(
 	clusterName string,
 	shard shard.Context,
-	historyService *historyEngineImpl,
-	visibilityMgr visibility.VisibilityManager,
-	matchingClient matchingservice.MatchingServiceClient,
+	historyEngine *historyEngineImpl,
 	taskAllocator taskAllocator,
 	nDCHistoryResender xdc.NDCHistoryResender,
-	queueTaskProcessor queueTaskProcessor,
 	logger log.Logger,
+	clientBean client.Bean,
+	matchingClient matchingservice.MatchingServiceClient,
 ) *transferQueueStandbyProcessorImpl {
 
 	config := shard.GetConfig()
@@ -85,12 +84,8 @@ func newTransferQueueStandbyProcessor(
 	}
 	logger = log.With(logger, tag.ClusterName(clusterName))
 
-	transferTaskFilter := func(taskInfo queueTaskInfo) (bool, error) {
-		task, ok := taskInfo.(*persistencespb.TransferTaskInfo)
-		if !ok {
-			return false, errUnexpectedQueueTask
-		}
-		return taskAllocator.verifyStandbyTask(clusterName, task.GetNamespaceId(), task)
+	transferTaskFilter := func(task tasks.Task) (bool, error) {
+		return taskAllocator.verifyStandbyTask(clusterName, namespace.ID(task.GetNamespaceID()), task)
 	}
 	maxReadAckLevel := func() int64 {
 		return shard.GetTransferMaxReadLevel()
@@ -108,15 +103,17 @@ func newTransferQueueStandbyProcessor(
 		config:             shard.GetConfig(),
 		transferTaskFilter: transferTaskFilter,
 		logger:             logger,
-		metricsClient:      historyService.metricsClient,
+		metricsClient:      historyEngine.metricsClient,
 		taskExecutor: newTransferQueueStandbyTaskExecutor(
 			shard,
-			historyService,
+			historyEngine,
 			nDCHistoryResender,
 			logger,
-			historyService.metricsClient,
+			historyEngine.metricsClient,
 			clusterName,
 			config,
+			clientBean,
+			matchingClient,
 		),
 		transferQueueProcessorBase: newTransferQueueProcessorBase(
 			shard,
@@ -136,35 +133,13 @@ func newTransferQueueStandbyProcessor(
 		logger,
 	)
 
-	redispatchQueue := collection.NewConcurrentQueue()
-
-	transferQueueTaskInitializer := func(taskInfo queueTaskInfo) queueTask {
-		return newTransferQueueTask(
-			shard,
-			taskInfo,
-			historyService.metricsClient.Scope(
-				getTransferTaskMetricsScope(taskInfo.GetTaskType(), false),
-			),
-			initializeLoggerForTask(shard.GetShardID(), taskInfo, logger),
-			transferTaskFilter,
-			processor.taskExecutor,
-			redispatchQueue,
-			shard.GetTimeSource(),
-			options.MaxRetryCount,
-			queueAckMgr,
-		)
-	}
-
 	queueProcessorBase := newQueueProcessorBase(
 		clusterName,
 		shard,
 		options,
 		processor,
-		queueTaskProcessor,
 		queueAckMgr,
-		redispatchQueue,
-		historyService.historyCache,
-		transferQueueTaskInitializer,
+		historyEngine.historyCache,
 		logger,
 		shard.GetMetricsClient().Scope(metrics.TransferStandbyQueueProcessorScope),
 	)
@@ -187,7 +162,7 @@ func (t *transferQueueStandbyProcessorImpl) complete(
 	taskInfo *taskInfo,
 ) {
 
-	t.queueProcessorBase.complete(taskInfo.task)
+	t.queueProcessorBase.complete(taskInfo.Task)
 }
 
 func (t *transferQueueStandbyProcessorImpl) process(
@@ -195,6 +170,6 @@ func (t *transferQueueStandbyProcessorImpl) process(
 	taskInfo *taskInfo,
 ) (int, error) {
 	// TODO: task metricScope should be determined when creating taskInfo
-	metricScope := getTransferTaskMetricsScope(taskInfo.task.GetTaskType(), false)
-	return metricScope, t.taskExecutor.execute(ctx, taskInfo.task, taskInfo.shouldProcessTask)
+	metricScope := getTransferTaskMetricsScope(taskInfo.Task, false)
+	return metricScope, t.taskExecutor.execute(ctx, taskInfo.Task, taskInfo.shouldProcessTask)
 }
