@@ -37,7 +37,6 @@ import (
 	"go.temporal.io/api/serviceerror"
 
 	"go.temporal.io/server/api/adminservice/v1"
-	enumsspb "go.temporal.io/server/api/enums/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/clock"
@@ -45,11 +44,13 @@ import (
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/rpc"
 	"go.temporal.io/server/service/history/configs"
 	"go.temporal.io/server/service/history/consts"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 const (
@@ -65,8 +66,8 @@ type (
 	CallerType int
 
 	Context interface {
-		GetNamespace() string
-		GetNamespaceID() string
+		GetNamespace() namespace.Name
+		GetNamespaceID() namespace.ID
 		GetExecution() *commonpb.WorkflowExecution
 
 		LoadWorkflowExecution() (MutableState, error)
@@ -136,10 +137,9 @@ type (
 
 type (
 	ContextImpl struct {
-		namespaceID       string
+		namespaceID       namespace.ID
 		workflowExecution commonpb.WorkflowExecution
 		shard             shard.Context
-		engine            shard.Engine
 		logger            log.Logger
 		metricsClient     metrics.Client
 		timeSource        clock.TimeSource
@@ -155,11 +155,11 @@ type (
 var _ Context = (*ContextImpl)(nil)
 
 var (
-	PersistenceOperationRetryPolicy = common.CreatePersistanceRetryPolicy()
+	PersistenceOperationRetryPolicy = common.CreatePersistenceRetryPolicy()
 )
 
 func NewContext(
-	namespaceID string,
+	namespaceID namespace.ID,
 	execution commonpb.WorkflowExecution,
 	shard shard.Context,
 	logger log.Logger,
@@ -168,7 +168,6 @@ func NewContext(
 		namespaceID:       namespaceID,
 		workflowExecution: execution,
 		shard:             shard,
-		engine:            shard.GetEngine(),
 		logger:            logger,
 		metricsClient:     shard.GetMetricsClient(),
 		timeSource:        shard.GetTimeSource(),
@@ -210,13 +209,16 @@ func (c *ContextImpl) Unlock(
 
 func (c *ContextImpl) Clear() {
 	c.metricsClient.IncCounter(metrics.WorkflowContextScope, metrics.WorkflowContextCleared)
+	if c.MutableState != nil {
+		c.MutableState.GetQueryRegistry().Clear()
+	}
 	c.MutableState = nil
 	c.stats = &persistencespb.ExecutionStats{
 		HistorySize: 0,
 	}
 }
 
-func (c *ContextImpl) GetNamespaceID() string {
+func (c *ContextImpl) GetNamespaceID() namespace.ID {
 	return c.namespaceID
 }
 
@@ -224,12 +226,12 @@ func (c *ContextImpl) GetExecution() *commonpb.WorkflowExecution {
 	return &c.workflowExecution
 }
 
-func (c *ContextImpl) GetNamespace() string {
-	namespaceEntry, err := c.shard.GetNamespaceCache().GetNamespaceByID(c.namespaceID)
+func (c *ContextImpl) GetNamespace() namespace.Name {
+	namespaceEntry, err := c.shard.GetNamespaceRegistry().GetNamespaceByID(c.namespaceID)
 	if err != nil {
 		return ""
 	}
-	return namespaceEntry.GetInfo().Name
+	return namespaceEntry.Name()
 }
 
 func (c *ContextImpl) GetHistorySize() int64 {
@@ -252,7 +254,7 @@ func (c *ContextImpl) LoadWorkflowExecutionForReplication(
 	incomingVersion int64,
 ) (MutableState, error) {
 
-	namespaceEntry, err := c.shard.GetNamespaceCache().GetNamespaceByID(c.namespaceID)
+	namespaceEntry, err := c.shard.GetNamespaceRegistry().GetNamespaceByID(c.namespaceID)
 	if err != nil {
 		return nil, err
 	}
@@ -260,8 +262,9 @@ func (c *ContextImpl) LoadWorkflowExecutionForReplication(
 	if c.MutableState == nil {
 		response, err := getWorkflowExecutionWithRetry(c.shard, &persistence.GetWorkflowExecutionRequest{
 			ShardID:     c.shard.GetShardID(),
-			NamespaceID: c.namespaceID,
-			Execution:   c.workflowExecution,
+			NamespaceID: c.namespaceID.String(),
+			WorkflowID:  c.workflowExecution.GetWorkflowId(),
+			RunID:       c.workflowExecution.GetRunId(),
 		})
 		if err != nil {
 			return nil, err
@@ -280,20 +283,6 @@ func (c *ContextImpl) LoadWorkflowExecutionForReplication(
 		}
 
 		c.stats = response.State.ExecutionInfo.ExecutionStats
-		if c.stats == nil {
-			// TODO (alex): This check is for compatibility only and can be removed later
-			c.stats = &persistencespb.ExecutionStats{
-				HistorySize: response.State.ExecutionInfo.GetHistorySize(),
-			}
-		}
-
-		// finally emit execution and session stats
-		emitWorkflowExecutionStats(
-			c.metricsClient,
-			c.GetNamespace(),
-			response.MutableStateStats,
-			c.GetHistorySize(),
-		)
 	}
 
 	lastWriteVersion, err := c.MutableState.GetLastWriteVersion()
@@ -334,7 +323,7 @@ func (c *ContextImpl) LoadWorkflowExecutionForReplication(
 
 func (c *ContextImpl) LoadWorkflowExecution() (MutableState, error) {
 
-	namespaceEntry, err := c.shard.GetNamespaceCache().GetNamespaceByID(c.namespaceID)
+	namespaceEntry, err := c.shard.GetNamespaceRegistry().GetNamespaceByID(c.namespaceID)
 	if err != nil {
 		return nil, err
 	}
@@ -342,8 +331,9 @@ func (c *ContextImpl) LoadWorkflowExecution() (MutableState, error) {
 	if c.MutableState == nil {
 		response, err := getWorkflowExecutionWithRetry(c.shard, &persistence.GetWorkflowExecutionRequest{
 			ShardID:     c.shard.GetShardID(),
-			NamespaceID: c.namespaceID,
-			Execution:   c.workflowExecution,
+			NamespaceID: c.namespaceID.String(),
+			WorkflowID:  c.workflowExecution.GetWorkflowId(),
+			RunID:       c.workflowExecution.GetRunId(),
 		})
 		if err != nil {
 			return nil, err
@@ -362,20 +352,6 @@ func (c *ContextImpl) LoadWorkflowExecution() (MutableState, error) {
 		}
 
 		c.stats = response.State.ExecutionInfo.ExecutionStats
-		if c.stats == nil {
-			// TODO (alex): This check is for compatibility only and can be removed later
-			c.stats = &persistencespb.ExecutionStats{
-				HistorySize: response.State.ExecutionInfo.GetHistorySize(),
-			}
-		}
-
-		// finally emit execution and session stats
-		emitWorkflowExecutionStats(
-			c.metricsClient,
-			c.GetNamespace(),
-			response.MutableStateStats,
-			c.GetHistorySize(),
-		)
 	}
 
 	flushBeforeReady, err := c.MutableState.StartTransaction(namespaceEntry)
@@ -443,9 +419,13 @@ func (c *ContextImpl) CreateWorkflowExecution(
 	if err != nil {
 		return err
 	}
-	c.SetHistorySize(resp.HistorySize)
+	c.SetHistorySize(int64(resp.NewMutableStateStats.HistoryStatistics.SizeDiff))
 
-	NotifyWorkflowSnapshotTasks(c.engine, newWorkflow)
+	engine, err := c.shard.GetEngine()
+	if err != nil {
+		return err
+	}
+	NotifyWorkflowSnapshotTasks(engine, newWorkflow)
 	emitStateTransitionCount(c.metricsClient, newMutableState)
 
 	return nil
@@ -725,13 +705,6 @@ func (c *ContextImpl) UpdateWorkflowExecutionWithNew(
 		int(c.GetHistorySize()),
 		int(c.MutableState.GetNextEventID()-1),
 	)
-	// emit workflow completion stats if any
-	if currentWorkflow.ExecutionState.State == enumsspb.WORKFLOW_EXECUTION_STATE_COMPLETED {
-		if event, err := c.MutableState.GetCompletionEvent(); err == nil {
-			taskQueue := currentWorkflow.ExecutionInfo.TaskQueue
-			emitWorkflowCompletionStats(c.metricsClient, namespace, taskQueue, event)
-		}
-	}
 
 	return nil
 }
@@ -763,13 +736,13 @@ func (c *ContextImpl) mergeContinueAsNewReplicationTasks(
 
 	// merge the new run first event batch replication task
 	// to current event batch replication task
-	newRunTask := newWorkflowSnapshot.ReplicationTasks[0].(*persistence.HistoryReplicationTask)
+	newRunTask := newWorkflowSnapshot.ReplicationTasks[0].(*tasks.HistoryReplicationTask)
 	newWorkflowSnapshot.ReplicationTasks = nil
 
 	newRunBranchToken := newRunTask.BranchToken
 	taskUpdated := false
 	for _, replicationTask := range currentWorkflowMutation.ReplicationTasks {
-		if task, ok := replicationTask.(*persistence.HistoryReplicationTask); ok {
+		if task, ok := replicationTask.(*tasks.HistoryReplicationTask); ok {
 			taskUpdated = true
 			task.NewRunBranchToken = newRunBranchToken
 		}
@@ -823,14 +796,14 @@ func (c *ContextImpl) ReapplyEvents(
 		return nil
 	}
 
-	namespaceID := eventBatches[0].NamespaceID
+	namespaceID := namespace.ID(eventBatches[0].NamespaceID)
 	workflowID := eventBatches[0].WorkflowID
 	runID := eventBatches[0].RunID
 	var reapplyEvents []*historypb.HistoryEvent
 	for _, events := range eventBatches {
-		if events.NamespaceID != namespaceID ||
+		if namespace.ID(events.NamespaceID) != namespaceID ||
 			events.WorkflowID != workflowID {
-			return serviceerror.NewInternal("Context encounter mismatch namespaceID / workflowID in events reapplication.")
+			return serviceerror.NewInternal("Context encountered mismatch namespaceID / workflowID in events reapplication.")
 		}
 
 		for _, e := range events.Events {
@@ -851,10 +824,9 @@ func (c *ContextImpl) ReapplyEvents(
 		WorkflowId: workflowID,
 		RunId:      runID,
 	}
-	namespaceCache := c.shard.GetNamespaceCache()
-	clientBean := c.shard.GetService().GetClientBean()
-	serializer := c.shard.GetService().GetPayloadSerializer()
-	namespaceEntry, err := namespaceCache.GetNamespaceByID(namespaceID)
+	namespaceRegistry := c.shard.GetNamespaceRegistry()
+	serializer := c.shard.GetPayloadSerializer()
+	namespaceEntry, err := namespaceRegistry.GetNamespaceByID(namespaceID)
 	if err != nil {
 		return err
 	}
@@ -862,9 +834,13 @@ func (c *ContextImpl) ReapplyEvents(
 	ctx, cancel := context.WithTimeout(context.Background(), defaultRemoteCallTimeout)
 	defer cancel()
 
-	activeCluster := namespaceEntry.GetReplicationConfig().ActiveClusterName
+	activeCluster := namespaceEntry.ActiveClusterName()
 	if activeCluster == c.shard.GetClusterMetadata().GetCurrentClusterName() {
-		return c.shard.GetEngine().ReapplyEvents(
+		engine, err := c.shard.GetEngine()
+		if err != nil {
+			return err
+		}
+		return engine.ReapplyEvents(
 			ctx,
 			namespaceID,
 			workflowID,
@@ -882,7 +858,7 @@ func (c *ContextImpl) ReapplyEvents(
 	// The active cluster of the namespace is differ from the current cluster
 	// Use frontend client to route this request to the active cluster
 	// Reapplication only happens in active cluster
-	sourceCluster := clientBean.GetRemoteAdminClient(activeCluster)
+	sourceCluster := c.shard.GetRemoteAdminClient(activeCluster)
 	if sourceCluster == nil {
 		return serviceerror.NewInternal(fmt.Sprintf("cannot find cluster config %v to do reapply", activeCluster))
 	}
@@ -891,7 +867,7 @@ func (c *ContextImpl) ReapplyEvents(
 	_, err = sourceCluster.ReapplyEvents(
 		ctx2,
 		&adminservice.ReapplyEventsRequest{
-			Namespace:         namespaceEntry.GetInfo().Name,
+			Namespace:         namespaceEntry.Name().String(),
 			WorkflowExecution: execution,
 			Events:            reapplyEventsDataBlob,
 		},
@@ -902,10 +878,11 @@ func (c *ContextImpl) ReapplyEvents(
 
 // Returns true if execution is forced terminated
 func (c *ContextImpl) enforceSizeCheck() (bool, error) {
-	historySizeLimitWarn := c.config.HistorySizeLimitWarn(c.GetNamespace())
-	historySizeLimitError := c.config.HistorySizeLimitError(c.GetNamespace())
-	historyCountLimitWarn := c.config.HistoryCountLimitWarn(c.GetNamespace())
-	historyCountLimitError := c.config.HistoryCountLimitError(c.GetNamespace())
+	namespaceName := c.GetNamespace().String()
+	historySizeLimitWarn := c.config.HistorySizeLimitWarn(namespaceName)
+	historySizeLimitError := c.config.HistorySizeLimitError(namespaceName)
+	historyCountLimitWarn := c.config.HistoryCountLimitWarn(namespaceName)
+	historyCountLimitError := c.config.HistoryCountLimitError(namespaceName)
 
 	historySize := int(c.GetHistorySize())
 	historyCount := int(c.MutableState.GetNextEventID() - 1)
@@ -914,7 +891,7 @@ func (c *ContextImpl) enforceSizeCheck() (bool, error) {
 	if (historySize > historySizeLimitError || historyCount > historyCountLimitError) &&
 		c.MutableState.IsWorkflowExecutionRunning() {
 		c.logger.Error("history size exceeds error limit.",
-			tag.WorkflowNamespaceID(c.namespaceID),
+			tag.WorkflowNamespaceID(c.namespaceID.String()),
 			tag.WorkflowID(c.workflowExecution.GetWorkflowId()),
 			tag.WorkflowRunID(c.workflowExecution.GetRunId()),
 			tag.WorkflowHistorySize(historySize),
@@ -968,6 +945,6 @@ func emitStateTransitionCount(
 
 	metricsClient.Scope(
 		metrics.WorkflowContextScope,
-		metrics.NamespaceTag(mutableState.GetNamespaceEntry().GetInfo().Name),
+		metrics.NamespaceTag(mutableState.GetNamespaceEntry().Name().String()),
 	).RecordDistribution(metrics.StateTransitionCount, int(mutableState.GetExecutionInfo().StateTransitionCount))
 }
