@@ -45,6 +45,7 @@ import (
 	workflowspb "go.temporal.io/server/api/workflow/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/clock"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/service/history/tests"
 )
@@ -65,8 +66,8 @@ type (
 )
 
 var (
-	testNamespaceID   = uuid.New()
-	testNamespaceName = "test namespace"
+	testNamespaceID   = namespace.ID(uuid.New())
+	testNamespaceName = namespace.Name("test namespace")
 	testWorkflowID    = "test workflow ID"
 	testRunID         = uuid.New()
 
@@ -167,7 +168,7 @@ func (s *historyBuilderSuite) TestWorkflowExecutionStarted() {
 	originalRunID := uuid.New()
 
 	request := &historyservice.StartWorkflowExecutionRequest{
-		NamespaceId: testNamespaceID,
+		NamespaceId: testNamespaceID.String(),
 		ParentExecutionInfo: &workflowspb.ParentExecutionInfo{
 			NamespaceId: testParentNamespaceID,
 			Namespace:   testParentNamespaceName,
@@ -185,7 +186,7 @@ func (s *historyBuilderSuite) TestWorkflowExecutionStarted() {
 		FirstWorkflowTaskBackoff:        firstWorkflowTaskBackoff,
 
 		StartRequest: &workflowservice.StartWorkflowExecutionRequest{
-			Namespace:                testNamespaceName,
+			Namespace:                testNamespaceName.String(),
 			WorkflowId:               testWorkflowID,
 			WorkflowType:             testWorkflowType,
 			TaskQueue:                testTaskQueue,
@@ -258,7 +259,7 @@ func (s *historyBuilderSuite) TestWorkflowExecutionStarted() {
 func (s *historyBuilderSuite) TestWorkflowExecutionCancelRequested() {
 	initiatedEventID := rand.Int63()
 	request := &historyservice.RequestCancelWorkflowExecutionRequest{
-		NamespaceId: tests.NamespaceID,
+		NamespaceId: tests.NamespaceID.String(),
 		CancelRequest: &workflowservice.RequestCancelWorkflowExecutionRequest{
 			// Namespace: not used for test
 			// WorkflowExecution: not used for test
@@ -303,6 +304,7 @@ func (s *historyBuilderSuite) TestWorkflowExecutionSignaled() {
 		signalName,
 		testPayloads,
 		testIdentity,
+		testHeader,
 	)
 	s.Equal(event, s.flush())
 	s.Equal(&historypb.HistoryEvent{
@@ -316,6 +318,7 @@ func (s *historyBuilderSuite) TestWorkflowExecutionSignaled() {
 				SignalName: signalName,
 				Input:      testPayloads,
 				Identity:   testIdentity,
+				Header:     testHeader,
 			},
 		},
 	}, event)
@@ -387,6 +390,7 @@ func (s *historyBuilderSuite) TestWorkflowExecutionCompleted() {
 	event := s.historyBuilder.AddCompletedWorkflowEvent(
 		workflowTaskCompletionEventID,
 		attributes,
+		"",
 	)
 	s.Equal(event, s.flush())
 	s.Equal(&historypb.HistoryEvent{
@@ -414,6 +418,7 @@ func (s *historyBuilderSuite) TestWorkflowExecutionFailed() {
 		workflowTaskCompletionEventID,
 		retryState,
 		attributes,
+		"",
 	)
 	s.Equal(event, s.flush())
 	s.Equal(&historypb.HistoryEvent{
@@ -436,6 +441,7 @@ func (s *historyBuilderSuite) TestWorkflowExecutionTimeout() {
 	retryState := enumspb.RetryState(rand.Int31n(int32(len(enumspb.RetryState_name))))
 	event := s.historyBuilder.AddTimeoutWorkflowEvent(
 		retryState,
+		"",
 	)
 	s.Equal(event, s.flush())
 	s.Equal(&historypb.HistoryEvent{
@@ -718,7 +724,7 @@ func (s *historyBuilderSuite) TestActivityTaskScheduled() {
 	startToCloseTimeout := timestamp.DurationPtr(time.Duration(rand.Int63()))
 	heartbeatTimeout := timestamp.DurationPtr(time.Duration(rand.Int63()))
 	attributes := &commandpb.ScheduleActivityTaskCommandAttributes{
-		Namespace:              testNamespaceName,
+		Namespace:              testNamespaceName.String(),
 		ActivityId:             activityID,
 		ActivityType:           testActivityType,
 		TaskQueue:              testTaskQueue,
@@ -744,7 +750,7 @@ func (s *historyBuilderSuite) TestActivityTaskScheduled() {
 		Attributes: &historypb.HistoryEvent_ActivityTaskScheduledEventAttributes{
 			ActivityTaskScheduledEventAttributes: &historypb.ActivityTaskScheduledEventAttributes{
 				WorkflowTaskCompletedEventId: workflowTaskCompletionEventID,
-				Namespace:                    testNamespaceName,
+				Namespace:                    testNamespaceName.String(),
 				ActivityId:                   activityID,
 				ActivityType:                 testActivityType,
 				TaskQueue:                    testTaskQueue,
@@ -1018,7 +1024,7 @@ func (s *historyBuilderSuite) TestRequestCancelExternalWorkflowExecutionInitiate
 	control := "random control"
 	childWorkflowOnly := rand.Int31()%2 == 0
 	attributes := &commandpb.RequestCancelExternalWorkflowExecutionCommandAttributes{
-		Namespace:         testNamespaceName,
+		Namespace:         testNamespaceName.String(),
 		WorkflowId:        testWorkflowID,
 		RunId:             testRunID,
 		Control:           control,
@@ -1038,7 +1044,7 @@ func (s *historyBuilderSuite) TestRequestCancelExternalWorkflowExecutionInitiate
 		Attributes: &historypb.HistoryEvent_RequestCancelExternalWorkflowExecutionInitiatedEventAttributes{
 			RequestCancelExternalWorkflowExecutionInitiatedEventAttributes: &historypb.RequestCancelExternalWorkflowExecutionInitiatedEventAttributes{
 				WorkflowTaskCompletedEventId: workflowTaskCompletionEventID,
-				Namespace:                    testNamespaceName,
+				Namespace:                    testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1068,7 +1074,7 @@ func (s *historyBuilderSuite) TestRequestCancelExternalWorkflowExecutionSuccess(
 		Attributes: &historypb.HistoryEvent_ExternalWorkflowExecutionCancelRequestedEventAttributes{
 			ExternalWorkflowExecutionCancelRequestedEventAttributes: &historypb.ExternalWorkflowExecutionCancelRequestedEventAttributes{
 				InitiatedEventId: scheduleEventID,
-				Namespace:        testNamespaceName,
+				Namespace:        testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1101,7 +1107,7 @@ func (s *historyBuilderSuite) TestRequestCancelExternalWorkflowExecutionFailed()
 			RequestCancelExternalWorkflowExecutionFailedEventAttributes: &historypb.RequestCancelExternalWorkflowExecutionFailedEventAttributes{
 				WorkflowTaskCompletedEventId: workflowTaskCompletionEventID,
 				InitiatedEventId:             scheduleEventID,
-				Namespace:                    testNamespaceName,
+				Namespace:                    testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1121,7 +1127,7 @@ func (s *historyBuilderSuite) TestSignalExternalWorkflowExecutionInitiated() {
 	control := "random control"
 	childWorkflowOnly := rand.Int31()%2 == 0
 	attributes := &commandpb.SignalExternalWorkflowExecutionCommandAttributes{
-		Namespace: testNamespaceName,
+		Namespace: testNamespaceName.String(),
 		Execution: &commonpb.WorkflowExecution{
 			WorkflowId: testWorkflowID,
 			RunId:      testRunID,
@@ -1130,6 +1136,7 @@ func (s *historyBuilderSuite) TestSignalExternalWorkflowExecutionInitiated() {
 		Input:             testPayloads,
 		Control:           control,
 		ChildWorkflowOnly: childWorkflowOnly,
+		Header:            testHeader,
 	}
 	event := s.historyBuilder.AddSignalExternalWorkflowExecutionInitiatedEvent(
 		workflowTaskCompletionEventID,
@@ -1145,7 +1152,7 @@ func (s *historyBuilderSuite) TestSignalExternalWorkflowExecutionInitiated() {
 		Attributes: &historypb.HistoryEvent_SignalExternalWorkflowExecutionInitiatedEventAttributes{
 			SignalExternalWorkflowExecutionInitiatedEventAttributes: &historypb.SignalExternalWorkflowExecutionInitiatedEventAttributes{
 				WorkflowTaskCompletedEventId: workflowTaskCompletionEventID,
-				Namespace:                    testNamespaceName,
+				Namespace:                    testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1154,6 +1161,7 @@ func (s *historyBuilderSuite) TestSignalExternalWorkflowExecutionInitiated() {
 				Input:             testPayloads,
 				Control:           control,
 				ChildWorkflowOnly: childWorkflowOnly,
+				Header:            testHeader,
 			},
 		},
 	}, event)
@@ -1179,7 +1187,7 @@ func (s *historyBuilderSuite) TestSignalExternalWorkflowExecutionSuccess() {
 		Attributes: &historypb.HistoryEvent_ExternalWorkflowExecutionSignaledEventAttributes{
 			ExternalWorkflowExecutionSignaledEventAttributes: &historypb.ExternalWorkflowExecutionSignaledEventAttributes{
 				InitiatedEventId: scheduleEventID,
-				Namespace:        testNamespaceName,
+				Namespace:        testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1215,7 +1223,7 @@ func (s *historyBuilderSuite) TestSignalExternalWorkflowExecutionFailed() {
 			SignalExternalWorkflowExecutionFailedEventAttributes: &historypb.SignalExternalWorkflowExecutionFailedEventAttributes{
 				WorkflowTaskCompletedEventId: workflowTaskCompletionEventID,
 				InitiatedEventId:             scheduleEventID,
-				Namespace:                    testNamespaceName,
+				Namespace:                    testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1240,7 +1248,7 @@ func (s *historyBuilderSuite) TestStartChildWorkflowExecutionInitiated() {
 	control := "random control"
 
 	attributes := &commandpb.StartChildWorkflowExecutionCommandAttributes{
-		Namespace:                testNamespaceName,
+		Namespace:                testNamespaceName.String(),
 		WorkflowId:               testWorkflowID,
 		WorkflowType:             testWorkflowType,
 		TaskQueue:                testTaskQueue,
@@ -1271,7 +1279,7 @@ func (s *historyBuilderSuite) TestStartChildWorkflowExecutionInitiated() {
 		Attributes: &historypb.HistoryEvent_StartChildWorkflowExecutionInitiatedEventAttributes{
 			StartChildWorkflowExecutionInitiatedEventAttributes: &historypb.StartChildWorkflowExecutionInitiatedEventAttributes{
 				WorkflowTaskCompletedEventId: workflowTaskCompletionEventID,
-				Namespace:                    testNamespaceName,
+				Namespace:                    testNamespaceName.String(),
 				WorkflowId:                   testWorkflowID,
 				WorkflowType:                 testWorkflowType,
 				TaskQueue:                    testTaskQueue,
@@ -1313,7 +1321,7 @@ func (s *historyBuilderSuite) TestStartChildWorkflowExecutionSuccess() {
 		Version:   s.version,
 		Attributes: &historypb.HistoryEvent_ChildWorkflowExecutionStartedEventAttributes{
 			ChildWorkflowExecutionStartedEventAttributes: &historypb.ChildWorkflowExecutionStartedEventAttributes{
-				Namespace: testNamespaceName,
+				Namespace: testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1350,7 +1358,7 @@ func (s *historyBuilderSuite) TestStartChildWorkflowExecutionFailed() {
 		Attributes: &historypb.HistoryEvent_StartChildWorkflowExecutionFailedEventAttributes{
 			StartChildWorkflowExecutionFailedEventAttributes: &historypb.StartChildWorkflowExecutionFailedEventAttributes{
 				WorkflowTaskCompletedEventId: workflowTaskCompletionEventID,
-				Namespace:                    testNamespaceName,
+				Namespace:                    testNamespaceName.String(),
 				WorkflowId:                   testWorkflowID,
 				WorkflowType:                 testWorkflowType,
 				InitiatedEventId:             scheduleEventID,
@@ -1387,7 +1395,7 @@ func (s *historyBuilderSuite) TestChildWorkflowExecutionCompleted() {
 			ChildWorkflowExecutionCompletedEventAttributes: &historypb.ChildWorkflowExecutionCompletedEventAttributes{
 				InitiatedEventId: scheduleEventID,
 				StartedEventId:   startedEventID,
-				Namespace:        testNamespaceName,
+				Namespace:        testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1426,7 +1434,7 @@ func (s *historyBuilderSuite) TestChildWorkflowExecutionFailed() {
 			ChildWorkflowExecutionFailedEventAttributes: &historypb.ChildWorkflowExecutionFailedEventAttributes{
 				InitiatedEventId: scheduleEventID,
 				StartedEventId:   startedEventID,
-				Namespace:        testNamespaceName,
+				Namespace:        testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1465,7 +1473,7 @@ func (s *historyBuilderSuite) TestChildWorkflowExecutionTimeout() {
 			ChildWorkflowExecutionTimedOutEventAttributes: &historypb.ChildWorkflowExecutionTimedOutEventAttributes{
 				InitiatedEventId: scheduleEventID,
 				StartedEventId:   startedEventID,
-				Namespace:        testNamespaceName,
+				Namespace:        testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1502,7 +1510,7 @@ func (s *historyBuilderSuite) TestChildWorkflowExecutionCancelled() {
 			ChildWorkflowExecutionCanceledEventAttributes: &historypb.ChildWorkflowExecutionCanceledEventAttributes{
 				InitiatedEventId: scheduleEventID,
 				StartedEventId:   startedEventID,
-				Namespace:        testNamespaceName,
+				Namespace:        testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
@@ -1538,7 +1546,7 @@ func (s *historyBuilderSuite) TestChildWorkflowExecutionTerminated() {
 			ChildWorkflowExecutionTerminatedEventAttributes: &historypb.ChildWorkflowExecutionTerminatedEventAttributes{
 				InitiatedEventId: scheduleEventID,
 				StartedEventId:   startedEventID,
-				Namespace:        testNamespaceName,
+				Namespace:        testNamespaceName.String(),
 				WorkflowExecution: &commonpb.WorkflowExecution{
 					WorkflowId: testWorkflowID,
 					RunId:      testRunID,
