@@ -38,13 +38,13 @@ import (
 
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/collection"
 	"go.temporal.io/server/common/definition"
 	"go.temporal.io/server/common/failure"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/service/history/consts"
@@ -59,7 +59,7 @@ type (
 		// resetWorkflow is the new NDC compatible workflow reset logic
 		resetWorkflow(
 			ctx context.Context,
-			namespaceID string,
+			namespaceID namespace.ID,
 			workflowID string,
 			baseRunID string,
 			baseBranchToken []byte,
@@ -79,10 +79,10 @@ type (
 
 	workflowResetterImpl struct {
 		shard             shard.Context
-		namespaceCache    cache.NamespaceCache
+		namespaceRegistry namespace.Registry
 		clusterMetadata   cluster.Metadata
 		executionMgr      persistence.ExecutionManager
-		historyCache      *workflow.Cache
+		historyCache      workflow.Cache
 		newStateRebuilder nDCStateRebuilderProvider
 		transaction       workflow.Transaction
 		logger            log.Logger
@@ -93,15 +93,15 @@ var _ workflowResetter = (*workflowResetterImpl)(nil)
 
 func newWorkflowResetter(
 	shard shard.Context,
-	historyCache *workflow.Cache,
+	historyCache workflow.Cache,
 	logger log.Logger,
 ) *workflowResetterImpl {
 	return &workflowResetterImpl{
-		shard:           shard,
-		namespaceCache:  shard.GetNamespaceCache(),
-		clusterMetadata: shard.GetClusterMetadata(),
-		executionMgr:    shard.GetExecutionManager(),
-		historyCache:    historyCache,
+		shard:             shard,
+		namespaceRegistry: shard.GetNamespaceRegistry(),
+		clusterMetadata:   shard.GetClusterMetadata(),
+		executionMgr:      shard.GetExecutionManager(),
+		historyCache:      historyCache,
 		newStateRebuilder: func() nDCStateRebuilder {
 			return newNDCStateRebuilder(shard, logger)
 		},
@@ -112,7 +112,7 @@ func newWorkflowResetter(
 
 func (r *workflowResetterImpl) resetWorkflow(
 	ctx context.Context,
-	namespaceID string,
+	namespaceID namespace.ID,
 	workflowID string,
 	baseRunID string,
 	baseBranchToken []byte,
@@ -127,11 +127,11 @@ func (r *workflowResetterImpl) resetWorkflow(
 	resetReapplyType enumspb.ResetReapplyType,
 ) (retError error) {
 
-	namespaceEntry, err := r.namespaceCache.GetNamespaceByID(namespaceID)
+	namespaceEntry, err := r.namespaceRegistry.GetNamespaceByID(namespaceID)
 	if err != nil {
 		return err
 	}
-	resetWorkflowVersion := namespaceEntry.GetFailoverVersion()
+	resetWorkflowVersion := namespaceEntry.FailoverVersion()
 
 	var currentWorkflowMutation *persistence.WorkflowMutation
 	var currentWorkflowEventsSeq []*persistence.WorkflowEvents
@@ -239,7 +239,7 @@ func (r *workflowResetterImpl) resetWorkflow(
 
 func (r *workflowResetterImpl) prepareResetWorkflow(
 	ctx context.Context,
-	namespaceID string,
+	namespaceID namespace.ID,
 	workflowID string,
 	baseRunID string,
 	baseBranchToken []byte,
@@ -276,7 +276,7 @@ func (r *workflowResetterImpl) prepareResetWorkflow(
 	}
 
 	if resetMutableState.GetCurrentVersion() > resetWorkflowVersion {
-		return nil, serviceerror.NewInternal("workflowResetter encounter version mismatch.")
+		return nil, serviceerror.NewInternal("workflowResetter encountered version mismatch.")
 	}
 	if err := resetMutableState.UpdateCurrentVersion(
 		resetWorkflowVersion,
@@ -383,7 +383,7 @@ func (r *workflowResetterImpl) persistToDB(
 
 	return resetWorkflow.getContext().CreateWorkflowExecution(
 		now,
-		persistence.CreateWorkflowModeContinueAsNew,
+		persistence.CreateWorkflowModeWorkflowIDReuse,
 		currentRunID,
 		currentLastWriteVersion,
 		resetWorkflow.getMutableState(),
@@ -394,7 +394,7 @@ func (r *workflowResetterImpl) persistToDB(
 
 func (r *workflowResetterImpl) replayResetWorkflow(
 	ctx context.Context,
-	namespaceID string,
+	namespaceID namespace.ID,
 	workflowID string,
 	baseRunID string,
 	baseBranchToken []byte,
@@ -427,16 +427,16 @@ func (r *workflowResetterImpl) replayResetWorkflow(
 	resetMutableState, resetHistorySize, err := r.newStateRebuilder().rebuild(
 		ctx,
 		r.shard.GetTimeSource().Now(),
-		definition.NewWorkflowIdentifier(
-			namespaceID,
+		definition.NewWorkflowKey(
+			namespaceID.String(),
 			workflowID,
 			baseRunID,
 		),
 		baseBranchToken,
 		baseRebuildLastEventID,
 		baseRebuildLastEventVersion,
-		definition.NewWorkflowIdentifier(
-			namespaceID,
+		definition.NewWorkflowKey(
+			namespaceID.String(),
 			workflowID,
 			resetRunID,
 		),
@@ -450,7 +450,7 @@ func (r *workflowResetterImpl) replayResetWorkflow(
 	resetContext.SetHistorySize(resetHistorySize)
 	return newNDCWorkflow(
 		ctx,
-		r.namespaceCache,
+		r.namespaceRegistry,
 		r.clusterMetadata,
 		resetContext,
 		resetMutableState,
@@ -543,7 +543,7 @@ func (r *workflowResetterImpl) failInflightActivity(
 }
 
 func (r *workflowResetterImpl) forkAndGenerateBranchToken(
-	namespaceID string,
+	namespaceID namespace.ID,
 	workflowID string,
 	forkBranchToken []byte,
 	forkNodeID int64,
@@ -554,7 +554,7 @@ func (r *workflowResetterImpl) forkAndGenerateBranchToken(
 	resp, err := r.executionMgr.ForkHistoryBranch(&persistence.ForkHistoryBranchRequest{
 		ForkBranchToken: forkBranchToken,
 		ForkNodeID:      forkNodeID,
-		Info:            persistence.BuildHistoryGarbageCleanupInfo(namespaceID, workflowID, resetRunID),
+		Info:            persistence.BuildHistoryGarbageCleanupInfo(namespaceID.String(), workflowID, resetRunID),
 		ShardID:         shardID,
 	})
 	if err != nil {
@@ -582,7 +582,7 @@ func (r *workflowResetterImpl) terminateWorkflow(
 func (r *workflowResetterImpl) reapplyContinueAsNewWorkflowEvents(
 	ctx context.Context,
 	resetMutableState workflow.MutableState,
-	namespaceID string,
+	namespaceID namespace.ID,
 	workflowID string,
 	baseRunID string,
 	baseBranchToken []byte,
@@ -607,7 +607,7 @@ func (r *workflowResetterImpl) reapplyContinueAsNewWorkflowEvents(
 		// noop
 	case *serviceerror.DataLoss:
 		// log event
-		r.logger.Error("encounter data loss event", tag.WorkflowNamespaceID(namespaceID), tag.WorkflowID(workflowID), tag.WorkflowRunID(baseRunID))
+		r.logger.Error("encountered data loss event", tag.WorkflowNamespaceID(namespaceID.String()), tag.WorkflowID(workflowID), tag.WorkflowRunID(baseRunID))
 		return "", err
 	default:
 		return "", err
@@ -661,7 +661,7 @@ func (r *workflowResetterImpl) reapplyContinueAsNewWorkflowEvents(
 			// noop
 		case *serviceerror.DataLoss:
 			// log event
-			r.logger.Error("encounter data loss event", tag.WorkflowNamespaceID(namespaceID), tag.WorkflowID(workflowID), tag.WorkflowRunID(nextRunID))
+			r.logger.Error("encounter data loss event", tag.WorkflowNamespaceID(namespaceID.String()), tag.WorkflowID(workflowID), tag.WorkflowRunID(nextRunID))
 			return "", err
 		default:
 			return "", err
@@ -723,6 +723,7 @@ func (r *workflowResetterImpl) reapplyEvents(
 				attr.GetSignalName(),
 				attr.GetInput(),
 				attr.GetIdentity(),
+				attr.GetHeader(),
 			); err != nil {
 				return err
 			}
