@@ -33,20 +33,21 @@ import (
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	failurepb "go.temporal.io/api/failure/v1"
-	historypb "go.temporal.io/api/history/v1"
 	"go.temporal.io/api/serviceerror"
+
 	"go.temporal.io/server/common/searchattribute"
 
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/backoff"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/enums"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/payloads"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/service/history/configs"
+	"go.temporal.io/server/service/history/shard"
 	"go.temporal.io/server/service/history/workflow"
 )
 
@@ -61,7 +62,7 @@ type (
 		hasBufferedEvents               bool
 		workflowTaskFailedCause         *workflowTaskFailedCause
 		activityNotStartedCancelled     bool
-		continueAsNewBuilder            workflow.MutableState
+		newStateBuilder                 workflow.MutableState
 		stopProcessing                  bool // should stop processing any more commands
 		mutableState                    workflow.MutableState
 		initiatedChildExecutionsInBatch map[string]struct{} // Set of initiated child executions in the workflow task
@@ -71,10 +72,11 @@ type (
 		sizeLimitChecker       *workflowSizeChecker
 		searchAttributesMapper searchattribute.Mapper
 
-		logger         log.Logger
-		namespaceCache cache.NamespaceCache
-		metricsClient  metrics.Client
-		config         *configs.Config
+		logger            log.Logger
+		namespaceRegistry namespace.Registry
+		metricsClient     metrics.Client
+		config            *configs.Config
+		shard             shard.Context
 	}
 
 	workflowTaskFailedCause struct {
@@ -90,9 +92,10 @@ func newWorkflowTaskHandler(
 	attrValidator *commandAttrValidator,
 	sizeLimitChecker *workflowSizeChecker,
 	logger log.Logger,
-	namespaceCache cache.NamespaceCache,
+	namespaceRegistry namespace.Registry,
 	metricsClient metrics.Client,
 	config *configs.Config,
+	shard shard.Context,
 	searchAttributesMapper searchattribute.Mapper,
 ) *workflowTaskHandlerImpl {
 
@@ -104,7 +107,7 @@ func newWorkflowTaskHandler(
 		hasBufferedEvents:               mutableState.HasBufferedEvents(),
 		workflowTaskFailedCause:         nil,
 		activityNotStartedCancelled:     false,
-		continueAsNewBuilder:            nil,
+		newStateBuilder:                 nil,
 		stopProcessing:                  false,
 		mutableState:                    mutableState,
 		initiatedChildExecutionsInBatch: make(map[string]struct{}),
@@ -114,25 +117,28 @@ func newWorkflowTaskHandler(
 		sizeLimitChecker:       sizeLimitChecker,
 		searchAttributesMapper: searchAttributesMapper,
 
-		logger:         logger,
-		namespaceCache: namespaceCache,
-		metricsClient:  metricsClient,
-		config:         config,
+		logger:            logger,
+		namespaceRegistry: namespaceRegistry,
+		metricsClient:     metricsClient,
+		config:            config,
+		shard:             shard,
 	}
 }
 
 func (handler *workflowTaskHandlerImpl) handleCommands(
 	commands []*commandpb.Command,
 ) error {
-
+	if err := handler.attrValidator.validateCommandSequence(
+		commands,
+	); err != nil {
+		return err
+	}
 	for _, command := range commands {
-
 		err := handler.handleCommand(command)
 		if err != nil || handler.stopProcessing {
 			return err
 		}
 	}
-
 	return nil
 }
 
@@ -192,14 +198,14 @@ func (handler *workflowTaskHandlerImpl) handleCommandScheduleActivity(
 	)
 
 	executionInfo := handler.mutableState.GetExecutionInfo()
-	namespaceID := executionInfo.NamespaceId
+	namespaceID := namespace.ID(executionInfo.NamespaceId)
 	targetNamespaceID := namespaceID
 	if attr.GetNamespace() != "" {
-		targetNamespaceEntry, err := handler.namespaceCache.GetNamespace(attr.GetNamespace())
+		targetNamespaceEntry, err := handler.namespaceRegistry.GetNamespace(namespace.Name(attr.GetNamespace()))
 		if err != nil {
-			return serviceerror.NewInternal(fmt.Sprintf("Unable to schedule activity across namespace %v.", attr.GetNamespace()))
+			return serviceerror.NewUnavailable(fmt.Sprintf("Unable to schedule activity across namespace %v.", attr.GetNamespace()))
 		}
-		targetNamespaceID = targetNamespaceEntry.GetInfo().Id
+		targetNamespaceID = targetNamespaceEntry.ID()
 	}
 
 	if err := handler.validateCommandAttr(
@@ -365,33 +371,24 @@ func (handler *workflowTaskHandlerImpl) handleCommandCompleteWorkflow(
 		return nil
 	}
 
-	// check if this is a cron workflow
-	cronBackoff, err := handler.mutableState.GetCronBackoffDuration()
-	if err != nil {
-		handler.stopProcessing = true
-		return err
-	}
-	if cronBackoff == backoff.NoBackoff {
-		// not cron, so complete this workflow execution
-		if _, err := handler.mutableState.AddCompletedWorkflowEvent(handler.workflowTaskCompletedID, attr); err != nil {
-			return serviceerror.NewInternal("Unable to add complete workflow event.")
-		}
-		return nil
+	cronBackoff := handler.mutableState.GetCronBackoffDuration()
+	var newExecutionRunID string
+	if cronBackoff != backoff.NoBackoff {
+		newExecutionRunID = uuid.New()
 	}
 
-	// this is a cron workflow
-	startEvent, err := handler.mutableState.GetStartEvent()
+	// Always add workflow completed event to this one
+	_, err = handler.mutableState.AddCompletedWorkflowEvent(handler.workflowTaskCompletedID, attr, newExecutionRunID)
 	if err != nil {
 		return err
 	}
-	startAttributes := startEvent.GetWorkflowExecutionStartedEventAttributes()
-	return handler.retryCronContinueAsNew(
-		startAttributes,
-		cronBackoff,
-		enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE,
-		nil,
-		attr.Result,
-	)
+
+	// Check if this workflow has a cron schedule
+	if cronBackoff != backoff.NoBackoff {
+		return handler.handleCron(cronBackoff, attr.GetResult(), nil, newExecutionRunID)
+	}
+
+	return nil
 }
 
 func (handler *workflowTaskHandlerImpl) handleCommandFailWorkflow(
@@ -440,41 +437,38 @@ func (handler *workflowTaskHandlerImpl) handleCommandFailWorkflow(
 		return nil
 	}
 
-	// below will check whether to do continue as new based on backoff & backoff or cron
-	backoffInterval, retryState := handler.mutableState.GetRetryBackoffDuration(attr.GetFailure())
-	continueAsNewInitiator := enumspb.CONTINUE_AS_NEW_INITIATOR_RETRY
-	// first check the backoff retry
-	if backoffInterval == backoff.NoBackoff {
-		// if no backoff retry, set the backoffInterval using cron schedule
-		backoffInterval, err = handler.mutableState.GetCronBackoffDuration()
-		if err != nil {
-			handler.stopProcessing = true
-			return err
-		}
-		continueAsNewInitiator = enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE
-	}
-	// second check the backoff / cron schedule
-	if backoffInterval == backoff.NoBackoff {
-		// no retry or cron
-		if _, err := handler.mutableState.AddFailWorkflowEvent(handler.workflowTaskCompletedID, retryState, attr); err != nil {
-			return err
-		}
-		return nil
+	// First check retry policy to do a retry.
+	retryBackoff, retryState := handler.mutableState.GetRetryBackoffDuration(attr.GetFailure())
+	cronBackoff := backoff.NoBackoff
+	if retryBackoff == backoff.NoBackoff {
+		// If no retry, check cron.
+		cronBackoff = handler.mutableState.GetCronBackoffDuration()
 	}
 
-	// this is a cron / backoff workflow
-	startEvent, err := handler.mutableState.GetStartEvent()
-	if err != nil {
+	var newExecutionRunID string
+	if retryBackoff != backoff.NoBackoff || cronBackoff != backoff.NoBackoff {
+		newExecutionRunID = uuid.New()
+	}
+
+	// Always add workflow failed event
+	if _, err = handler.mutableState.AddFailWorkflowEvent(
+		handler.workflowTaskCompletedID,
+		retryState,
+		attr,
+		newExecutionRunID,
+	); err != nil {
 		return err
 	}
-	startAttributes := startEvent.GetWorkflowExecutionStartedEventAttributes()
-	return handler.retryCronContinueAsNew(
-		startAttributes,
-		backoffInterval,
-		continueAsNewInitiator,
-		attr.GetFailure(),
-		startAttributes.LastCompletionResult,
-	)
+
+	// Handle retry or cron
+	if retryBackoff != backoff.NoBackoff {
+		return handler.handleRetry(retryBackoff, retryState, attr.GetFailure(), newExecutionRunID)
+	} else if cronBackoff != backoff.NoBackoff {
+		return handler.handleCron(cronBackoff, nil, attr.GetFailure(), newExecutionRunID)
+	}
+
+	// No retry or cron
+	return nil
 }
 
 func (handler *workflowTaskHandlerImpl) handleCommandCancelTimer(
@@ -564,14 +558,14 @@ func (handler *workflowTaskHandlerImpl) handleCommandRequestCancelExternalWorkfl
 	)
 
 	executionInfo := handler.mutableState.GetExecutionInfo()
-	namespaceID := executionInfo.NamespaceId
+	namespaceID := namespace.ID(executionInfo.NamespaceId)
 	targetNamespaceID := namespaceID
 	if attr.GetNamespace() != "" {
-		targetNamespaceEntry, err := handler.namespaceCache.GetNamespace(attr.GetNamespace())
+		targetNamespaceEntry, err := handler.namespaceRegistry.GetNamespace(namespace.Name(attr.GetNamespace()))
 		if err != nil {
-			return serviceerror.NewInternal(fmt.Sprintf("Unable to cancel workflow across namespace: %v.", attr.GetNamespace()))
+			return serviceerror.NewUnavailable(fmt.Sprintf("Unable to cancel workflow across namespace: %v.", attr.GetNamespace()))
 		}
-		targetNamespaceID = targetNamespaceEntry.GetInfo().Id
+		targetNamespaceID = targetNamespaceEntry.ID()
 	}
 
 	if err := handler.validateCommandAttr(
@@ -641,12 +635,12 @@ func (handler *workflowTaskHandlerImpl) handleCommandContinueAsNewWorkflow(
 		return handler.failCommand(enumspb.WORKFLOW_TASK_FAILED_CAUSE_UNHANDLED_COMMAND, nil)
 	}
 
-	namespace := handler.mutableState.GetNamespaceEntry().GetInfo().Name
+	namespaceName := handler.mutableState.GetNamespaceEntry().Name()
 
 	if err := handler.validateCommandAttr(
 		func() error {
 			return handler.attrValidator.validateContinueAsNewWorkflowExecutionAttributes(
-				namespace,
+				namespaceName,
 				attr,
 				handler.mutableState.GetExecutionInfo(),
 				handler.config.DefaultVisibilityIndexName,
@@ -679,7 +673,7 @@ func (handler *workflowTaskHandlerImpl) handleCommandContinueAsNewWorkflow(
 
 	failWorkflow, err = handler.sizeLimitChecker.failWorkflowIfSearchAttributesSizeExceedsLimit(
 		attr.GetSearchAttributes(),
-		namespace,
+		namespaceName,
 		metrics.CommandTypeTag(enumspb.COMMAND_TYPE_CONTINUE_AS_NEW_WORKFLOW_EXECUTION.String()),
 	)
 	if err != nil || failWorkflow {
@@ -687,7 +681,7 @@ func (handler *workflowTaskHandlerImpl) handleCommandContinueAsNewWorkflow(
 		return err
 	}
 
-	if err := searchattribute.SubstituteAliases(handler.searchAttributesMapper, attr.GetSearchAttributes(), namespace); err != nil {
+	if err := searchattribute.SubstituteAliases(handler.searchAttributesMapper, attr.GetSearchAttributes(), namespaceName.String()); err != nil {
 		handler.stopProcessing = true
 		return err
 	}
@@ -706,15 +700,15 @@ func (handler *workflowTaskHandlerImpl) handleCommandContinueAsNewWorkflow(
 		return nil
 	}
 
-	// Extract parentNamespace so it can be passed down to next run of workflow execution
-	var parentNamespace string
+	// Extract parentNamespace, so it can be passed down to next run of workflow execution
+	var parentNamespace namespace.Name
 	if handler.mutableState.HasParentExecution() {
-		parentNamespaceID := handler.mutableState.GetExecutionInfo().ParentNamespaceId
-		parentNamespaceEntry, err := handler.namespaceCache.GetNamespaceByID(parentNamespaceID)
+		parentNamespaceID := namespace.ID(handler.mutableState.GetExecutionInfo().ParentNamespaceId)
+		parentNamespaceEntry, err := handler.namespaceRegistry.GetNamespaceByID(parentNamespaceID)
 		if err != nil {
 			return err
 		}
-		parentNamespace = parentNamespaceEntry.GetInfo().Name
+		parentNamespace = parentNamespaceEntry.Name()
 	}
 
 	_, newStateBuilder, err := handler.mutableState.AddContinueAsNewEvent(
@@ -727,33 +721,32 @@ func (handler *workflowTaskHandlerImpl) handleCommandContinueAsNewWorkflow(
 		return err
 	}
 
-	handler.continueAsNewBuilder = newStateBuilder
+	handler.newStateBuilder = newStateBuilder
 	return nil
 }
 
 func (handler *workflowTaskHandlerImpl) handleCommandStartChildWorkflow(
 	attr *commandpb.StartChildWorkflowExecutionCommandAttributes,
 ) error {
-
 	handler.metricsClient.IncCounter(
 		metrics.HistoryRespondWorkflowTaskCompletedScope,
 		metrics.CommandTypeChildWorkflowCounter,
 	)
 
 	parentNamespaceEntry := handler.mutableState.GetNamespaceEntry()
-	parentNamespaceID := parentNamespaceEntry.GetInfo().Id
-	parentNamespace := parentNamespaceEntry.GetInfo().Name
+	parentNamespaceID := parentNamespaceEntry.ID()
+	parentNamespace := parentNamespaceEntry.Name()
 	targetNamespaceID := parentNamespaceID
 	targetNamespace := parentNamespace
 	if attr.GetNamespace() != "" {
-		targetNamespaceEntry, err := handler.namespaceCache.GetNamespace(attr.GetNamespace())
+		targetNamespaceEntry, err := handler.namespaceRegistry.GetNamespace(namespace.Name(attr.GetNamespace()))
 		if err != nil {
-			return serviceerror.NewInternal(fmt.Sprintf("Unable to schedule child execution across namespace %v.", attr.GetNamespace()))
+			return serviceerror.NewUnavailable(fmt.Sprintf("Unable to schedule child execution across namespace %v.", attr.GetNamespace()))
 		}
-		targetNamespace = targetNamespaceEntry.GetInfo().Name
-		targetNamespaceID = targetNamespaceEntry.GetInfo().Id
+		targetNamespace = targetNamespaceEntry.Name()
+		targetNamespaceID = targetNamespaceEntry.ID()
 	} else {
-		attr.Namespace = parentNamespace
+		attr.Namespace = parentNamespace.String()
 	}
 
 	if err := handler.validateCommandAttr(
@@ -803,12 +796,12 @@ func (handler *workflowTaskHandlerImpl) handleCommandStartChildWorkflow(
 		return err
 	}
 
-	if err := searchattribute.SubstituteAliases(handler.searchAttributesMapper, attr.GetSearchAttributes(), targetNamespace); err != nil {
+	if err := searchattribute.SubstituteAliases(handler.searchAttributesMapper, attr.GetSearchAttributes(), targetNamespace.String()); err != nil {
 		handler.stopProcessing = true
 		return err
 	}
 
-	enabled := handler.config.EnableParentClosePolicy(parentNamespace)
+	enabled := handler.config.EnableParentClosePolicy(parentNamespace.String())
 	if enabled {
 		enums.SetDefaultParentClosePolicy(&attr.ParentClosePolicy)
 	} else {
@@ -838,14 +831,14 @@ func (handler *workflowTaskHandlerImpl) handleCommandSignalExternalWorkflow(
 	)
 
 	executionInfo := handler.mutableState.GetExecutionInfo()
-	namespaceID := executionInfo.NamespaceId
+	namespaceID := namespace.ID(executionInfo.NamespaceId)
 	targetNamespaceID := namespaceID
 	if attr.GetNamespace() != "" {
-		targetNamespaceEntry, err := handler.namespaceCache.GetNamespace(attr.GetNamespace())
+		targetNamespaceEntry, err := handler.namespaceRegistry.GetNamespace(namespace.Name(attr.GetNamespace()))
 		if err != nil {
-			return serviceerror.NewInternal(fmt.Sprintf("Unable to signal workflow across namespace: %v.", attr.GetNamespace()))
+			return serviceerror.NewUnavailable(fmt.Sprintf("Unable to signal workflow across namespace: %v.", attr.GetNamespace()))
 		}
-		targetNamespaceID = targetNamespaceEntry.GetInfo().Id
+		targetNamespaceID = targetNamespaceEntry.ID()
 	}
 
 	if err := handler.validateCommandAttr(
@@ -889,12 +882,12 @@ func (handler *workflowTaskHandlerImpl) handleCommandUpsertWorkflowSearchAttribu
 
 	// get namespace name
 	executionInfo := handler.mutableState.GetExecutionInfo()
-	namespaceID := executionInfo.NamespaceId
-	namespaceEntry, err := handler.namespaceCache.GetNamespaceByID(namespaceID)
+	namespaceID := namespace.ID(executionInfo.NamespaceId)
+	namespaceEntry, err := handler.namespaceRegistry.GetNamespaceByID(namespaceID)
 	if err != nil {
-		return serviceerror.NewInternal(fmt.Sprintf("Unable to get namespace for namespaceID: %v.", namespaceID))
+		return serviceerror.NewUnavailable(fmt.Sprintf("Unable to get namespace for namespaceID: %v.", namespaceID))
 	}
-	namespace := namespaceEntry.GetInfo().Name
+	namespace := namespaceEntry.Name()
 
 	// valid search attributes for upsert
 	if err := handler.validateCommandAttr(
@@ -931,7 +924,7 @@ func (handler *workflowTaskHandlerImpl) handleCommandUpsertWorkflowSearchAttribu
 		return err
 	}
 
-	if err := searchattribute.SubstituteAliases(handler.searchAttributesMapper, attr.GetSearchAttributes(), namespace); err != nil {
+	if err := searchattribute.SubstituteAliases(handler.searchAttributesMapper, attr.GetSearchAttributes(), namespace.String()); err != nil {
 		handler.stopProcessing = true
 		return err
 	}
@@ -952,42 +945,83 @@ func searchAttributesSize(fields map[string]*commonpb.Payload) int {
 	return result
 }
 
-func (handler *workflowTaskHandlerImpl) retryCronContinueAsNew(
-	attr *historypb.WorkflowExecutionStartedEventAttributes,
+func (handler *workflowTaskHandlerImpl) handleRetry(
 	backoffInterval time.Duration,
-	continueAsNewInitiator enumspb.ContinueAsNewInitiator,
+	retryState enumspb.RetryState,
 	failure *failurepb.Failure,
-	lastCompletionResult *commonpb.Payloads,
+	newRunID string,
 ) error {
-
-	continueAsNewAttributes := &commandpb.ContinueAsNewWorkflowExecutionCommandAttributes{
-		WorkflowType:         attr.WorkflowType,
-		TaskQueue:            attr.TaskQueue,
-		RetryPolicy:          attr.RetryPolicy,
-		Input:                attr.Input,
-		WorkflowRunTimeout:   attr.WorkflowRunTimeout,
-		WorkflowTaskTimeout:  attr.WorkflowTaskTimeout,
-		CronSchedule:         attr.CronSchedule,
-		BackoffStartInterval: &backoffInterval,
-		Initiator:            continueAsNewInitiator,
-		Failure:              failure,
-		LastCompletionResult: lastCompletionResult,
-		Header:               attr.Header,
-		Memo:                 attr.Memo,
-		SearchAttributes:     attr.SearchAttributes,
+	startEvent, err := handler.mutableState.GetStartEvent()
+	if err != nil {
+		return err
 	}
+	startAttr := startEvent.GetWorkflowExecutionStartedEventAttributes()
 
-	_, newStateBuilder, err := handler.mutableState.AddContinueAsNewEvent(
-		handler.workflowTaskCompletedID,
-		handler.workflowTaskCompletedID,
-		attr.GetParentWorkflowNamespace(),
-		continueAsNewAttributes,
+	newStateBuilder, err := createMutableState(
+		handler.shard,
+		handler.mutableState.GetNamespaceEntry(),
+		newRunID,
+	)
+	if err != nil {
+		return err
+	}
+	err = workflow.SetupNewWorkflowForRetryOrCron(
+		handler.mutableState,
+		newStateBuilder,
+		newRunID,
+		startAttr,
+		nil,
+		failure,
+		backoffInterval,
+		enumspb.CONTINUE_AS_NEW_INITIATOR_RETRY,
 	)
 	if err != nil {
 		return err
 	}
 
-	handler.continueAsNewBuilder = newStateBuilder
+	handler.newStateBuilder = newStateBuilder
+	return nil
+}
+
+func (handler *workflowTaskHandlerImpl) handleCron(
+	backoffInterval time.Duration,
+	lastCompletionResult *commonpb.Payloads,
+	failure *failurepb.Failure,
+	newRunID string,
+) error {
+	startEvent, err := handler.mutableState.GetStartEvent()
+	if err != nil {
+		return err
+	}
+	startAttr := startEvent.GetWorkflowExecutionStartedEventAttributes()
+
+	if failure != nil {
+		lastCompletionResult = startAttr.LastCompletionResult
+	}
+
+	newStateBuilder, err := createMutableState(
+		handler.shard,
+		handler.mutableState.GetNamespaceEntry(),
+		newRunID,
+	)
+	if err != nil {
+		return err
+	}
+	err = workflow.SetupNewWorkflowForRetryOrCron(
+		handler.mutableState,
+		newStateBuilder,
+		newRunID,
+		startAttr,
+		lastCompletionResult,
+		failure,
+		backoffInterval,
+		enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE,
+	)
+	if err != nil {
+		return err
+	}
+
+	handler.newStateBuilder = newStateBuilder
 	return nil
 }
 
