@@ -34,16 +34,17 @@ import (
 	querypb "go.temporal.io/api/query/v1"
 	"go.temporal.io/api/serviceerror"
 	taskqueuepb "go.temporal.io/api/taskqueue/v1"
+
 	"go.temporal.io/server/common/searchattribute"
 
 	historyspb "go.temporal.io/server/api/history/v1"
 	"go.temporal.io/server/api/historyservice/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/payloads"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/primitives/timestamp"
@@ -73,8 +74,8 @@ type (
 		shard                  shard.Context
 		timeSource             clock.TimeSource
 		historyEngine          *historyEngineImpl
-		namespaceCache         cache.NamespaceCache
-		historyCache           *workflow.Cache
+		namespaceRegistry      namespace.Registry
+		historyCache           workflow.Cache
 		txProcessor            transferQueueProcessor
 		timerProcessor         timerQueueProcessor
 		tokenSerializer        common.TaskTokenSerializer
@@ -93,7 +94,7 @@ func newWorkflowTaskHandlerCallback(historyEngine *historyEngineImpl) *workflowT
 		shard:              historyEngine.shard,
 		timeSource:         historyEngine.shard.GetTimeSource(),
 		historyEngine:      historyEngine,
-		namespaceCache:     historyEngine.shard.GetNamespaceCache(),
+		namespaceRegistry:  historyEngine.shard.GetNamespaceRegistry(),
 		historyCache:       historyEngine.historyCache,
 		txProcessor:        historyEngine.txProcessor,
 		timerProcessor:     historyEngine.timerProcessor,
@@ -102,7 +103,7 @@ func newWorkflowTaskHandlerCallback(historyEngine *historyEngineImpl) *workflowT
 		logger:             historyEngine.logger,
 		throttledLogger:    historyEngine.throttledLogger,
 		commandAttrValidator: newCommandAttrValidator(
-			historyEngine.shard.GetNamespaceCache(),
+			historyEngine.shard.GetNamespaceRegistry(),
 			historyEngine.config,
 			historyEngine.searchAttributesValidator,
 		),
@@ -115,11 +116,11 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleWorkflowTaskScheduled(
 	req *historyservice.ScheduleWorkflowTaskRequest,
 ) error {
 
-	namespaceEntry, err := handler.historyEngine.getActiveNamespaceEntry(req.GetNamespaceId())
+	namespaceEntry, err := handler.historyEngine.getActiveNamespaceEntry(namespace.ID(req.GetNamespaceId()))
 	if err != nil {
 		return err
 	}
-	namespaceID := namespaceEntry.GetInfo().Id
+	namespaceID := namespaceEntry.ID()
 
 	execution := commonpb.WorkflowExecution{
 		WorkflowId: req.WorkflowExecution.WorkflowId,
@@ -160,11 +161,11 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleWorkflowTaskStarted(
 	req *historyservice.RecordWorkflowTaskStartedRequest,
 ) (*historyservice.RecordWorkflowTaskStartedResponse, error) {
 
-	namespaceEntry, err := handler.historyEngine.getActiveNamespaceEntry(req.GetNamespaceId())
+	namespaceEntry, err := handler.historyEngine.getActiveNamespaceEntry(namespace.ID(req.GetNamespaceId()))
 	if err != nil {
 		return nil, err
 	}
-	namespaceID := namespaceEntry.GetInfo().Id
+	namespaceID := namespaceEntry.ID()
 
 	execution := commonpb.WorkflowExecution{
 		WorkflowId: req.WorkflowExecution.WorkflowId,
@@ -185,11 +186,12 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleWorkflowTaskStarted(
 			}
 
 			workflowTask, isRunning := mutableState.GetWorkflowTaskInfo(scheduleID)
+			metricsScope := handler.metricsClient.Scope(metrics.HistoryRecordWorkflowTaskStartedScope)
 
 			// First check to see if cache needs to be refreshed as we could potentially have stale workflow execution in
 			// some extreme cassandra failure cases.
 			if !isRunning && scheduleID >= mutableState.GetNextEventID() {
-				handler.metricsClient.IncCounter(metrics.HistoryRecordWorkflowTaskStartedScope, metrics.StaleMutableStateCounter)
+				metricsScope.IncCounter(metrics.StaleMutableStateCounter)
 				// Reload workflow execution history
 				// ErrStaleState will trigger updateWorkflowExecution function to reload the mutable state
 				return nil, consts.ErrStaleState
@@ -208,7 +210,7 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleWorkflowTaskStarted(
 			if workflowTask.StartedID != common.EmptyEventID {
 				// If workflow task is started as part of the current request scope then return a positive response
 				if workflowTask.RequestID == requestID {
-					resp, err = handler.createRecordWorkflowTaskStartedResponse(namespaceID, mutableState, workflowTask, req.PollRequest.GetIdentity())
+					resp, err = handler.createRecordWorkflowTaskStartedResponse(mutableState, workflowTask, req.PollRequest.GetIdentity())
 					if err != nil {
 						return nil, err
 					}
@@ -229,10 +231,17 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleWorkflowTaskStarted(
 			)
 			if err != nil {
 				// Unable to add WorkflowTaskStarted event to history
-				return nil, serviceerror.NewInternal("Unable to add WorkflowTaskStarted event to history.")
+				return nil, err
 			}
 
-			resp, err = handler.createRecordWorkflowTaskStartedResponse(namespaceID, mutableState, workflowTask, req.PollRequest.GetIdentity())
+			workflowScheduleToStartLatency := workflowTask.StartedTime.Sub(*workflowTask.ScheduledTime)
+			namespaceName := namespaceEntry.Name()
+			taskQueue := workflowTask.TaskQueue
+			metrics.GetPerTaskQueueScope(metricsScope, namespaceName.String(), taskQueue.GetName(), taskQueue.GetKind()).
+				Tagged(metrics.TaskTypeTag("workflow")).
+				RecordTimer(metrics.TaskScheduleToStartLatency, workflowScheduleToStartLatency)
+
+			resp, err = handler.createRecordWorkflowTaskStartedResponse(mutableState, workflowTask, req.PollRequest.GetIdentity())
 			if err != nil {
 				return nil, err
 			}
@@ -250,11 +259,11 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleWorkflowTaskFailed(
 	req *historyservice.RespondWorkflowTaskFailedRequest,
 ) (retError error) {
 
-	namespaceEntry, err := handler.historyEngine.getActiveNamespaceEntry(req.GetNamespaceId())
+	namespaceEntry, err := handler.historyEngine.getActiveNamespaceEntry(namespace.ID(req.GetNamespaceId()))
 	if err != nil {
 		return err
 	}
-	namespaceID := namespaceEntry.GetInfo().Id
+	namespaceID := namespaceEntry.ID()
 
 	request := req.FailedRequest
 	token, err := handler.tokenSerializer.Deserialize(request.TaskToken)
@@ -299,11 +308,11 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleWorkflowTaskCompleted(
 	req *historyservice.RespondWorkflowTaskCompletedRequest,
 ) (resp *historyservice.RespondWorkflowTaskCompletedResponse, retError error) {
 
-	namespaceEntry, err := handler.historyEngine.getActiveNamespaceEntry(req.GetNamespaceId())
+	namespaceEntry, err := handler.historyEngine.getActiveNamespaceEntry(namespace.ID(req.GetNamespaceId()))
 	if err != nil {
 		return nil, err
 	}
-	namespaceID := namespaceEntry.GetInfo().Id
+	namespaceID := namespaceEntry.ID()
 
 	request := req.CompleteRequest
 	token, err0 := handler.tokenSerializer.Deserialize(request.TaskToken)
@@ -361,7 +370,7 @@ Update_History_Loop:
 		}
 
 		startedID := currentWorkflowTask.StartedID
-		maxResetPoints := handler.config.MaxAutoResetPoints(namespaceEntry.GetInfo().Name)
+		maxResetPoints := handler.config.MaxAutoResetPoints(namespaceEntry.Name().String())
 		if msBuilder.GetExecutionInfo().AutoResetPoints != nil && maxResetPoints == len(msBuilder.GetExecutionInfo().AutoResetPoints.Points) {
 			handler.metricsClient.IncCounter(metrics.HistoryRespondWorkflowTaskCompletedScope, metrics.AutoResetPointsLimitExceededCounter)
 		}
@@ -370,35 +379,35 @@ Update_History_Loop:
 		var workflowTaskHeartbeatTimeout bool
 		var completedEvent *historypb.HistoryEvent
 		if workflowTaskHeartbeating {
-			namespace := namespaceEntry.GetInfo().Name
-			timeout := handler.config.WorkflowTaskHeartbeatTimeout(namespace)
+			namespace := namespaceEntry.Name()
+			timeout := handler.config.WorkflowTaskHeartbeatTimeout(namespace.String())
 			origSchedTime := timestamp.TimeValue(currentWorkflowTask.OriginalScheduledTime)
 			if origSchedTime.UnixNano() > 0 && handler.timeSource.Now().After(origSchedTime.Add(timeout)) {
 				workflowTaskHeartbeatTimeout = true
-				scope := handler.metricsClient.Scope(metrics.HistoryRespondWorkflowTaskCompletedScope, metrics.NamespaceTag(namespace))
+				scope := handler.metricsClient.Scope(metrics.HistoryRespondWorkflowTaskCompletedScope, metrics.NamespaceTag(namespace.String()))
 				scope.IncCounter(metrics.WorkflowTaskHeartbeatTimeoutCounter)
 				completedEvent, err = msBuilder.AddWorkflowTaskTimedOutEvent(currentWorkflowTask.ScheduleID, currentWorkflowTask.StartedID)
 				if err != nil {
-					return nil, serviceerror.NewInternal("Failed to add workflow task timeout event.")
+					return nil, err
 				}
 				msBuilder.ClearStickyness()
 			} else {
 				completedEvent, err = msBuilder.AddWorkflowTaskCompletedEvent(scheduleID, startedID, request, maxResetPoints)
 				if err != nil {
-					return nil, serviceerror.NewInternal("Unable to add WorkflowTaskCompleted event to history.")
+					return nil, err
 				}
 			}
 		} else {
 			completedEvent, err = msBuilder.AddWorkflowTaskCompletedEvent(scheduleID, startedID, request, maxResetPoints)
 			if err != nil {
-				return nil, serviceerror.NewInternal("Unable to add WorkflowTaskCompleted event to history.")
+				return nil, err
 			}
 		}
 
 		var (
 			wtFailedCause               *workflowTaskFailedCause
 			activityNotStartedCancelled bool
-			continueAsNewBuilder        workflow.MutableState
+			newStateBuilder             workflow.MutableState
 
 			hasUnhandledEvents bool
 		)
@@ -415,24 +424,24 @@ Update_History_Loop:
 		}
 
 		binChecksum := request.GetBinaryChecksum()
-		if _, ok := namespaceEntry.GetConfig().GetBadBinaries().GetBinaries()[binChecksum]; ok {
+		if err := namespaceEntry.VerifyBinaryChecksum(binChecksum); err != nil {
 			wtFailedCause = NewWorkflowTaskFailedCause(enumspb.WORKFLOW_TASK_FAILED_CAUSE_BAD_BINARY, serviceerror.NewInvalidArgument(fmt.Sprintf("binary %v is already marked as bad deployment", binChecksum)))
 		} else {
-			namespace := namespaceEntry.GetInfo().Name
+			namespace := namespaceEntry.Name()
 			workflowSizeChecker := newWorkflowSizeChecker(
-				handler.config.BlobSizeLimitWarn(namespace),
-				handler.config.BlobSizeLimitError(namespace),
-				handler.config.MemoSizeLimitWarn(namespace),
-				handler.config.MemoSizeLimitError(namespace),
-				handler.config.HistorySizeLimitWarn(namespace),
-				handler.config.HistorySizeLimitError(namespace),
-				handler.config.HistoryCountLimitWarn(namespace),
-				handler.config.HistoryCountLimitError(namespace),
+				handler.config.BlobSizeLimitWarn(namespace.String()),
+				handler.config.BlobSizeLimitError(namespace.String()),
+				handler.config.MemoSizeLimitWarn(namespace.String()),
+				handler.config.MemoSizeLimitError(namespace.String()),
+				handler.config.HistorySizeLimitWarn(namespace.String()),
+				handler.config.HistorySizeLimitError(namespace.String()),
+				handler.config.HistoryCountLimitWarn(namespace.String()),
+				handler.config.HistoryCountLimitError(namespace.String()),
 				completedEvent.GetEventId(),
 				msBuilder,
 				handler.historyEngine.searchAttributesValidator,
 				executionStats,
-				handler.metricsClient.Scope(metrics.HistoryRespondWorkflowTaskCompletedScope, metrics.NamespaceTag(namespace)),
+				handler.metricsClient.Scope(metrics.HistoryRespondWorkflowTaskCompletedScope, metrics.NamespaceTag(namespace.String())),
 				handler.throttledLogger,
 			)
 
@@ -443,9 +452,10 @@ Update_History_Loop:
 				handler.commandAttrValidator,
 				workflowSizeChecker,
 				handler.logger,
-				handler.namespaceCache,
+				handler.namespaceRegistry,
 				handler.metricsClient,
 				handler.config,
+				handler.shard,
 				handler.searchAttributesMapper,
 			)
 
@@ -463,7 +473,7 @@ Update_History_Loop:
 			activityNotStartedCancelled = workflowTaskHandler.activityNotStartedCancelled
 			// continueAsNewTimerTasks is not used by workflowTaskHandlerCallbacks
 
-			continueAsNewBuilder = workflowTaskHandler.continueAsNewBuilder
+			newStateBuilder = workflowTaskHandler.newStateBuilder
 
 			hasUnhandledEvents = workflowTaskHandler.hasBufferedEvents
 		}
@@ -474,13 +484,13 @@ Update_History_Loop:
 				tag.Value(wtFailedCause.Message()),
 				tag.WorkflowID(token.GetWorkflowId()),
 				tag.WorkflowRunID(token.GetRunId()),
-				tag.WorkflowNamespaceID(namespaceID))
+				tag.WorkflowNamespaceID(namespaceID.String()))
 			msBuilder, err = handler.historyEngine.failWorkflowTask(weContext, scheduleID, startedID, wtFailedCause, request)
 			if err != nil {
 				return nil, err
 			}
 			hasUnhandledEvents = true
-			continueAsNewBuilder = nil
+			newStateBuilder = nil
 		}
 
 		createNewWorkflowTask := msBuilder.IsWorkflowExecutionRunning() && (hasUnhandledEvents || request.GetForceCreateNewWorkflowTask() || activityNotStartedCancelled)
@@ -499,7 +509,7 @@ Update_History_Loop:
 				)
 			}
 			if err != nil {
-				return nil, serviceerror.NewInternal("Failed to add workflow task scheduled event.")
+				return nil, err
 			}
 
 			newWorkflowTaskScheduledID = newWorkflowTask.ScheduleID
@@ -522,21 +532,21 @@ Update_History_Loop:
 		// We apply the update to execution using optimistic concurrency.  If it fails due to a conflict then reload
 		// the history and try the operation again.
 		var updateErr error
-		if continueAsNewBuilder != nil {
-			continueAsNewExecutionInfo := continueAsNewBuilder.GetExecutionInfo()
-			continueAsNewExecutionState := continueAsNewBuilder.GetExecutionState()
+		if newStateBuilder != nil {
+			newWorkflowExecutionInfo := newStateBuilder.GetExecutionInfo()
+			newWorkflowExecutionState := newStateBuilder.GetExecutionState()
 			updateErr = weContext.UpdateWorkflowExecutionWithNewAsActive(
 				handler.shard.GetTimeSource().Now(),
 				workflow.NewContext(
-					continueAsNewExecutionInfo.NamespaceId,
+					namespace.ID(newWorkflowExecutionInfo.NamespaceId),
 					commonpb.WorkflowExecution{
-						WorkflowId: continueAsNewExecutionInfo.WorkflowId,
-						RunId:      continueAsNewExecutionState.RunId,
+						WorkflowId: newWorkflowExecutionInfo.WorkflowId,
+						RunId:      newWorkflowExecutionState.RunId,
 					},
 					handler.shard,
 					handler.logger,
 				),
-				continueAsNewBuilder,
+				newStateBuilder,
 			)
 		} else {
 			updateErr = weContext.UpdateWorkflowExecutionAsActive(handler.shard.GetTimeSource().Now())
@@ -592,7 +602,7 @@ Update_History_Loop:
 		resp = &historyservice.RespondWorkflowTaskCompletedResponse{}
 		if request.GetReturnNewWorkflowTask() && createNewWorkflowTask {
 			workflowTask, _ := msBuilder.GetWorkflowTaskInfo(newWorkflowTaskScheduledID)
-			resp.StartedResponse, err = handler.createRecordWorkflowTaskStartedResponse(namespaceID, msBuilder, workflowTask, request.GetIdentity())
+			resp.StartedResponse, err = handler.createRecordWorkflowTaskStartedResponse(msBuilder, workflowTask, request.GetIdentity())
 			if err != nil {
 				return nil, err
 			}
@@ -607,7 +617,6 @@ Update_History_Loop:
 }
 
 func (handler *workflowTaskHandlerCallbacksImpl) createRecordWorkflowTaskStartedResponse(
-	namespaceID string,
 	msBuilder workflow.MutableState,
 	workflowTask *workflow.WorkflowTaskInfo,
 	identity string,
@@ -662,20 +671,19 @@ func (handler *workflowTaskHandlerCallbacksImpl) createRecordWorkflowTaskStarted
 	return response, nil
 }
 
-func (handler *workflowTaskHandlerCallbacksImpl) handleBufferedQueries(msBuilder workflow.MutableState, queryResults map[string]*querypb.WorkflowQueryResult, createNewWorkflowTask bool, namespaceEntry *cache.NamespaceCacheEntry, workflowTaskHeartbeating bool) {
+func (handler *workflowTaskHandlerCallbacksImpl) handleBufferedQueries(msBuilder workflow.MutableState, queryResults map[string]*querypb.WorkflowQueryResult, createNewWorkflowTask bool, namespaceEntry *namespace.Namespace, workflowTaskHeartbeating bool) {
 	queryRegistry := msBuilder.GetQueryRegistry()
 	if !queryRegistry.HasBufferedQuery() {
 		return
 	}
 
-	namespaceID := namespaceEntry.GetInfo().Id
-	namespace := namespaceEntry.GetInfo().Name
+	namespaceName := namespaceEntry.Name()
 	workflowID := msBuilder.GetExecutionInfo().WorkflowId
 	runID := msBuilder.GetExecutionState().GetRunId()
 
 	scope := handler.metricsClient.Scope(
 		metrics.HistoryRespondWorkflowTaskCompletedScope,
-		metrics.NamespaceTag(namespaceEntry.GetInfo().Name),
+		metrics.NamespaceTag(namespaceEntry.Name().String()),
 		metrics.CommandTypeTag("ConsistentQuery"))
 
 	// if its a heartbeat workflow task it means local activities may still be running on the worker
@@ -684,8 +692,8 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleBufferedQueries(msBuilder
 		return
 	}
 
-	sizeLimitError := handler.config.BlobSizeLimitError(namespace)
-	sizeLimitWarn := handler.config.BlobSizeLimitWarn(namespace)
+	sizeLimitError := handler.config.BlobSizeLimitError(namespaceName.String())
+	sizeLimitWarn := handler.config.BlobSizeLimitWarn(namespaceName.String())
 
 	// Complete or fail all queries we have results for
 	for id, result := range queryResults {
@@ -693,7 +701,7 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleBufferedQueries(msBuilder
 			result.GetAnswer().Size(),
 			sizeLimitWarn,
 			sizeLimitError,
-			namespaceID,
+			namespaceName.String(),
 			workflowID,
 			runID,
 			scope,
@@ -701,7 +709,7 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleBufferedQueries(msBuilder
 			tag.BlobSizeViolationOperation("ConsistentQuery"),
 		); err != nil {
 			handler.logger.Info("failing query because query result size is too large",
-				tag.WorkflowNamespace(namespace),
+				tag.WorkflowNamespace(namespaceName.String()),
 				tag.WorkflowID(workflowID),
 				tag.WorkflowRunID(runID),
 				tag.QueryID(id),
@@ -713,7 +721,7 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleBufferedQueries(msBuilder
 			if err := queryRegistry.SetTerminationState(id, failedTerminationState); err != nil {
 				handler.logger.Error(
 					"failed to set query termination state to failed",
-					tag.WorkflowNamespace(namespace),
+					tag.WorkflowNamespace(namespaceName.String()),
 					tag.WorkflowID(workflowID),
 					tag.WorkflowRunID(runID),
 					tag.QueryID(id),
@@ -728,7 +736,7 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleBufferedQueries(msBuilder
 			if err := queryRegistry.SetTerminationState(id, completedTerminationState); err != nil {
 				handler.logger.Error(
 					"failed to set query termination state to completed",
-					tag.WorkflowNamespace(namespace),
+					tag.WorkflowNamespace(namespaceName.String()),
 					tag.WorkflowID(workflowID),
 					tag.WorkflowRunID(runID),
 					tag.QueryID(id),
@@ -749,7 +757,7 @@ func (handler *workflowTaskHandlerCallbacksImpl) handleBufferedQueries(msBuilder
 			if err := queryRegistry.SetTerminationState(id, unblockTerminationState); err != nil {
 				handler.logger.Error(
 					"failed to set query termination state to unblocked",
-					tag.WorkflowNamespace(namespace),
+					tag.WorkflowNamespace(namespaceName.String()),
 					tag.WorkflowID(workflowID),
 					tag.WorkflowRunID(runID),
 					tag.QueryID(id),
