@@ -42,6 +42,7 @@ import (
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/convert"
+	"go.temporal.io/server/common/namespace"
 )
 
 type ForwarderTestSuite struct {
@@ -88,7 +89,7 @@ func (t *ForwarderTestSuite) TestForwardWorkflowTask() {
 
 	var request *matchingservice.AddWorkflowTaskRequest
 	t.client.EXPECT().AddWorkflowTask(gomock.Any(), gomock.Any(), gomock.Any()).Do(
-		func(arg0 context.Context, arg1 *matchingservice.AddWorkflowTaskRequest) {
+		func(arg0 context.Context, arg1 *matchingservice.AddWorkflowTaskRequest, arg2 ...interface{}) {
 			request = arg1
 		},
 	).Return(&matchingservice.AddWorkflowTaskResponse{}, nil)
@@ -115,7 +116,7 @@ func (t *ForwarderTestSuite) TestForwardActivityTask() {
 
 	var request *matchingservice.AddActivityTaskRequest
 	t.client.EXPECT().AddActivityTask(gomock.Any(), gomock.Any(), gomock.Any()).Do(
-		func(arg0 context.Context, arg1 *matchingservice.AddActivityTaskRequest) {
+		func(arg0 context.Context, arg1 *matchingservice.AddActivityTaskRequest, arg2 ...interface{}) {
 			request = arg1
 		},
 	).Return(&matchingservice.AddActivityTaskResponse{}, nil)
@@ -126,7 +127,7 @@ func (t *ForwarderTestSuite) TestForwardActivityTask() {
 	t.NotNil(request)
 	t.Equal(t.taskQueue.Parent(20), request.TaskQueue.GetName())
 	t.Equal(t.fwdr.taskQueueKind, request.TaskQueue.GetKind())
-	t.Equal(t.taskQueue.namespaceID, request.GetNamespaceId())
+	t.Equal(t.taskQueue.namespaceID, namespace.ID(request.GetNamespaceId()))
 	t.Equal(taskInfo.Data.GetNamespaceId(), request.GetSourceNamespaceId())
 	t.Equal(taskInfo.Data.GetWorkflowId(), request.GetExecution().GetWorkflowId())
 	t.Equal(taskInfo.Data.GetRunId(), request.GetExecution().GetRunId())
@@ -165,8 +166,8 @@ func (t *ForwarderTestSuite) TestForwardQueryTask() {
 	task := newInternalQueryTask("id1", &matchingservice.QueryWorkflowRequest{})
 	resp := &matchingservice.QueryWorkflowResponse{}
 	var request *matchingservice.QueryWorkflowRequest
-	t.client.EXPECT().QueryWorkflow(gomock.Any(), gomock.Any()).Do(
-		func(arg0 context.Context, arg1 *matchingservice.QueryWorkflowRequest) {
+	t.client.EXPECT().QueryWorkflow(gomock.Any(), gomock.Any(), gomock.Any()).Do(
+		func(arg0 context.Context, arg1 *matchingservice.QueryWorkflowRequest, arg2 ...interface{}) {
 			request = arg1
 		},
 	).Return(resp, nil)
@@ -213,8 +214,8 @@ func (t *ForwarderTestSuite) TestForwardPollWorkflowTaskQueue() {
 	resp := &matchingservice.PollWorkflowTaskQueueResponse{}
 
 	var request *matchingservice.PollWorkflowTaskQueueRequest
-	t.client.EXPECT().PollWorkflowTaskQueue(gomock.Any(), gomock.Any()).Do(
-		func(arg0 context.Context, arg1 *matchingservice.PollWorkflowTaskQueueRequest) {
+	t.client.EXPECT().PollWorkflowTaskQueue(gomock.Any(), gomock.Any(), gomock.Any()).Do(
+		func(arg0 context.Context, arg1 *matchingservice.PollWorkflowTaskQueueRequest, arg2 ...interface{}) {
 			request = arg1
 		},
 	).Return(resp, nil)
@@ -224,7 +225,7 @@ func (t *ForwarderTestSuite) TestForwardPollWorkflowTaskQueue() {
 	t.NotNil(task)
 	t.NotNil(request)
 	t.Equal(pollerID, request.GetPollerId())
-	t.Equal(t.taskQueue.namespaceID, request.GetNamespaceId())
+	t.Equal(t.taskQueue.namespaceID, namespace.ID(request.GetNamespaceId()))
 	t.Equal("id1", request.GetPollRequest().GetIdentity())
 	t.Equal(t.taskQueue.Parent(20), request.GetPollRequest().GetTaskQueue().GetName())
 	t.Equal(enumspb.TaskQueueKind(t.fwdr.taskQueueKind), request.GetPollRequest().GetTaskQueue().GetKind())
@@ -241,8 +242,8 @@ func (t *ForwarderTestSuite) TestForwardPollForActivity() {
 	resp := &matchingservice.PollActivityTaskQueueResponse{}
 
 	var request *matchingservice.PollActivityTaskQueueRequest
-	t.client.EXPECT().PollActivityTaskQueue(gomock.Any(), gomock.Any()).Do(
-		func(arg0 context.Context, arg1 *matchingservice.PollActivityTaskQueueRequest) {
+	t.client.EXPECT().PollActivityTaskQueue(gomock.Any(), gomock.Any(), gomock.Any()).Do(
+		func(arg0 context.Context, arg1 *matchingservice.PollActivityTaskQueueRequest, arg2 ...interface{}) {
 			request = arg1
 		},
 	).Return(resp, nil)
@@ -252,7 +253,7 @@ func (t *ForwarderTestSuite) TestForwardPollForActivity() {
 	t.NotNil(task)
 	t.NotNil(request)
 	t.Equal(pollerID, request.GetPollerId())
-	t.Equal(t.taskQueue.namespaceID, request.GetNamespaceId())
+	t.Equal(t.taskQueue.namespaceID, namespace.ID(request.GetNamespaceId()))
 	t.Equal("id1", request.GetPollRequest().GetIdentity())
 	t.Equal(t.taskQueue.Parent(20), request.GetPollRequest().GetTaskQueue().GetName())
 	t.Equal(enumspb.TaskQueueKind(t.fwdr.taskQueueKind), request.GetPollRequest().GetTaskQueue().GetKind())
