@@ -49,16 +49,14 @@ type TaskMatcher struct {
 	// not active in a cluster
 	queryTaskC chan *internalTask
 
-	// dynamicRate is the dynamic rate for rate limiter
-	dynamicRate quotas.DynamicRate
-	// dynamicBurst is the dynamic burst for rate limiter
-	dynamicBurst quotas.DynamicBurst
+	// dynamicRate is the dynamic rate & burst for rate limiter
+	dynamicRateBurst quotas.MutableRateBurst
 	// rateLimiter that limits the rate at which tasks can be dispatched to consumers
 	rateLimiter quotas.RateLimiter
 
 	fwdr          *Forwarder
-	scope         func() metrics.Scope // namespace metric scope
-	numPartitions func() int           // number of task queue partitions
+	scope         metrics.Scope // namespace metric scope
+	numPartitions func() int    // number of task queue partitions
 }
 
 const (
@@ -69,32 +67,32 @@ const (
 // newTaskMatcher returns an task matcher instance. The returned instance can be
 // used by task producers and consumers to find a match. Both sync matches and non-sync
 // matches should use this implementation
-func newTaskMatcher(config *taskQueueConfig, fwdr *Forwarder, scopeFunc func() metrics.Scope) *TaskMatcher {
-	dynamicRate := quotas.NewDynamicRate(defaultTaskDispatchRPS)
-	dynamicBurst := quotas.NewDynamicBurst(int(defaultTaskDispatchRPS))
+func newTaskMatcher(config *taskQueueConfig, fwdr *Forwarder, scope metrics.Scope) *TaskMatcher {
+	dynamicRateBurst := quotas.NewMutableRateBurst(
+		defaultTaskDispatchRPS,
+		int(defaultTaskDispatchRPS),
+	)
 	limiter := quotas.NewMultiRateLimiter([]quotas.RateLimiter{
 		quotas.NewDynamicRateLimiter(
-			dynamicRate.RateFn(),
-			dynamicBurst.BurstFn(),
+			dynamicRateBurst,
 			defaultTaskDispatchRPSTTL,
 		),
-		quotas.NewDefaultOutgoingDynamicRateLimiter(
+		quotas.NewDefaultOutgoingRateLimiter(
 			config.AdminNamespaceTaskQueueToPartitionDispatchRate,
 		),
-		quotas.NewDefaultOutgoingDynamicRateLimiter(
+		quotas.NewDefaultOutgoingRateLimiter(
 			config.AdminNamespaceToPartitionDispatchRate,
 		),
 	})
 	return &TaskMatcher{
-		config:        config,
-		dynamicRate:   dynamicRate,
-		dynamicBurst:  dynamicBurst,
-		rateLimiter:   limiter,
-		scope:         scopeFunc,
-		fwdr:          fwdr,
-		taskC:         make(chan *internalTask),
-		queryTaskC:    make(chan *internalTask),
-		numPartitions: config.NumReadPartitions,
+		config:           config,
+		dynamicRateBurst: dynamicRateBurst,
+		rateLimiter:      limiter,
+		scope:            scope,
+		fwdr:             fwdr,
+		taskC:            make(chan *internalTask),
+		queryTaskC:       make(chan *internalTask),
+		numPartitions:    config.NumReadPartitions,
 	}
 }
 
@@ -130,7 +128,7 @@ func newTaskMatcher(config *taskQueueConfig, fwdr *Forwarder, scopeFunc func() m
 func (tm *TaskMatcher) Offer(ctx context.Context, task *internalTask) (bool, error) {
 	if !task.isForwarded() {
 		if err := tm.rateLimiter.Wait(ctx); err != nil {
-			tm.scope().IncCounter(metrics.SyncThrottlePerTaskQueueCounter)
+			tm.scope.IncCounter(metrics.SyncThrottlePerTaskQueueCounter)
 			return false, err
 		}
 	}
@@ -251,7 +249,7 @@ forLoop:
 			err := tm.fwdr.ForwardTask(childCtx, task)
 			token.release()
 			if err != nil {
-				tm.scope().IncCounter(metrics.ForwardTaskErrorsPerTaskQueue)
+				tm.scope.IncCounter(metrics.ForwardTaskErrorsPerTaskQueue)
 				// forwarder returns error only when the call is rate limited. To
 				// avoid a busy loop on such rate limiting events, we only attempt to make
 				// the next forwarded call after this childCtx expires. Till then, we block
@@ -312,8 +310,8 @@ func (tm *TaskMatcher) UpdateRatelimit(rps *float64) {
 		burst = minTaskThrottlingBurstSize
 	}
 
-	tm.dynamicRate.Store(rate)
-	tm.dynamicBurst.Store(burst)
+	tm.dynamicRateBurst.SetRate(rate)
+	tm.dynamicRateBurst.SetBurst(burst)
 }
 
 // Rate returns the current rate at which tasks are dispatched
@@ -341,7 +339,7 @@ func (tm *TaskMatcher) poll(ctx context.Context, queryOnly bool) (*internalTask,
 	// 1. ctx.Done
 	select {
 	case <-ctx.Done():
-		tm.scope().IncCounter(metrics.PollTimeoutPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollTimeoutPerTaskQueueCounter)
 		return nil, ErrNoTasks
 	default:
 	}
@@ -350,13 +348,13 @@ func (tm *TaskMatcher) poll(ctx context.Context, queryOnly bool) (*internalTask,
 	select {
 	case task := <-taskC:
 		if task.responseC != nil {
-			tm.scope().IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
+			tm.scope.IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
 		}
-		tm.scope().IncCounter(metrics.PollSuccessPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollSuccessPerTaskQueueCounter)
 		return task, nil
 	case task := <-queryTaskC:
-		tm.scope().IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
-		tm.scope().IncCounter(metrics.PollSuccessPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollSuccessPerTaskQueueCounter)
 		return task, nil
 	default:
 	}
@@ -364,17 +362,17 @@ func (tm *TaskMatcher) poll(ctx context.Context, queryOnly bool) (*internalTask,
 	// 3. forwarding (and all other clauses repeated again)
 	select {
 	case <-ctx.Done():
-		tm.scope().IncCounter(metrics.PollTimeoutPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollTimeoutPerTaskQueueCounter)
 		return nil, ErrNoTasks
 	case task := <-taskC:
 		if task.responseC != nil {
-			tm.scope().IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
+			tm.scope.IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
 		}
-		tm.scope().IncCounter(metrics.PollSuccessPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollSuccessPerTaskQueueCounter)
 		return task, nil
 	case task := <-queryTaskC:
-		tm.scope().IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
-		tm.scope().IncCounter(metrics.PollSuccessPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollSuccessPerTaskQueueCounter)
 		return task, nil
 	case token := <-tm.fwdrPollReqTokenC():
 		if task, err := tm.fwdr.ForwardPoll(ctx); err == nil {
@@ -387,17 +385,17 @@ func (tm *TaskMatcher) poll(ctx context.Context, queryOnly bool) (*internalTask,
 	// 4. blocking local poll
 	select {
 	case <-ctx.Done():
-		tm.scope().IncCounter(metrics.PollTimeoutPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollTimeoutPerTaskQueueCounter)
 		return nil, ErrNoTasks
 	case task := <-taskC:
 		if task.responseC != nil {
-			tm.scope().IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
+			tm.scope.IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
 		}
-		tm.scope().IncCounter(metrics.PollSuccessPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollSuccessPerTaskQueueCounter)
 		return task, nil
 	case task := <-queryTaskC:
-		tm.scope().IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
-		tm.scope().IncCounter(metrics.PollSuccessPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollSuccessWithSyncPerTaskQueueCounter)
+		tm.scope.IncCounter(metrics.PollSuccessPerTaskQueueCounter)
 		return task, nil
 	}
 }
