@@ -46,11 +46,12 @@ import (
 	tokenspb "go.temporal.io/server/api/token/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/backoff"
-	"go.temporal.io/server/common/cache"
+	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/membership"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/primitives/timestamp"
 	serviceerrors "go.temporal.io/server/common/serviceerror"
@@ -72,6 +73,12 @@ type (
 		queryTaskMap map[string]chan *queryResult
 	}
 
+	taskQueueCounterKey struct {
+		namespaceID namespace.ID
+		taskType    enumspb.TaskQueueType
+		queueType   enumspb.TaskQueueKind
+	}
+
 	matchingEngineImpl struct {
 		status               int32
 		taskManager          persistence.TaskManager
@@ -82,10 +89,12 @@ type (
 		metricsClient        metrics.Client
 		taskQueuesLock       sync.RWMutex                     // locks mutation of taskQueues
 		taskQueues           map[taskQueueID]taskQueueManager // Convert to LRU cache
+		taskQueueCount       map[taskQueueCounterKey]int      // per-namespace task queue counter
 		config               *Config
 		lockableQueryTaskMap lockableQueryTaskMap
-		namespaceCache       cache.NamespaceCache
+		namespaceRegistry    namespace.Registry
 		keyResolver          membership.ServiceResolver
+		clusterMeta          cluster.Metadata
 	}
 )
 
@@ -94,7 +103,7 @@ var (
 	emptyPollWorkflowTaskQueueResponse = &matchingservice.PollWorkflowTaskQueueResponse{}
 	// EmptyPollActivityTaskQueueResponse is the response when there are no activity tasks to hand out
 	emptyPollActivityTaskQueueResponse = &matchingservice.PollActivityTaskQueueResponse{}
-	persistenceOperationRetryPolicy    = common.CreatePersistanceRetryPolicy()
+	persistenceOperationRetryPolicy    = common.CreatePersistenceRetryPolicy()
 	historyServiceOperationRetryPolicy = common.CreateHistoryServiceRetryPolicy()
 
 	// ErrNoTasks is exported temporarily for integration test
@@ -114,8 +123,9 @@ func NewEngine(taskManager persistence.TaskManager,
 	config *Config,
 	logger log.Logger,
 	metricsClient metrics.Client,
-	namespaceCache cache.NamespaceCache,
+	namespaceRegistry namespace.Registry,
 	resolver membership.ServiceResolver,
+	clusterMeta cluster.Metadata,
 ) Engine {
 
 	return &matchingEngineImpl{
@@ -124,13 +134,15 @@ func NewEngine(taskManager persistence.TaskManager,
 		historyService:       historyService,
 		tokenSerializer:      common.NewProtoTaskTokenSerializer(),
 		taskQueues:           make(map[taskQueueID]taskQueueManager),
+		taskQueueCount:       make(map[taskQueueCounterKey]int),
 		logger:               log.With(logger, tag.ComponentMatchingEngine),
 		metricsClient:        metricsClient,
 		matchingClient:       matchingClient,
 		config:               config,
 		lockableQueryTaskMap: lockableQueryTaskMap{queryTaskMap: make(map[string]chan *queryResult)},
-		namespaceCache:       namespaceCache,
+		namespaceRegistry:    namespaceRegistry,
 		keyResolver:          resolver,
+		clusterMeta:          clusterMeta,
 	}
 }
 
@@ -199,14 +211,17 @@ func (e *matchingEngineImpl) getTaskQueueManager(taskQueue *taskQueueID, taskQue
 	if result, ok := e.taskQueues[*taskQueue]; ok {
 		return result, nil
 	}
-	mgr, err := newTaskQueueManager(e, taskQueue, taskQueueKind, e.config)
+	mgr, err := newTaskQueueManager(e, taskQueue, taskQueueKind, e.config, e.clusterMeta)
 	if err != nil {
 		return nil, err
 	}
-	e.logger.Info("", tag.LifeCycleStarting, tag.WorkflowTaskQueueName(taskQueue.name), tag.WorkflowTaskQueueType(taskQueue.taskType))
 	mgr.Start()
 	e.taskQueues[*taskQueue] = mgr
-	e.logger.Info("", tag.LifeCycleStarted, tag.WorkflowTaskQueueName(taskQueue.name), tag.WorkflowTaskQueueType(taskQueue.taskType))
+	countKey := taskQueueCounterKey{namespaceID: taskQueue.namespaceID, taskType: taskQueue.taskType, queueType: taskQueueKind}
+	e.taskQueueCount[countKey]++
+	taskQueueCount := e.taskQueueCount[countKey]
+	e.updateTaskQueueGauge(countKey, taskQueueCount)
+
 	return mgr, nil
 }
 
@@ -217,23 +232,12 @@ func (e *matchingEngineImpl) updateTaskQueue(taskQueue *taskQueueID, mgr taskQue
 	e.taskQueues[*taskQueue] = mgr
 }
 
-func (e *matchingEngineImpl) removeTaskQueueManager(delTQM taskQueueManager) {
-	queueID := delTQM.QueueID()
-	e.taskQueuesLock.Lock()
-	defer e.taskQueuesLock.Unlock()
-	foundTQM, ok := e.taskQueues[*queueID]
-	if !ok || foundTQM != delTQM {
-		return
-	}
-	delete(e.taskQueues, *queueID)
-}
-
 // AddWorkflowTask either delivers task directly to waiting poller or save it into task queue persistence.
 func (e *matchingEngineImpl) AddWorkflowTask(
 	hCtx *handlerContext,
 	addRequest *matchingservice.AddWorkflowTaskRequest,
 ) (bool, error) {
-	namespaceID := addRequest.GetNamespaceId()
+	namespaceID := namespace.ID(addRequest.GetNamespaceId())
 	taskQueueName := addRequest.TaskQueue.GetName()
 	taskQueueKind := addRequest.TaskQueue.GetKind()
 
@@ -265,7 +269,7 @@ func (e *matchingEngineImpl) AddWorkflowTask(
 		expirationTime = timestamp.TimePtr(now.Add(expirationDuration))
 	}
 	taskInfo := &persistencespb.TaskInfo{
-		NamespaceId: namespaceID,
+		NamespaceId: namespaceID.String(),
 		RunId:       addRequest.Execution.GetRunId(),
 		WorkflowId:  addRequest.Execution.GetWorkflowId(),
 		ScheduleId:  addRequest.GetScheduleId(),
@@ -286,7 +290,7 @@ func (e *matchingEngineImpl) AddActivityTask(
 	hCtx *handlerContext,
 	addRequest *matchingservice.AddActivityTaskRequest,
 ) (bool, error) {
-	namespaceID := addRequest.GetNamespaceId()
+	namespaceID := namespace.ID(addRequest.GetNamespaceId())
 	sourceNamespaceID := addRequest.GetSourceNamespaceId()
 	runID := addRequest.Execution.GetRunId()
 	taskQueueName := addRequest.TaskQueue.GetName()
@@ -338,7 +342,7 @@ func (e *matchingEngineImpl) PollWorkflowTaskQueue(
 	hCtx *handlerContext,
 	req *matchingservice.PollWorkflowTaskQueueRequest,
 ) (*matchingservice.PollWorkflowTaskQueueResponse, error) {
-	namespaceID := req.GetNamespaceId()
+	namespaceID := namespace.ID(req.GetNamespaceId())
 	pollerID := req.GetPollerId()
 	request := req.PollRequest
 	taskQueueName := request.TaskQueue.GetName()
@@ -431,7 +435,7 @@ func (e *matchingEngineImpl) PollActivityTaskQueue(
 	hCtx *handlerContext,
 	req *matchingservice.PollActivityTaskQueueRequest,
 ) (*matchingservice.PollActivityTaskQueueResponse, error) {
-	namespaceID := req.GetNamespaceId()
+	namespaceID := namespace.ID(req.GetNamespaceId())
 	pollerID := req.GetPollerId()
 	request := req.PollRequest
 	taskQueueName := request.TaskQueue.GetName()
@@ -501,7 +505,7 @@ func (e *matchingEngineImpl) QueryWorkflow(
 	hCtx *handlerContext,
 	queryRequest *matchingservice.QueryWorkflowRequest,
 ) (*matchingservice.QueryWorkflowResponse, error) {
-	namespaceID := queryRequest.GetNamespaceId()
+	namespaceID := namespace.ID(queryRequest.GetNamespaceId())
 	taskQueueName := queryRequest.TaskQueue.GetName()
 	taskQueueKind := queryRequest.TaskQueue.GetKind()
 	taskQueue, err := newTaskQueueID(namespaceID, taskQueueName, enumspb.TASK_QUEUE_TYPE_WORKFLOW)
@@ -572,7 +576,7 @@ func (e *matchingEngineImpl) CancelOutstandingPoll(
 	hCtx *handlerContext,
 	request *matchingservice.CancelOutstandingPollRequest,
 ) error {
-	namespaceID := request.GetNamespaceId()
+	namespaceID := namespace.ID(request.GetNamespaceId())
 	taskQueueType := request.GetTaskQueueType()
 	taskQueueName := request.TaskQueue.GetName()
 	pollerID := request.GetPollerId()
@@ -595,7 +599,7 @@ func (e *matchingEngineImpl) DescribeTaskQueue(
 	hCtx *handlerContext,
 	request *matchingservice.DescribeTaskQueueRequest,
 ) (*matchingservice.DescribeTaskQueueResponse, error) {
-	namespaceID := request.GetNamespaceId()
+	namespaceID := namespace.ID(request.GetNamespaceId())
 	taskQueueType := request.DescRequest.GetTaskQueueType()
 	taskQueueName := request.DescRequest.TaskQueue.GetName()
 	taskQueue, err := newTaskQueueID(namespaceID, taskQueueName, taskQueueType)
@@ -632,7 +636,7 @@ func (e *matchingEngineImpl) ListTaskQueuePartitions(
 
 func (e *matchingEngineImpl) listTaskQueuePartitions(request *matchingservice.ListTaskQueuePartitionsRequest, taskQueueType enumspb.TaskQueueType) ([]*taskqueuepb.TaskQueuePartitionMetadata, error) {
 	partitions, err := e.getAllPartitions(
-		request.GetNamespace(),
+		namespace.Name(request.GetNamespace()),
 		*request.TaskQueue,
 		taskQueueType,
 	)
@@ -666,12 +670,12 @@ func (e *matchingEngineImpl) getHostInfo(partitionKey string) (string, error) {
 }
 
 func (e *matchingEngineImpl) getAllPartitions(
-	namespace string,
+	namespace namespace.Name,
 	taskQueue taskqueuepb.TaskQueue,
 	taskQueueType enumspb.TaskQueueType,
 ) ([]string, error) {
 	var partitionKeys []string
-	namespaceID, err := e.namespaceCache.GetNamespaceID(namespace)
+	namespaceID, err := e.namespaceRegistry.GetNamespaceID(namespace)
 	if err != nil {
 		return partitionKeys, err
 	}
@@ -681,7 +685,7 @@ func (e *matchingEngineImpl) getAllPartitions(
 	partitionKeys = append(partitionKeys, rootPartition)
 
 	nWritePartitions := e.config.NumTaskqueueWritePartitions
-	n := nWritePartitions(namespace, rootPartition, taskQueueType)
+	n := nWritePartitions(namespace.String(), rootPartition, taskQueueType)
 	if n <= 0 {
 		return partitionKeys, nil
 	}
@@ -716,10 +720,30 @@ func (e *matchingEngineImpl) unloadTaskQueue(unloadTQM taskQueueManager) {
 		return
 	}
 	delete(e.taskQueues, *queueID)
+	countKey := taskQueueCounterKey{namespaceID: queueID.namespaceID, taskType: queueID.taskType, queueType: foundTQM.TaskQueueKind()}
+	e.taskQueueCount[countKey]--
+	taskQueueCount := e.taskQueueCount[countKey]
 	e.taskQueuesLock.Unlock()
+
+	e.updateTaskQueueGauge(countKey, taskQueueCount)
 	foundTQM.Stop()
 }
 
+func (e *matchingEngineImpl) updateTaskQueueGauge(countKey taskQueueCounterKey, taskQueueCount int) {
+	nsEntry, err := e.namespaceRegistry.GetNamespaceByID(countKey.namespaceID)
+	namespace := namespace.Name("unknown")
+	if err == nil {
+		namespace = nsEntry.Name()
+	}
+
+	e.metricsClient.Scope(
+		metrics.MatchingEngineScope,
+		metrics.NamespaceTag(namespace.String()),
+		metrics.TaskTypeTag(countKey.taskType.String()),
+		metrics.QueueTypeTag(countKey.queueType.String()),
+	).UpdateGauge(metrics.LoadedTaskQueueGauge, float64(taskQueueCount))
+}
+
 // Populate the workflow task response based on context and scheduled/started events.
 func (e *matchingEngineImpl) createPollWorkflowTaskQueueResponse(
 	task *internalTask,
