@@ -41,14 +41,15 @@ import (
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	"go.temporal.io/server/api/matchingservice/v1"
 	"go.temporal.io/server/common/clock"
+	"go.temporal.io/server/common/cluster"
 
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/backoff"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 )
 
@@ -61,16 +62,6 @@ const (
 	syncMatchTaskId = -137
 )
 
-// fatalSignalBehavior indicates to functions that may inovke the
-// taskQueueManagerImpl.signalFatalProblem callback whether they should or
-// should not do that.
-type fatalSignalBehavior int
-
-const (
-	emitFatalSignal fatalSignalBehavior = iota + 1
-	suppressFatalSignal
-)
-
 type (
 	taskQueueManagerOpt func(*taskQueueManagerImpl)
 
@@ -109,27 +100,27 @@ type (
 		DescribeTaskQueue(includeTaskQueueStatus bool) *matchingservice.DescribeTaskQueueResponse
 		String() string
 		QueueID() *taskQueueID
+		TaskQueueKind() enumspb.TaskQueueKind
 	}
 
 	// Single task queue in memory state
 	taskQueueManagerImpl struct {
-		status           int32
-		taskQueueID      *taskQueueID
-		taskQueueKind    enumspb.TaskQueueKind // sticky taskQueue has different process in persistence
-		config           *taskQueueConfig
-		db               *taskQueueDB
-		engine           *matchingEngineImpl
-		taskWriter       *taskWriter
-		taskReader       *taskReader // reads tasks from db and async matches it with poller
-		liveness         *liveness
-		taskGC           *taskGC
-		taskAckManager   ackManager   // tracks ackLevel for delivered messages
-		matcher          *TaskMatcher // for matching a task producer with a poller
-		namespaceCache   cache.NamespaceCache
-		logger           log.Logger
-		metricsClient    metrics.Client
-		namespaceValue   atomic.Value
-		metricScopeValue atomic.Value // namespace/taskqueue tagged metric scope
+		status            int32
+		taskQueueID       *taskQueueID
+		taskQueueKind     enumspb.TaskQueueKind // sticky taskQueue has different process in persistence
+		config            *taskQueueConfig
+		db                *taskQueueDB
+		taskWriter        *taskWriter
+		taskReader        *taskReader // reads tasks from db and async matches it with poller
+		liveness          *liveness
+		taskGC            *taskGC
+		taskAckManager    ackManager   // tracks ackLevel for delivered messages
+		matcher           *TaskMatcher // for matching a task producer with a poller
+		namespaceRegistry namespace.Registry
+		logger            log.Logger
+		metricsClient     metrics.Client
+		namespace         namespace.Name
+		metricScope       metrics.Scope // namespace/taskqueue tagged metric scope
 		// pollerHistory stores poller which poll from this taskqueue in last few minutes
 		pollerHistory *pollerHistory
 		// outstandingPollsMap is needed to keep track of all outstanding pollers for a
@@ -139,8 +130,8 @@ type (
 		// prevent tasks being dispatched to zombie pollers.
 		outstandingPollsLock sync.Mutex
 		outstandingPollsMap  map[string]context.CancelFunc
-		shutdownCh           chan struct{} // Delivers stop to the pump that populates taskBuffer
 		signalFatalProblem   func(taskQueueManager)
+		clusterMeta          cluster.Metadata
 	}
 )
 
@@ -159,25 +150,38 @@ func newTaskQueueManager(
 	taskQueue *taskQueueID,
 	taskQueueKind enumspb.TaskQueueKind,
 	config *Config,
+	clusterMeta cluster.Metadata,
 	opts ...taskQueueManagerOpt,
 ) (taskQueueManager, error) {
+	namespaceEntry, err := e.namespaceRegistry.GetNamespaceByID(taskQueue.namespaceID)
+	if err != nil {
+		return nil, err
+	}
+	nsName := namespaceEntry.Name()
 
-	taskQueueConfig, err := newTaskQueueConfig(taskQueue, config, e.namespaceCache)
+	taskQueueConfig, err := newTaskQueueConfig(taskQueue, config, nsName)
 	if err != nil {
 		return nil, err
 	}
 
 	db := newTaskQueueDB(e.taskManager, taskQueue.namespaceID, taskQueue.name, taskQueue.taskType, taskQueueKind, e.logger)
-
+	logger := log.With(e.logger,
+		tag.WorkflowTaskQueueName(taskQueue.name),
+		tag.WorkflowTaskQueueType(taskQueue.taskType),
+		tag.WorkflowNamespace(nsName.String()))
+	metricsScope := metrics.GetPerTaskQueueScope(
+		e.metricsClient.Scope(metrics.MatchingTaskQueueMgrScope),
+		nsName.String(),
+		taskQueue.name,
+		taskQueueKind,
+	)
 	tlMgr := &taskQueueManagerImpl{
 		status:              common.DaemonStatusInitialized,
-		namespaceCache:      e.namespaceCache,
+		namespaceRegistry:   e.namespaceRegistry,
 		metricsClient:       e.metricsClient,
-		engine:              e,
-		shutdownCh:          make(chan struct{}),
 		taskQueueID:         taskQueue,
 		taskQueueKind:       taskQueueKind,
-		logger:              log.With(e.logger, tag.WorkflowTaskQueueName(taskQueue.name), tag.WorkflowTaskQueueType(taskQueue.taskType)),
+		logger:              logger,
 		db:                  db,
 		taskAckManager:      newAckManager(e.logger),
 		taskGC:              newTaskGC(db, taskQueueConfig),
@@ -185,21 +189,16 @@ func newTaskQueueManager(
 		pollerHistory:       newPollerHistory(),
 		outstandingPollsMap: make(map[string]context.CancelFunc),
 		signalFatalProblem:  e.unloadTaskQueue,
+		clusterMeta:         clusterMeta,
+		namespace:           nsName,
+		metricScope:         metricsScope,
 	}
 
-	tlMgr.namespaceValue.Store("")
-	if tlMgr.metricScope() == nil { // namespace name lookup failed
-		// metric scope to use when namespace lookup fails
-		tlMgr.metricScopeValue.Store(newPerTaskQueueScope(
-			"",
-			tlMgr.taskQueueID.name,
-			tlMgr.taskQueueKind,
-			e.metricsClient,
-			metrics.MatchingTaskQueueMgrScope,
-		))
-	}
-
-	tlMgr.liveness = newLiveness(clock.NewRealTimeSource(), taskQueueConfig.IdleTaskqueueCheckInterval(), tlMgr.Stop)
+	tlMgr.liveness = newLiveness(
+		clock.NewRealTimeSource(),
+		taskQueueConfig.IdleTaskqueueCheckInterval(),
+		func() { tlMgr.signalFatalProblem(tlMgr) },
+	)
 	tlMgr.taskWriter = newTaskWriter(tlMgr)
 	tlMgr.taskReader = newTaskReader(tlMgr)
 
@@ -214,13 +213,21 @@ func newTaskQueueManager(
 	return tlMgr, nil
 }
 
-func (c *taskQueueManagerImpl) errShouldUnload(err error) bool {
-	return err != nil && (errIndicatesForeignLessee(err) || !c.config.ResilientSyncMatch())
-}
-
-func errIndicatesForeignLessee(err error) bool {
+// signalIfFatal calls signalFatalProblem on this taskQueueManagerImpl instance
+// if and only if the supplied error represents a fatal condition, e.g. the
+// existence of another taskQueueManager newer lease. Returns true if the signal
+// is emitted, false otherwise.
+func (c *taskQueueManagerImpl) signalIfFatal(err error) bool {
+	if err == nil {
+		return false
+	}
 	var condfail *persistence.ConditionFailedError
-	return errors.As(err, &condfail)
+	if errors.As(err, &condfail) {
+		c.metricScope.IncCounter(metrics.ConditionFailedErrorPerTaskQueueCounter)
+		c.signalFatalProblem(c)
+		return true
+	}
+	return false
 }
 
 // Start reading pump for the given task queue.
@@ -236,6 +243,8 @@ func (c *taskQueueManagerImpl) Start() {
 	c.liveness.Start()
 	c.taskWriter.Start()
 	c.taskReader.Start()
+	c.logger.Info("", tag.LifeCycleStarted)
+	c.metricScope.IncCounter(metrics.TaskQueueStartedCounter)
 }
 
 // Stop pump that fills up taskBuffer from persistence.
@@ -247,17 +256,13 @@ func (c *taskQueueManagerImpl) Stop() {
 	) {
 		return
 	}
-
-	close(c.shutdownCh)
-
 	_ = c.db.UpdateState(c.taskAckManager.getAckLevel())
 	c.taskGC.RunNow(c.taskAckManager.getAckLevel())
-
 	c.liveness.Stop()
 	c.taskWriter.Stop()
 	c.taskReader.Stop()
-	c.engine.removeTaskQueueManager(c)
 	c.logger.Info("", tag.LifeCycleStopped)
+	c.metricScope.IncCounter(metrics.TaskQueueStoppedCounter)
 }
 
 // AddTask adds a task to the task queue. This method will first attempt a synchronous
@@ -276,12 +281,12 @@ func (c *taskQueueManagerImpl) AddTask(
 	_, err := c.executeWithRetry(func() (interface{}, error) {
 		td := params.taskInfo
 
-		namespaceEntry, err := c.namespaceCache.GetNamespaceByID(td.GetNamespaceId())
+		namespaceEntry, err := c.namespaceRegistry.GetNamespaceByID(namespace.ID(td.GetNamespaceId()))
 		if err != nil {
 			return nil, err
 		}
 
-		if namespaceEntry.GetNamespaceNotActiveErr() != nil {
+		if !namespaceEntry.ActiveInCluster(c.clusterMeta.GetCurrentClusterName()) {
 			r, err := c.taskWriter.appendTask(params.execution, td)
 			syncMatch = false
 			return r, err
@@ -298,9 +303,11 @@ func (c *taskQueueManagerImpl) AddTask(
 			return &persistence.CreateTasksResponse{}, errRemoteSyncMatchFailed
 		}
 
-		return c.taskWriter.appendTask(params.execution, params.taskInfo)
+		resp, err := c.taskWriter.appendTask(params.execution, params.taskInfo)
+		c.signalIfFatal(err)
+		return resp, err
 	})
-	if err == nil {
+	if !syncMatch && err == nil {
 		c.taskReader.Signal()
 	}
 	return syncMatch, err
@@ -342,7 +349,7 @@ func (c *taskQueueManagerImpl) GetTask(
 		c.pollerHistory.updatePollerInfo(pollerIdentity(identity), maxDispatchPerSecond)
 	}
 
-	namespaceEntry, err := c.namespaceCache.GetNamespaceByID(c.taskQueueID.namespaceID)
+	namespaceEntry, err := c.namespaceRegistry.GetNamespaceByID(c.taskQueueID.namespaceID)
 	if err != nil {
 		return nil, err
 	}
@@ -354,7 +361,7 @@ func (c *taskQueueManagerImpl) GetTask(
 	// value. Last poller wins if different pollers provide different values
 	c.matcher.UpdateRatelimit(maxDispatchPerSecond)
 
-	if namespaceEntry.GetNamespaceNotActiveErr() != nil {
+	if !namespaceEntry.ActiveInCluster(c.clusterMeta.GetCurrentClusterName()) {
 		return c.matcher.PollForQuery(childCtx)
 	}
 
@@ -363,7 +370,7 @@ func (c *taskQueueManagerImpl) GetTask(
 		return nil, err
 	}
 
-	task.namespace = c.namespace()
+	task.namespace = c.namespace
 	task.backlogCountHint = c.taskAckManager.getBacklogCountHint()
 	return task, nil
 }
@@ -489,7 +496,7 @@ func rangeIDToTaskIDBlock(rangeID int64, rangeSize int64) taskIDBlock {
 	}
 }
 
-// Retry operation on transient error. On rangeID update by another process calls c.Stop().
+// Retry operation on transient error.
 func (c *taskQueueManagerImpl) executeWithRetry(
 	operation func() (interface{}, error)) (result interface{}, err error) {
 
@@ -507,12 +514,6 @@ func (c *taskQueueManagerImpl) executeWithRetry(
 		}
 		return common.IsPersistenceTransientError(err)
 	})
-
-	if _, ok := err.(*persistence.ConditionFailedError); ok {
-		c.metricScope().IncCounter(metrics.ConditionFailedErrorPerTaskQueueCounter)
-		c.logger.Debug("Stopping task queue due to persistence condition failure", tag.Error(err))
-		c.Stop()
-	}
 	return
 }
 
@@ -562,46 +563,10 @@ func (c *taskQueueManagerImpl) isFowardingAllowed(taskQueue *taskQueueID, kind e
 	return !taskQueue.IsRoot() && kind != enumspb.TASK_QUEUE_KIND_STICKY
 }
 
-func (c *taskQueueManagerImpl) metricScope() metrics.Scope {
-	c.tryInitNamespaceAndScope()
-	return c.metricScopeValue.Load().(metrics.Scope)
-}
-
-func (c *taskQueueManagerImpl) namespace() string {
-	name := c.namespaceValue.Load().(string)
-	if len(name) > 0 {
-		return name
-	}
-	c.tryInitNamespaceAndScope()
-	return c.namespaceValue.Load().(string)
-}
-
-// reload from namespaceCache in case it got empty result during construction
-func (c *taskQueueManagerImpl) tryInitNamespaceAndScope() {
-	namespace := c.namespaceValue.Load().(string)
-	if namespace != "" {
-		return
-	}
-
-	entry, err := c.namespaceCache.GetNamespaceByID(c.taskQueueID.namespaceID)
-	if err != nil {
-		return
-	}
-
-	namespace = entry.GetInfo().Name
-
-	scope := newPerTaskQueueScope(
-		namespace,
-		c.taskQueueID.name,
-		c.taskQueueKind,
-		c.metricsClient,
-		metrics.MatchingTaskQueueMgrScope,
-	)
-
-	c.metricScopeValue.Store(scope)
-	c.namespaceValue.Store(namespace)
-}
-
 func (c *taskQueueManagerImpl) QueueID() *taskQueueID {
 	return c.taskQueueID
 }
+
+func (c *taskQueueManagerImpl) TaskQueueKind() enumspb.TaskQueueKind {
+	return c.taskQueueKind
+}
