@@ -35,11 +35,11 @@ import (
 
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/archiver/provider"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 )
 
@@ -51,17 +51,17 @@ type (
 	}
 
 	clientWorker struct {
-		worker         worker.Worker
-		namespaceCache cache.NamespaceCache
+		worker            worker.Worker
+		namespaceRegistry namespace.Registry
 	}
 
 	// BootstrapContainer contains everything need for bootstrapping
 	BootstrapContainer struct {
-		SdkClient        sdkclient.Client
+		SdkSystemClient  sdkclient.Client
 		MetricsClient    metrics.Client
 		Logger           log.Logger
 		HistoryV2Manager persistence.ExecutionManager
-		NamespaceCache   cache.NamespaceCache
+		NamespaceCache   namespace.Registry
 		Config           *Config
 		ArchiverProvider provider.ArchiverProvider
 	}
@@ -113,8 +113,8 @@ func NewClientWorker(container *BootstrapContainer) ClientWorker {
 		BackgroundActivityContext:              actCtx,
 	}
 	clientWorker := &clientWorker{
-		worker:         worker.New(container.SdkClient, workflowTaskQueue, wo),
-		namespaceCache: container.NamespaceCache,
+		worker:            worker.New(container.SdkSystemClient, workflowTaskQueue, wo),
+		namespaceRegistry: container.NamespaceCache,
 	}
 
 	clientWorker.worker.RegisterWorkflowWithOptions(archivalWorkflow, workflow.RegisterOptions{Name: archivalWorkflowFnName})
@@ -137,5 +137,5 @@ func (w *clientWorker) Start() error {
 // Stop the ClientWorker
 func (w *clientWorker) Stop() {
 	w.worker.Stop()
-	w.namespaceCache.Stop()
+	w.namespaceRegistry.Stop()
 }
