@@ -37,11 +37,12 @@ import (
 	"go.temporal.io/server/common/resolver"
 	"go.temporal.io/server/common/rpc/encryption"
 	"go.temporal.io/server/common/searchattribute"
+	"google.golang.org/grpc"
 )
 
 type (
 	serverOptions struct {
-		serviceNames []string
+		serviceNames map[string]struct{}
 
 		config    *config.Config
 		configDir string
@@ -64,21 +65,24 @@ type (
 		customDataStoreFactory     persistenceClient.AbstractDataStoreFactory
 		clientFactoryProvider      client.FactoryProvider
 		searchAttributesMapper     searchattribute.Mapper
+		customInterceptors         []grpc.UnaryServerInterceptor
 	}
 )
 
 func newServerOptions(opts []ServerOption) *serverOptions {
 	so := &serverOptions{
 		// Set defaults here.
+		persistenceServiceResolver: resolver.NewNoopResolver(),
 	}
 	for _, opt := range opts {
 		opt.apply(so)
 	}
+
 	return so
 }
 
 func (so *serverOptions) loadAndValidate() error {
-	for _, serviceName := range so.serviceNames {
+	for serviceName, _ := range so.serviceNames {
 		if !isValidService(serviceName) {
 			return fmt.Errorf("invalid service %q in service list %v", serviceName, so.serviceNames)
 		}
@@ -114,7 +118,7 @@ func (so *serverOptions) validateConfig() error {
 		return err
 	}
 
-	for _, name := range so.serviceNames {
+	for name, _ := range so.serviceNames {
 		if _, ok := so.config.Services[name]; !ok {
 			return fmt.Errorf("%q service is missing in config", name)
 		}
