@@ -27,8 +27,9 @@ package cli
 import (
 	"context"
 	"fmt"
-	"io/ioutil"
 	"math"
+	"net/url"
+	"os"
 	"strconv"
 	"time"
 
@@ -38,9 +39,6 @@ import (
 	enumspb "go.temporal.io/api/enums/v1"
 	historypb "go.temporal.io/api/history/v1"
 	"go.temporal.io/server/api/adminservice/v1"
-	esclient "go.temporal.io/server/common/persistence/visibility/elasticsearch/client"
-	"go.temporal.io/server/common/searchattribute"
-
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
@@ -49,56 +47,59 @@ import (
 	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
-	"go.temporal.io/server/common/membership"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/persistence/cassandra"
 	"go.temporal.io/server/common/persistence/nosql/nosqlplugin/cassandra/gocql"
 	"go.temporal.io/server/common/persistence/serialization"
 	"go.temporal.io/server/common/persistence/versionhistory"
+	esclient "go.temporal.io/server/common/persistence/visibility/store/elasticsearch/client"
 	"go.temporal.io/server/common/primitives"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/common/resolver"
+	"go.temporal.io/server/common/searchattribute"
 )
 
 const maxEventID = 9999
 
 // AdminShowWorkflow shows history
 func AdminShowWorkflow(c *cli.Context) {
-	tid := c.String(FlagTreeID)
-	bid := c.String(FlagBranchID)
-	sid := int32(c.Int(FlagShardID))
+	namespace := c.String(FlagNamespace)
+	wid := getRequiredOption(c, FlagWorkflowID)
+	rid := getRequiredOption(c, FlagRunID)
+	startEventId := c.Int64(FlagMinEventID)
+	endEventId := c.Int64(FlagMaxEventID)
+	startEventVerion := int64(c.Int(FlagMinEventVersion))
+	endEventVersion := int64(c.Int(FlagMaxEventVersion))
 	outputFileName := c.String(FlagOutputFilename)
 
-	session := connectToCassandra(c)
+	client := cFactory.AdminClient(c)
+
 	serializer := serialization.NewSerializer()
 	var history []*commonpb.DataBlob
-	if len(tid) != 0 {
-		histV2 := cassandra.NewExecutionStore(session, log.NewNoopLogger())
-		resp, err := histV2.ReadHistoryBranch(&persistence.InternalReadHistoryBranchRequest{
-			TreeID:    tid,
-			BranchID:  bid,
-			MinNodeID: 1,
-			MaxNodeID: maxEventID,
-			PageSize:  maxEventID,
-			ShardID:   sid,
-		})
-		if err != nil {
-			ErrorAndExit("ReadHistoryBranch err", err)
-		}
 
-		for _, node := range resp.Nodes {
-			history = append(history, node.Events)
-		}
-	} else {
-		ErrorAndExit("need to specify TreeId/BranchId/ShardId", nil)
+	ctx, cancel := newContext(c)
+	defer cancel()
+
+	resp, err := client.GetWorkflowExecutionRawHistoryV2(ctx, &adminservice.GetWorkflowExecutionRawHistoryV2Request{
+		Namespace: namespace,
+		Execution: &commonpb.WorkflowExecution{
+			WorkflowId: wid,
+			RunId:      rid,
+		},
+		StartEventId:      startEventId,
+		EndEventId:        endEventId,
+		StartEventVersion: startEventVerion,
+		EndEventVersion:   endEventVersion,
+		MaximumPageSize:   100,
+		NextPageToken:     nil,
+	})
+	if err != nil {
+		ErrorAndExit("ReadHistoryBranch err", err)
 	}
 
-	if len(history) == 0 {
-		ErrorAndExit("no events", nil)
-	}
 	allEvents := &historypb.History{}
 	totalSize := 0
-	for idx, b := range history {
+	for idx, b := range resp.HistoryBatches {
 		totalSize += len(b.Data)
 		fmt.Printf("======== batch %v, blob len: %v ======\n", idx+1, len(b.Data))
 		historyBatchThrift, err := serializer.DeserializeEvents(b)
@@ -122,7 +123,7 @@ func AdminShowWorkflow(c *cli.Context) {
 		if err != nil {
 			ErrorAndExit("Failed to serialize history data.", err)
 		}
-		if err := ioutil.WriteFile(outputFileName, data, 0666); err != nil {
+		if err := os.WriteFile(outputFileName, data, 0666); err != nil {
 			ErrorAndExit("Failed to export history data file.", err)
 		}
 	}
@@ -184,34 +185,6 @@ func describeMutableState(c *cli.Context) *adminservice.DescribeMutableStateResp
 	return resp
 }
 
-// AdminListNamespaces outputs a list of all namespaces
-func AdminListNamespaces(c *cli.Context) {
-	pFactory := CreatePersistenceFactory(c)
-	metadataManager, err := pFactory.NewMetadataManager()
-	if err != nil {
-		ErrorAndExit("Failed to initialize metadata manager", err)
-	}
-
-	req := &persistence.ListNamespacesRequest{
-		PageSize: c.Int(FlagPageSize),
-	}
-	paginationFunc := func(paginationToken []byte) ([]interface{}, []byte, error) {
-		req.NextPageToken = paginationToken
-		response, err := metadataManager.ListNamespaces(req)
-		if err != nil {
-			return nil, nil, err
-		}
-		token := response.NextPageToken
-
-		var items []interface{}
-		for _, task := range response.Namespaces {
-			items = append(items, task)
-		}
-		return items, token, nil
-	}
-	paginate(c, paginationFunc)
-}
-
 // AdminDeleteWorkflow delete a workflow execution from Cassandra and visibility document from Elasticsearch.
 func AdminDeleteWorkflow(c *cli.Context) {
 	resp := describeMutableState(c)
@@ -292,11 +265,11 @@ func AdminDeleteWorkflow(c *cli.Context) {
 }
 
 func adminDeleteVisibilityDocument(c *cli.Context, namespaceID string) {
-	if !c.IsSet(FlagIndex) {
+	if !c.IsSet(FlagElasticsearchIndex) {
 		prompt("Elasticsearch index name is not specified. Continue without visibility document deletion?", c.GlobalBool(FlagAutoConfirm))
 	}
 
-	indexName := getRequiredOption(c, FlagIndex)
+	indexName := getRequiredOption(c, FlagElasticsearchIndex)
 	esClient := newESClient(c)
 
 	query := elastic.NewBoolQuery().
@@ -307,7 +280,7 @@ func adminDeleteVisibilityDocument(c *cli.Context, namespaceID string) {
 		query = query.Filter(elastic.NewTermQuery(searchattribute.RunID, c.String(FlagRunID)))
 	}
 	searchParams := &esclient.SearchParameters{
-		Index:    c.String(FlagIndex),
+		Index:    c.String(FlagElasticsearchIndex),
 		Query:    query,
 		PageSize: 10000,
 	}
@@ -369,48 +342,29 @@ func connectToCassandra(c *cli.Context) gocql.Session {
 	return session
 }
 
-// AdminGetNamespaceIDOrName map namespace
-func AdminGetNamespaceIDOrName(c *cli.Context) {
-	namespaceID := c.String(FlagNamespaceID)
-	namespace := c.String(FlagNamespace)
-	if len(namespaceID) == 0 && len(namespace) == 0 {
-		ErrorAndExit("Need either namespace or namespaceId", nil)
+func newESClient(c *cli.Context) esclient.CLIClient {
+	esUrl := getRequiredOption(c, FlagElasticsearchURL)
+	parsedESUrl, err := url.Parse(esUrl)
+	if err != nil {
+		ErrorAndExit("Unable to parse URL.", err)
 	}
 
-	session := connectToCassandra(c)
-
-	if len(namespaceID) > 0 {
-		tmpl := "select namespace from namespaces where id = ? "
-		query := session.Query(tmpl, namespaceID)
-		res, err := readOneRow(query)
-		if err != nil {
-			ErrorAndExit("readOneRow", err)
-		}
-		namespaceName := res["name"].(string)
-		fmt.Printf("namespace for namespaceId %v is %v \n", namespaceID, namespaceName)
-	} else {
-		tmpl := "select namespace from namespaces_by_name where name = ?"
-		tmplV2 := "select namespace from namespaces where namespaces_partition=0 and name = ?"
-
-		query := session.Query(tmpl, namespace)
-		res, err := readOneRow(query)
-		if err != nil {
-			fmt.Printf("v1 return error: %v , trying v2...\n", err)
-
-			query := session.Query(tmplV2, namespace)
-			res, err := readOneRow(query)
-			if err != nil {
-				ErrorAndExit("readOneRow for v2", err)
-			}
-			namespace := res["namespace"].(map[string]interface{})
-			namespaceID := gocql.UUIDToString(namespace["id"])
-			fmt.Printf("namespaceId for namespace %v is %v \n", namespace, namespaceID)
-		} else {
-			namespace := res["namespace"].(map[string]interface{})
-			namespaceID := gocql.UUIDToString(namespace["id"])
-			fmt.Printf("namespaceId for namespace %v is %v \n", namespace, namespaceID)
-		}
+	esConfig := &esclient.Config{
+		URL:      *parsedESUrl,
+		Username: c.String(FlagElasticsearchUsername),
+		Password: c.String(FlagElasticsearchPassword),
 	}
+
+	if c.IsSet(FlagVersion) {
+		esConfig.Version = c.String(FlagVersion)
+	}
+
+	client, err := esclient.NewCLIClient(esConfig, log.NewCLILogger())
+	if err != nil {
+		ErrorAndExit("Unable to create Elasticsearch client", err)
+	}
+
+	return client
 }
 
 // AdminGetShardID get shardID
@@ -480,8 +434,8 @@ func AdminDescribeTask(c *cli.Context) {
 	}
 }
 
-// AdminListTasks outputs a list of a tasks for given Shard and Task Type
-func AdminListTasks(c *cli.Context) {
+// AdminListShardTasks outputs a list of a tasks for given Shard and Task Category
+func AdminListShardTasks(c *cli.Context) {
 	sid := int32(getRequiredIntOption(c, FlagShardID))
 	categoryInt, err := stringToEnum(c.String(FlagTaskType), enumsspb.TaskCategory_value)
 	if err != nil {
@@ -492,18 +446,16 @@ func AdminListTasks(c *cli.Context) {
 		ErrorAndExit(fmt.Sprintf("Task type %s is currently not supported", category), nil)
 	}
 
-	pFactory := CreatePersistenceFactory(c)
-	executionManager, err := pFactory.NewExecutionManager()
-	if err != nil {
-		ErrorAndExit("Failed to initialize execution manager", err)
-	}
+	client := cFactory.AdminClient(c)
 
+	ctx, cancel := newContext(c)
+	defer cancel()
 	if category == enumsspb.TASK_CATEGORY_TRANSFER {
-		req := &persistence.GetTransferTasksRequest{ShardID: sid}
+		req := &adminservice.ListTransferTasksRequest{ShardId: sid}
 
 		paginationFunc := func(paginationToken []byte) ([]interface{}, []byte, error) {
 			req.NextPageToken = paginationToken
-			response, err := executionManager.GetTransferTasks(req)
+			response, err := client.ListTransferTasks(ctx, req)
 			if err != nil {
 				return nil, nil, err
 			}
@@ -517,11 +469,11 @@ func AdminListTasks(c *cli.Context) {
 		}
 		paginate(c, paginationFunc)
 	} else if category == enumsspb.TASK_CATEGORY_VISIBILITY {
-		req := &persistence.GetVisibilityTasksRequest{ShardID: sid}
+		req := &adminservice.ListVisibilityTasksRequest{ShardId: sid}
 
 		paginationFunc := func(paginationToken []byte) ([]interface{}, []byte, error) {
 			req.NextPageToken = paginationToken
-			response, err := executionManager.GetVisibilityTasks(req)
+			response, err := client.ListVisibilityTasks(ctx, req)
 			if err != nil {
 				return nil, nil, err
 			}
@@ -538,31 +490,31 @@ func AdminListTasks(c *cli.Context) {
 		minVis := parseTime(c.String(FlagMinVisibilityTimestamp), time.Time{}, time.Now().UTC())
 		maxVis := parseTime(c.String(FlagMaxVisibilityTimestamp), time.Time{}, time.Now().UTC())
 
-		req := &persistence.GetTimerIndexTasksRequest{
-			ShardID:      sid,
-			MinTimestamp: minVis,
-			MaxTimestamp: maxVis,
+		req := &adminservice.ListTimerTasksRequest{
+			ShardId: sid,
+			MinTime: &minVis,
+			MaxTime: &maxVis,
 		}
 		paginationFunc := func(paginationToken []byte) ([]interface{}, []byte, error) {
 			req.NextPageToken = paginationToken
-			response, err := executionManager.GetTimerIndexTasks(req)
+			response, err := client.ListTimerTasks(ctx, req)
 			if err != nil {
 				return nil, nil, err
 			}
 			token := response.NextPageToken
 
 			var items []interface{}
-			for _, task := range response.Timers {
+			for _, task := range response.Tasks {
 				items = append(items, task)
 			}
 			return items, token, nil
 		}
 		paginate(c, paginationFunc)
 	} else if category == enumsspb.TASK_CATEGORY_REPLICATION {
-		req := &persistence.GetReplicationTasksRequest{}
+		req := &adminservice.ListReplicationTasksRequest{ShardId: sid}
 		paginationFunc := func(paginationToken []byte) ([]interface{}, []byte, error) {
 			req.NextPageToken = paginationToken
-			response, err := executionManager.GetReplicationTasks(req)
+			response, err := client.ListReplicationTasks(ctx, req)
 			if err != nil {
 				return nil, nil, err
 			}
@@ -617,17 +569,16 @@ func AdminRemoveTask(c *cli.Context) {
 // AdminDescribeShard describes shard by shard id
 func AdminDescribeShard(c *cli.Context) {
 	sid := getRequiredIntOption(c, FlagShardID)
-	pFactory := CreatePersistenceFactory(c)
-	shardManager, err := pFactory.NewShardManager()
+	adminClient := cFactory.AdminClient(c)
+	ctx, cancel := newContext(c)
+	defer cancel()
+	response, err := adminClient.GetShard(ctx, &adminservice.GetShardRequest{ShardId: int32(sid)})
 
 	if err != nil {
 		ErrorAndExit("Failed to initialize shard manager", err)
 	}
 
-	getShardReq := &persistence.GetShardRequest{ShardID: int32(sid)}
-	shard, err := shardManager.GetShard(getShardReq)
-
-	prettyPrintJSONObject(shard)
+	prettyPrintJSONObject(response.ShardInfo)
 }
 
 // AdminShardManagement describes history host
@@ -674,32 +625,29 @@ func AdminListGossipMembers(c *cli.Context) {
 	prettyPrintJSONObject(members)
 }
 
-// AdminListClusterMembership outputs a list of cluster membership items
-func AdminListClusterMembership(c *cli.Context) {
-	roleFlag := c.String(FlagClusterMembershipRole)
-	role, err := membership.ServiceNameToServiceTypeEnum(roleFlag)
-	if err != nil {
-		ErrorAndExit("Failed to map membership role", err)
-	}
+// AdminListClusterMembers outputs a list of cluster members
+func AdminListClusterMembers(c *cli.Context) {
+	role, _ := stringToEnum(c.String(FlagClusterMembershipRole), enumsspb.ClusterMemberRole_value)
 	// TODO: refactor this: parseTime shouldn't be used for duration.
 	heartbeatFlag := parseTime(c.String(FlagEarliestTime), time.Time{}, time.Now().UTC()).UnixNano()
 	heartbeat := time.Duration(heartbeatFlag)
 
-	pFactory := CreatePersistenceFactory(c)
-	manager, err := pFactory.NewClusterMetadataManager()
-	if err != nil {
-		ErrorAndExit("Failed to initialize cluster metadata manager", err)
+	adminClient := cFactory.AdminClient(c)
+	ctx, cancel := newContext(c)
+	defer cancel()
+
+	req := &adminservice.ListClusterMembersRequest{
+		Role:                enumsspb.ClusterMemberRole(role),
+		LastHeartbeatWithin: &heartbeat,
 	}
 
-	req := &persistence.GetClusterMembersRequest{
-		RoleEquals:          role,
-		LastHeartbeatWithin: heartbeat,
-	}
-	members, err := manager.GetClusterMembers(req)
+	resp, err := adminClient.ListClusterMembers(ctx, req)
 	if err != nil {
-		ErrorAndExit("Failed to get cluster members", err)
+		ErrorAndExit("unable to list cluster members", err)
 	}
 
+	members := resp.ActiveMembers
+
 	prettyPrintJSONObject(members)
 }
 
