@@ -33,9 +33,9 @@ import (
 	"time"
 
 	"github.com/urfave/cli"
-	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	"go.temporal.io/api/serviceerror"
+
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
@@ -718,10 +718,8 @@ func concreteExecutionStillExists(
 	getConcreteExecution := &persistence.GetWorkflowExecutionRequest{
 		ShardID:     shardID,
 		NamespaceID: executionInfo.NamespaceId,
-		Execution: commonpb.WorkflowExecution{
-			WorkflowId: executionInfo.WorkflowId,
-			RunId:      executionState.GetRunId(),
-		},
+		WorkflowID:  executionInfo.WorkflowId,
+		RunID:       executionState.GetRunId(),
 	}
 	preconditionForDBCall(totalDBRequests, limiter)
 	_, err := executionStore.GetWorkflowExecution(getConcreteExecution)
@@ -755,10 +753,8 @@ func concreteExecutionStillOpen(
 	getConcreteExecution := &persistence.GetWorkflowExecutionRequest{
 		ShardID:     shardID,
 		NamespaceID: executionInfo.NamespaceId,
-		Execution: commonpb.WorkflowExecution{
-			WorkflowId: executionInfo.WorkflowId,
-			RunId:      executionState.GetRunId(),
-		},
+		WorkflowID:  executionInfo.WorkflowId,
+		RunID:       executionState.GetRunId(),
 	}
 	preconditionForDBCall(totalDBRequests, limiter)
 	ce, err := execMan.GetWorkflowExecution(getConcreteExecution)
@@ -898,7 +894,7 @@ func getRateLimiter(startRPS int, targetRPS int) quotas.RateLimiter {
 	if startRPS >= targetRPS {
 		ErrorAndExit("startRPS is greater than target RPS", nil)
 	}
-	return quotas.NewDefaultOutgoingDynamicRateLimiter(
+	return quotas.NewDefaultOutgoingRateLimiter(
 		func() float64 { return float64(targetRPS) },
 	)
 }
