@@ -24,7 +24,11 @@
 
 package cli
 
-import "github.com/urfave/cli"
+import (
+	"fmt"
+
+	"github.com/urfave/cli"
+)
 
 // Flags used to specify cli command line arguments
 const (
@@ -33,11 +37,13 @@ const (
 	FlagKeyspace                              = "keyspace"
 	FlagAddress                               = "address"
 	FlagAddressWithAlias                      = FlagAddress + ", ad"
-	FlagHistoryAddress                        = "history_address"
 	FlagDBEngine                              = "db_engine"
 	FlagDBAddress                             = "db_address"
 	FlagDBPort                                = "db_port"
+	FlagHistoryAddress                        = "history_address"
 	FlagHistoryAddressWithAlias               = FlagHistoryAddress + ", had"
+	FlagFrontendAddress                       = "frontend_address"
+	FlagFrontendAddressWithAlias              = FlagFrontendAddress + ", fad"
 	FlagNamespaceID                           = "namespace_id"
 	FlagNamespace                             = "namespace"
 	FlagNamespaceWithAlias                    = FlagNamespace + ", ns"
@@ -53,7 +59,8 @@ const (
 	FlagTargetCluster                         = "target_cluster"
 	FlagMinEventID                            = "min_event_id"
 	FlagMaxEventID                            = "max_event_id"
-	FlagStartEventVersion                     = "start_event_version"
+	FlagMinEventVersion                       = "min_event_version"
+	FlagMaxEventVersion                       = "max_event_version"
 	FlagTaskQueue                             = "taskqueue"
 	FlagTaskQueueWithAlias                    = FlagTaskQueue + ", tq"
 	FlagTaskQueueType                         = "taskqueuetype"
@@ -65,8 +72,10 @@ const (
 	FlagWorkflowTypeWithAlias                 = FlagWorkflowType + ", wt"
 	FlagWorkflowStatus                        = "status"
 	FlagWorkflowStatusWithAlias               = FlagWorkflowStatus + ", s"
-	FlagExecutionTimeout                      = "execution_timeout"
-	FlagExecutionTimeoutWithAlias             = FlagExecutionTimeout + ", et"
+	FlagWorkflowExecutionTimeout              = "execution_timeout"
+	FlagWorkflowExecutionTimeoutWithAlias     = FlagWorkflowExecutionTimeout + ", et"
+	FlagWorkflowRunTimeout                    = "run-timeout"
+	FlagWorkflowRunTimeoutWithAlias           = FlagWorkflowRunTimeout + ", rt"
 	FlagWorkflowTaskTimeout                   = "workflow_task_timeout"
 	FlagWorkflowTaskTimeoutWithAlias          = FlagWorkflowTaskTimeout + ", wtt"
 	FlagContextTimeout                        = "context_timeout"
@@ -82,14 +91,7 @@ const (
 	FlagSkipBaseIsNotCurrent                  = "skip_base_is_not_current"
 	FlagDryRun                                = "dry_run"
 	FlagNonDeterministicOnly                  = "only_non_deterministic"
-	FlagInputTopic                            = "input_topic"
-	FlagInputTopicWithAlias                   = FlagInputTopic + ", it"
-	FlagHostFile                              = "host_file"
 	FlagCluster                               = "cluster"
-	FlagInputCluster                          = "input_cluster"
-	FlagStartOffset                           = "start_offset"
-	FlagTopic                                 = "topic"
-	FlagGroup                                 = "group"
 	FlagResult                                = "result"
 	FlagIdentity                              = "identity"
 	FlagDetail                                = "detail"
@@ -156,6 +158,8 @@ const (
 	FlagClusterMembershipRole                 = "role"
 	FlagIsGlobalNamespace                     = "global_namespace"
 	FlagIsGlobalNamespaceWithAlias            = FlagIsGlobalNamespace + ", gd"
+	FlagPromoteNamespace                      = "promote_namespace"
+	FlagPromoteNamespaceWithAlias             = FlagPromoteNamespace + ", pn"
 	FlagNamespaceData                         = "namespace_data"
 	FlagNamespaceDataWithAlias                = FlagNamespaceData + ", dmd"
 	FlagEventID                               = "event_id"
@@ -166,20 +170,15 @@ const (
 	FlagMaxFieldLengthWithAlias               = FlagMaxFieldLength + ", maxl"
 	FlagSkipErrorMode                         = "skip_errors"
 	FlagSkipErrorModeWithAlias                = FlagSkipErrorMode + ", serr"
-	FlagHeadersMode                           = "headers"
-	FlagHeadersModeWithAlias                  = FlagHeadersMode + ", he"
-	FlagMessageType                           = "message_type"
-	FlagMessageTypeWithAlias                  = FlagMessageType + ", mt"
-	FlagURL                                   = "url"
-	FlagIndex                                 = "index"
-	FlagBatchSize                             = "batch_size"
-	FlagBatchSizeWithAlias                    = FlagBatchSize + ", bs"
+	FlagElasticsearchURL                      = "url"
+	FlagElasticsearchUsername                 = "es-username"
+	FlagElasticsearchPassword                 = "es-password"
+	FlagElasticsearchIndex                    = "index"
 	FlagMemoKey                               = "memo_key"
 	FlagMemo                                  = "memo"
 	FlagMemoFile                              = "memo_file"
-	FlagSearchAttributesKey                   = "search_attr_key"
-	FlagSearchAttributesVal                   = "search_attr_value"
-	FlagSearchAttributesType                  = "search_attr_type"
+	FlagSearchAttributeKey                    = "search_attr_key"
+	FlagSearchAttributeValue                  = "search_attr_value"
 	FlagAddBadBinary                          = "add_bad_binary"
 	FlagRemoveBadBinary                       = "remove_bad_binary"
 	FlagResetType                             = "reset_type"
@@ -194,8 +193,8 @@ const (
 	FlagSignalNameWithAlias                   = FlagSignalName + ", sig"
 	FlagTaskID                                = "task_id"
 	FlagTaskType                              = "task_type"
-	FlagMinReadLevel                          = "min_read_level"
-	FlagMaxReadLevel                          = "max_read_level"
+	FlagMinTaskID                             = "min_task_id"
+	FlagMaxTaskID                             = "max_task_id"
 	FlagTaskVisibilityTimestamp               = "task_timestamp"
 	FlagMinVisibilityTimestamp                = "min_visibility_ts"
 	FlagMaxVisibilityTimestamp                = "max_visibility_ts"
@@ -216,6 +215,8 @@ const (
 	FlagTLSCaPath                             = "tls_ca_path"
 	FlagTLSDisableHostVerification            = "tls_disable_host_verification"
 	FlagTLSServerName                         = "tls_server_name"
+	FlagTLSRootCaData                         = "tls_ca_data"
+	FlagTLSForceEnable                        = "tls_force_enable"
 	FlagDLQType                               = "dlq_type"
 	FlagDLQTypeWithAlias                      = FlagDLQType + ", dt"
 	FlagMaxMessageCount                       = "max_message_count"
@@ -237,6 +238,9 @@ const (
 	FlagType                                  = "type"
 	FlagTypeWithAlias                         = FlagType + ", t"
 	FlagVersion                               = "version"
+	FlagPort                                  = "port"
+	FlagConnectionEnable                      = "enable_connection"
+	FlagConnectionEnableWithAlias             = FlagConnectionEnable + ", ec"
 
 	FlagProtoType  = "type"
 	FlagHexData    = "hex_data"
@@ -328,13 +332,17 @@ func getFlagsForStart() []cli.Flag {
 			Usage: "WorkflowTypeName",
 		},
 		cli.IntFlag{
-			Name:  FlagExecutionTimeoutWithAlias,
-			Usage: "Execution start to close timeout in seconds",
+			Name:  FlagWorkflowExecutionTimeoutWithAlias,
+			Usage: "Workflow execution timeout, including retries and continue-as-new (seconds)",
+		},
+		cli.IntFlag{
+			Name:  FlagWorkflowRunTimeoutWithAlias,
+			Usage: "Single workflow run timeout (seconds)",
 		},
 		cli.IntFlag{
 			Name:  FlagWorkflowTaskTimeoutWithAlias,
 			Value: defaultWorkflowTaskTimeoutInSeconds,
-			Usage: "Workflow task start to close timeout in seconds",
+			Usage: "Workflow task start to close timeout (seconds)",
 		},
 		cli.StringFlag{
 			Name: FlagCronSchedule,
@@ -362,27 +370,27 @@ func getFlagsForStart() []cli.Flag {
 			Usage: "Optional input for the workflow from JSON file. If there are multiple JSON, concatenate them and separate by space or newline. " +
 				"Input from file will be overwrite by input from command line",
 		},
-		cli.StringFlag{
+		cli.StringSliceFlag{
 			Name:  FlagMemoKey,
-			Usage: "Optional key of memo. If there are multiple keys, concatenate them and separate by space",
+			Usage: fmt.Sprintf("Optional key of memo. If there are multiple keys, provide multiple %s flags", FlagMemoKey),
 		},
-		cli.StringFlag{
+		cli.StringSliceFlag{
 			Name: FlagMemo,
-			Usage: "Optional info that can be showed when list workflow, in JSON format. If there are multiple JSON, concatenate them and separate by space. " +
-				"The order must be same as memo_key",
+			Usage: fmt.Sprintf("Optional info that can be showed when list workflow. If there are multiple values, provide multiple %s flags. "+
+				"The order must be same as %s", FlagMemo, FlagMemoKey),
 		},
 		cli.StringFlag{
 			Name: FlagMemoFile,
-			Usage: "Optional info that can be listed in list workflow, from JSON format file. If there are multiple JSON, concatenate them and separate by space or newline. " +
-				"The order must be same as memo_key",
+			Usage: fmt.Sprintf("File name of optional info that can be showed when list workflow. If there are multiple values, separate them by newline. "+
+				"The order of lines must be same as %s", FlagMemoKey),
 		},
 		cli.StringFlag{
-			Name: FlagSearchAttributesKey,
+			Name: FlagSearchAttributeKey,
 			Usage: "Optional search attributes keys that can be be used in list query. If there are multiple keys, concatenate them and separate by |. " +
 				"Use 'cluster get-search-attr' cmd to list legal keys.",
 		},
 		cli.StringFlag{
-			Name: FlagSearchAttributesVal,
+			Name: FlagSearchAttributeValue,
 			Usage: "Optional search attributes value that can be be used in list query. If there are multiple keys, concatenate them and separate by |. " +
 				"If value is array, use json array like [\"a\",\"b\"], [1,2], [\"true\",\"false\"], [\"2019-06-07T17:16:34-08:00\",\"2019-06-07T18:16:34-08:00\"]. " +
 				"Use 'cluster get-search-attr' cmd to list legal keys and value types",
@@ -519,7 +527,7 @@ func getFlagsForCount() []cli.Flag {
 	return []cli.Flag{
 		cli.StringFlag{
 			Name:  FlagListQueryWithAlias,
-			Usage: "Optional SQL like query. e.g count all open workflows 'CloseTime = missing'; 'WorkflowType=\"wtype\" and CloseTime > 0'",
+			Usage: "Optional SQL like query. e.g count all open workflows \"ExecutionStatus='Running'\"; 'WorkflowType=\"wtype\" and CloseTime > 0'",
 		},
 	}
 }
@@ -659,10 +667,20 @@ func getDBFlags() []cli.Flag {
 func getESFlags(index bool) []cli.Flag {
 	flags := []cli.Flag{
 		cli.StringFlag{
-			Name:  FlagURL,
+			Name:  FlagElasticsearchURL,
 			Value: "http://127.0.0.1:9200",
 			Usage: "URL of Elasticsearch cluster",
 		},
+		cli.StringFlag{
+			Name:  FlagElasticsearchUsername,
+			Value: "",
+			Usage: "Username for Elasticsearch cluster",
+		},
+		cli.StringFlag{
+			Name:  FlagElasticsearchPassword,
+			Value: "",
+			Usage: "Password for Elasticsearch cluster",
+		},
 		cli.StringFlag{
 			Name:  FlagVersion,
 			Value: "v7",
@@ -672,7 +690,7 @@ func getESFlags(index bool) []cli.Flag {
 	if index {
 		flags = append(flags,
 			cli.StringFlag{
-				Name:  FlagIndex,
+				Name:  FlagElasticsearchIndex,
 				Usage: "Elasticsearch index name",
 			},
 		)
