@@ -26,12 +26,9 @@ package cli
 
 import (
 	"bufio"
-	"bytes"
 	"context"
 	"encoding/json"
 	"fmt"
-	"io"
-	"io/ioutil"
 	"os"
 	"reflect"
 	"regexp"
@@ -615,7 +612,7 @@ func newContextWithTimeout(c *cli.Context, timeout time.Duration) (context.Conte
 
 // process and validate input provided through cmd or file
 func processJSONInput(c *cli.Context) *commonpb.Payloads {
-	jsonsRaw := readJSONInputs(c, jsonTypeInput)
+	jsonsRaw := readJSONInputs(c)
 
 	var jsons []interface{}
 	for _, jsonRaw := range jsonsRaw {
@@ -639,23 +636,9 @@ func processJSONInput(c *cli.Context) *commonpb.Payloads {
 }
 
 // read multiple inputs presented in json format
-func readJSONInputs(c *cli.Context, jType jsonType) [][]byte {
-	var flagRawInput string
-	var flagInputFileName string
-
-	switch jType {
-	case jsonTypeInput:
-		flagRawInput = FlagInput
-		flagInputFileName = FlagInputFile
-	case jsonTypeMemo:
-		flagRawInput = FlagMemo
-		flagInputFileName = FlagMemoFile
-	default:
-		return nil
-	}
-
-	if c.IsSet(flagRawInput) {
-		inputsG := c.Generic(flagRawInput)
+func readJSONInputs(c *cli.Context) [][]byte {
+	if c.IsSet(FlagInput) {
+		inputsG := c.Generic(FlagInput)
 
 		var inputs *cli.StringSlice
 		var ok bool
@@ -676,11 +659,11 @@ func readJSONInputs(c *cli.Context, jType jsonType) [][]byte {
 		}
 
 		return inputsRaw
-	} else if c.IsSet(flagInputFileName) {
-		inputFile := c.String(flagInputFileName)
+	} else if c.IsSet(FlagInputFile) {
+		inputFile := c.String(FlagInputFile)
 		// This method is purely used to parse input from the CLI. The input comes from a trusted user
 		// #nosec
-		data, err := ioutil.ReadFile(inputFile)
+		data, err := os.ReadFile(inputFile)
 		if err != nil {
 			ErrorAndExit("Error reading input file", err)
 		}
@@ -689,21 +672,6 @@ func readJSONInputs(c *cli.Context, jType jsonType) [][]byte {
 	return nil
 }
 
-// validate whether str is a valid json or multi valid json concatenated with spaces/newlines
-func validateJSONs(str string) error {
-	input := []byte(str)
-	dec := json.NewDecoder(bytes.NewReader(input))
-	for {
-		_, err := dec.Token()
-		if err == io.EOF {
-			return nil // End of input, valid JSON
-		}
-		if err != nil {
-			return err // Invalid input
-		}
-	}
-}
-
 func truncate(str string) string {
 	if len(str) > maxOutputStringLength {
 		return str[:maxOutputStringLength]
