@@ -31,11 +31,9 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"io/ioutil"
 	"math/rand"
 	"os"
 	"reflect"
-	"regexp"
 	"strings"
 	"sync"
 	"time"
@@ -43,13 +41,11 @@ import (
 	"github.com/olekukonko/tablewriter"
 	"github.com/pborman/uuid"
 	"github.com/urfave/cli"
-	"github.com/valyala/fastjson"
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	failurepb "go.temporal.io/api/failure/v1"
 	filterpb "go.temporal.io/api/filter/v1"
 	historypb "go.temporal.io/api/history/v1"
-	namespacepb "go.temporal.io/api/namespace/v1"
 	querypb "go.temporal.io/api/query/v1"
 	"go.temporal.io/api/serviceerror"
 	workflowpb "go.temporal.io/api/workflow/v1"
@@ -169,7 +165,7 @@ func showHistoryHelper(c *cli.Context, wid, rid string) {
 		if err != nil {
 			ErrorAndExit("Failed to serialize history data.", err)
 		}
-		if err := ioutil.WriteFile(outputFileName, data, 0666); err != nil {
+		if err := os.WriteFile(outputFileName, data, 0666); err != nil {
 			ErrorAndExit("Failed to export history data file.", err)
 		}
 	}
@@ -191,7 +187,8 @@ func startWorkflowHelper(c *cli.Context, shouldPrintProgress bool) {
 	namespace := getRequiredGlobalOption(c, FlagNamespace)
 	taskQueue := getRequiredOption(c, FlagTaskQueue)
 	workflowType := getRequiredOption(c, FlagWorkflowType)
-	et := c.Int(FlagExecutionTimeout)
+	et := c.Int(FlagWorkflowExecutionTimeout)
+	rt := c.Int(FlagWorkflowRunTimeout)
 	dt := c.Int(FlagWorkflowTaskTimeout)
 	wid := c.String(FlagWorkflowID)
 	if len(wid) == 0 {
@@ -212,6 +209,7 @@ func startWorkflowHelper(c *cli.Context, shouldPrintProgress bool) {
 		TaskQueue:                taskQueue,
 		WorkflowExecutionTimeout: time.Duration(et) * time.Second,
 		WorkflowTaskTimeout:      time.Duration(dt) * time.Second,
+		WorkflowRunTimeout:       time.Duration(rt) * time.Second,
 		WorkflowIDReusePolicy:    reusePolicy,
 	}
 	if c.IsSet(FlagCronSchedule) {
@@ -269,7 +267,7 @@ func startWorkflowHelper(c *cli.Context, shouldPrintProgress bool) {
 }
 
 func unmarshalInputsFromCLI(c *cli.Context) []interface{} {
-	jsonsRaw := readJSONInputs(c, jsonTypeInput)
+	jsonsRaw := readJSONInputs(c)
 
 	var result []interface{}
 	for _, jsonRaw := range jsonsRaw {
@@ -311,11 +309,11 @@ func unmarshalSearchAttrFromCLI(c *cli.Context) map[string]interface{} {
 		return result
 	}
 
-	searchAttrKeys := sanitize(c.String(FlagSearchAttributesKey))
+	searchAttrKeys := sanitize(c.String(FlagSearchAttributeKey))
 	if len(searchAttrKeys) == 0 {
 		return nil
 	}
-	rawSearchAttrVals := sanitize(c.String(FlagSearchAttributesVal))
+	rawSearchAttrVals := sanitize(c.String(FlagSearchAttributeValue))
 	if len(rawSearchAttrVals) == 0 {
 		return nil
 	}
@@ -338,34 +336,41 @@ func unmarshalSearchAttrFromCLI(c *cli.Context) map[string]interface{} {
 }
 
 func unmarshalMemoFromCLI(c *cli.Context) map[string]interface{} {
-	rawMemoKey := c.String(FlagMemoKey)
-	var memoKeys []string
-	if strings.TrimSpace(rawMemoKey) != "" {
-		memoKeys = strings.Split(rawMemoKey, " ")
-	}
-
-	jsonsRaw := readJSONInputs(c, jsonTypeMemo)
-	if len(jsonsRaw) == 0 {
+	// Memo flags were not passed => Memo is not provided.
+	if !c.IsSet(FlagMemoKey) && !c.IsSet(FlagMemo) && !c.IsSet(FlagMemoFile) {
 		return nil
 	}
-	rawMemoValue := string(jsonsRaw[0]) // StringFlag may contain up to one json
 
-	if err := validateJSONs(rawMemoValue); err != nil {
-		ErrorAndExit("Input is not valid JSON, or JSONs concatenated with spaces/newlines.", err)
+	if !c.IsSet(FlagMemoKey) {
+		ErrorAndExit(fmt.Sprintf("Memo keys must be provided using %s.", FlagMemoKey), nil)
 	}
 
-	var memoValues []interface{}
+	if c.IsSet(FlagMemo) && c.IsSet(FlagMemoFile) {
+		ErrorAndExit(fmt.Sprintf("Only one of %s or %s should be used.", FlagMemo, FlagMemoFile), nil)
+	}
 
-	var sc fastjson.Scanner
-	sc.Init(rawMemoValue)
-	for sc.Next() {
-		memoValues = append(memoValues, sc.Value())
+	if !c.IsSet(FlagMemo) && !c.IsSet(FlagMemoFile) {
+		ErrorAndExit(fmt.Sprintf("Memo values must be provided using %s or %s.", FlagMemo, FlagMemoFile), nil)
 	}
-	if err := sc.Error(); err != nil {
-		ErrorAndExit("Memo JSON parse error.", err)
+
+	memoKeys := c.StringSlice(FlagMemoKey)
+
+	var memoValues []string
+	if c.IsSet(FlagMemoFile) {
+		inputFile := c.String(FlagMemoFile)
+		// This method is purely used to parse input from the CLI. The input comes from a trusted user
+		// #nosec
+		data, err := os.ReadFile(inputFile)
+		if err != nil {
+			ErrorAndExit(fmt.Sprintf("Error reading memo file %s.", inputFile), err)
+		}
+		memoValues = strings.Split(string(data), "\n")
+	} else if c.IsSet(FlagMemo) {
+		memoValues = c.StringSlice(FlagMemo)
 	}
+
 	if len(memoKeys) != len(memoValues) {
-		ErrorAndExit("Number of memo keys and values are not equal.", nil)
+		ErrorAndExit(fmt.Sprintf("Number of memo keys %d and values %d are not equal.", len(memoKeys), len(memoValues)), nil)
 	}
 
 	fields := make(map[string]interface{}, len(memoKeys))
@@ -381,17 +386,16 @@ func getPrintableMemo(memo *commonpb.Memo) string {
 		var memo string
 		err := payload.Decode(v, &memo)
 		if err != nil {
-			ErrorAndExit("Memo has incorrect format.", err)
+			memo = "Memo is not a string"
 		}
-
 		_, _ = fmt.Fprintf(buf, "%s=%s\n", k, memo)
 	}
 	return buf.String()
 }
 
-func getPrintableSearchAttr(searchAttr *commonpb.SearchAttributes) string {
+func getPrintableSearchAttributes(searchAttributes *commonpb.SearchAttributes) string {
 	var buf bytes.Buffer
-	searchAttributesString, err := searchattribute.Stringify(searchAttr, nil)
+	searchAttributesString, err := searchattribute.Stringify(searchAttributes, nil)
 	if err != nil {
 		fmt.Printf("%s: unable to stringify search attribute: %v\n",
 			colorMagenta("Warning"),
@@ -692,9 +696,8 @@ func ScanAllWorkflow(c *cli.Context) {
 		return
 	}
 
-	isQueryOpen := isQueryOpen(c.String(FlagListQuery))
-	table := createTableForListWorkflow(c, true, isQueryOpen)
-	prepareTable := scanWorkflow(c, table, isQueryOpen)
+	table := createTableForListWorkflow(c, true, true)
+	prepareTable := scanWorkflow(c, table, true)
 	var nextPageToken []byte
 	for {
 		nextPageToken, _ = prepareTable(nextPageToken)
@@ -705,11 +708,6 @@ func ScanAllWorkflow(c *cli.Context) {
 	table.Render()
 }
 
-func isQueryOpen(query string) bool {
-	var openWFPattern = regexp.MustCompile(`CloseTime[ ]*=[ ]*missing`)
-	return openWFPattern.MatchString(query)
-}
-
 // CountWorkflow count number of workflows
 func CountWorkflow(c *cli.Context) {
 	sdkClient := getSDKClient(c)
@@ -1114,7 +1112,7 @@ func appendWorkflowExecutionsToTable(
 			row = append(row, getPrintableMemo(e.Memo))
 		}
 		if printSearchAttr {
-			row = append(row, getPrintableSearchAttr(e.SearchAttributes))
+			row = append(row, getPrintableSearchAttributes(e.SearchAttributes))
 		}
 		table.Append(row)
 	}
@@ -1367,7 +1365,7 @@ func scanWorkflow(c *cli.Context, table *tablewriter.Table, queryOpen bool) func
 				row = append(row, getPrintableMemo(e.Memo))
 			}
 			if printSearchAttr {
-				row = append(row, getPrintableSearchAttr(e.SearchAttributes))
+				row = append(row, getPrintableSearchAttributes(e.SearchAttributes))
 			}
 			table.Append(row)
 		}
@@ -1838,6 +1836,15 @@ func getLastWorkflowTaskEventID(ctx context.Context, namespace, wid, rid string,
 	return
 }
 
+func badChecksum(bad string) func(string) error {
+	return func(maybeBad string) error {
+		if maybeBad == bad {
+			return fmt.Errorf("Bad checksum %q", bad)
+		}
+		return nil
+	}
+}
+
 func getBadWorkflowTaskCompletedID(ctx context.Context, namespace, wid, rid, binChecksum string, frontendClient workflowservice.WorkflowServiceClient) (resetBaseRunID string, workflowTaskCompletedID int64, err error) {
 	resetBaseRunID = rid
 	resp, err := frontendClient.DescribeWorkflowExecution(ctx, &workflowservice.DescribeWorkflowExecutionRequest{
@@ -1851,11 +1858,7 @@ func getBadWorkflowTaskCompletedID(ctx context.Context, namespace, wid, rid, bin
 		return "", 0, printErrorAndReturn("DescribeWorkflowExecution failed", err)
 	}
 
-	_, p := workflow.FindAutoResetPoint(clock.NewRealTimeSource(), &namespacepb.BadBinaries{
-		Binaries: map[string]*namespacepb.BadBinaryInfo{
-			binChecksum: {},
-		},
-	}, resp.WorkflowExecutionInfo.AutoResetPoints)
+	_, p := workflow.FindAutoResetPoint(clock.NewRealTimeSource(), badChecksum(binChecksum), resp.WorkflowExecutionInfo.AutoResetPoints)
 	if p != nil {
 		workflowTaskCompletedID = p.GetFirstWorkflowTaskCompletedId()
 	}
